/* blacklist.c
 *
 * part of ed2k-gtk-gui
 *
 * (c) 2002 Tim-Philipp Muller <t.i.m@orange.net>
 *
 * the blacklist contains hashes of files we don't want to see when we search
 * it is not saved anywhere, but the gui reads the gui_blacklist file at startup
 * and then creates its blacklist
 *
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* last general code review: 2002-06-20 by Tim */

#include "global.h"
#include "misc.h"
#include "gui_options.h"
#include "status_page.h"
#include "blacklist.h"
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>


/* local defines */

#define		no_of_blacklist_items()		g_slist_length(blacklist)
#define		get_blacklist_filename()	complete_pref_path("gui_blacklist",GUI_CONFIG_IGNORE_INSTANCE)


/* local variables */

static GSList	*blacklist = NULL;
static time_t	last_blacklistfile_modtime = 0;


/* local functions */

static void	append_to_blacklist_file (const guint8 *evilhash, guint32 size, const gchar *name);
static void	reread_blacklist (void);


/* is_item_in_blacklist
 *
 * Checks if a filelist item is blacklisted.
 * Returns TRUE or FALSE
 *
 */

gboolean
is_item_in_blacklist (const guint8 *hash, guint size)
{
	GSList *next = blacklist;

	g_return_val_if_fail (hash!=NULL,FALSE);
	while (next)
	{
		if (next->data)
		{
			blacklist_item	*checkitem = (blacklist_item *)next->data;
			if (memcmp(checkitem->hash, hash, 16)==0 && checkitem->filesize==size)
			{
				return TRUE;	/* yup, is in list */
			}
		}
		next = next->next;
	}
	return FALSE; /* nope, is not in list */
}


/* append_to_blacklist_file
 *
 * Appends an item to the gui_blacklist file (name is printed for 'readability' reasons,
 *	but doesn't matter. We only check for hash and file size when checking if
 *	items are blacklisted. This function does not check if file is already in list.
 *
 */

static void
append_to_blacklist_file (const guint8 *evilhash, guint32 size, const gchar *name)
{
	FILE *lf;

	g_return_if_fail (evilhash!=NULL);
	g_return_if_fail (name!=NULL);

	if ((lf=fopen(get_blacklist_filename(), N_("a")))!=NULL)
	{
		fprintf (lf,N_("ed2k://|file|%s|%u|%s|\n"), name, size, hash_to_hash_str (evilhash));
		fclose(lf);
	} else status_system_error_msg (_("couldn't write to blacklist file!"));
	last_blacklistfile_modtime = time(NULL);
}


/* reread_blacklist
 *
 * Rereads the list of items in the blacklist from the gui_blacklist file
 *
 */

static void
reread_blacklist (void)
{
	gchar	**line, **linearray;
	guint	commented_out=0, invalid=0;

	status_msg (N_("%s"), _("GUI: blacklist file has changed - rereading it.\n"));
	linearray = misc_get_array_of_lines_from_textfile (get_blacklist_filename());
	if (linearray)
	{
		if ((*linearray) && (**linearray!=0x00))	/* file not empty */
		{
			clear_blacklist();
			line = linearray;
			while (*line)
			{
				if (**line!='#')
				{
					gchar *thisline = g_strchug(*line);	/* remove leading whitespaces */
					if (g_strncasecmp(thisline,N_("ed2k://|file|"),13)==0)
					{
						guint		 size=0;
						gchar		*hash = NULL;
						gchar		*filenamebuf = NULL;

						if (misc_check_if_ed2klink_is_valid(thisline,&filenamebuf,&size,&hash))
						{
							guint8 *hexhash = (guint8*) hash_str_to_hash(hash);
							if (hexhash)
							{
								/* "" is a marker, so the add_to_.. doesn't write to file */
								add_to_blacklist (hexhash, size, N_(""));
							} else invalid++;
						} else invalid++;
						G_FREE(filenamebuf);
						G_FREE(hash);
					} else {
						if (*thisline!=0x00)
						{
							status_parse_error_msg (N_("blacklist"), (linearray-line)/sizeof(gchar*), thisline);
							invalid++;
						}
					}
				} else commented_out++;
				line++;
			}
		}
		g_strfreev(linearray);	/* free line array */
	} else status_system_error_msg (_("couldn't read blacklist file."));
	last_blacklistfile_modtime = time(NULL);

	/* was: if (window) ... else write_to_logfile(msg); */
	status_msg (_("GUI: read in %u valid blacklist items (another %u were commented out - %u were invalid)\n"),
				no_of_blacklist_items(), commented_out, invalid);
}


/* check_if_blacklistfile_changed
 *
 * Checks if the blacklist file has changed (ie. has a newer date returned by fstat() than
 *	than that that we have saved) and re-reads the blacklist file if it has changed
 */

void
check_if_blacklistfile_changed (void)
{
	struct stat blstat;
	if (stat (get_blacklist_filename(), &blstat)==0)	/* if not, error or file doesn't exist */
	{
		if (blstat.st_mtime > last_blacklistfile_modtime)
		{
			reread_blacklist();
		}
	}
}


/* add_to_blacklist
 *
 * Adds an item to the blacklist if it is not already in there.
 *
 */

void
add_to_blacklist (const guint8 *evilhash, guint32 size, const gchar *name)
{
	blacklist_item	*bli;

	g_return_if_fail (evilhash!=NULL);
	/* we don't check for name!=NULL, because that's an indicator that
	 *	we've just read the item in from disk
	 */

	if (!is_item_in_blacklist(evilhash,size))
	{
		bli = g_new (blacklist_item, 1);
		g_return_if_fail (bli!=NULL);
		g_memmove (bli->hash, evilhash, 16);
		bli->filesize = size;
		blacklist = g_slist_append (blacklist, (gpointer) bli );
/*		g_print ("BLACKLIST APPEND: %s - %10u - %s\n", hash_to_hash_str(evilhash), size, name); */
		/* don't append items again that we've just read in */
		if ((!name) || *name==0x00) return;
		append_to_blacklist_file (evilhash, size, name);
	}
}



/* clear_blacklist
 *
 * Frees the blacklist and all data it contains
 *
 */

void
clear_blacklist (void)
{
	GSList *next = blacklist;
	while (next)
	{
		if (next->data) g_free (next->data);	/* free the blacklist_item */
		next = next->next;
	}
	g_slist_free (blacklist);
	blacklist = NULL;
}

