/*
*   connect_gui.c     (part of the eDonkey2000 linux GUI)
*
*   Functions to
*    - show/destroy 'GUI connect' dialogue
*    - cleanup GUI variables after disconnect
*    - init GUI variables after connect
*
*   (c) 2001-2002 Tim-Philipp Mueller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* last code review: 2002-11-01 by Tim (the code in here is still a bit messy) */

#define GNET_EXPERIMENTAL
#include <gnet/gnet.h>

#include "global.h"
#include "mainwindow.h"
#include "misc.h"
#include "misc_gtk.h"
#include "connect_gui.h"
#include "gui_options.h"
#include "downloads_data.h"
#include "downloads_page.h"
#include "options_page.h"
#include "servers_page.h"
#include "search_data.h"
#include "statusbar.h"
#include "status_page.h"
#include "shared_dirs.h"
#include "shared_files.h"
#include "shared_page.h"
#include "spawn_core.h"
#include "servers_data.h"
#include "toolbar.h"
#include "uploads_data.h"
#include "uploads_page.h"
#include "dns_lookup.h"
#include "gui_flags.h"

#include "guicoreconn.h"
#include "send_command.h"
#include "parse_message.h"
#include "packet.h"

#include "icons.h"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gtk/gtk.h>


#define QUICK_START_TEXT	\
		_(" * You have the GUI. Good. Now get the eDonkey2000 core from http://ed2k-gtk-gui.sf.net/core.shtml  \n"\
		  " * Download the archive into a subdirectory of your home directory (e.g. /home/joe/ed2k/). \n"\
		  " * Unpack it in there with 'tar xzf donkey_bla.tar.gz'. \n"\
		  " * Put the full path to the binary, e.g. '/home/joe/ed2k/donkey_59-gaps-4' into the 'path' field \n"\
		  " * Choose an arbitrary username and password. Avoid special characters. \n"\
		  " * The hostname should be 'localhost' and the port should be 4663. 'ping localhost' should work. \n"\
		  " * Hit the 'spawn' button. The dialog should display 'running' now. Hit the 'Go!' button. \n"\
		  " * You should now be connected to the core properly. Make sure that on the 'Shared' \n"\
		  "     page there is at least one directory listed. If that is so, you are ready to go. \n"\
		  " * Right-click on the server list and fetch an up-to-date list of servers and connect to one. \n\n"\
			\
		  " If this does not work, please look at the status page and consult the FAQ/documentation.")


/* global variables */

gchar DUMMY_NICKNAME[] = N_("please wait");

static GtkWidget	*connect_dialog = NULL;
static GtkWidget	*ew_connect_user = NULL;
static GtkWidget	*ew_connect_pass = NULL;
static GtkWidget	*cb_connect_makedef = NULL;
static GtkWidget	*ew_connect_host = NULL;
static GtkWidget	*ew_connect_path = NULL;
static GtkWidget	*spinbutton_connect_port = NULL;
static GtkWidget 	*spawn_button = NULL;
static GtkWidget	*ok_button = NULL;
static GtkWidget	*frame_quick_help = NULL;

// Timer that is running while the 'connect to..'
//   dialog is up to show periodically if donkey
//   is running currently or not (on local machine)

static gint 		 connect_timer_handle = 0;


/* functions */
static gint	gui_connect_dialog_exit_button_clicked (GtkWidget *widget, gpointer data);
static gint	gui_connect_dialog_connect_button_clicked (GtkWidget *widget, gpointer connectdialogwidget);
static gint	gui_connect_dialog_help_button_clicked (GtkWidget *widget, gpointer data);
static gint	gui_connect_dialog_quickhelp_button_clicked (GtkWidget *widget, gpointer data);
static gint	gui_connect_find_binary (void);
static gint	gui_connect_dialog_spawn_button_clicked (GtkWidget *widget, gpointer data);


/* clean_up_stuff
 *
 *
 */

void
clean_up_stuff (void)
{
	// gnet_conn_disconnect (guicoreconn); ??? - nope, done in guicoreconn_close_and_cleanup()

	show_restart_success_message_on_next_packet = 0;

	// free memory
	searchlist_clean_up_all();
	downloads_cleanup();
	downloads_completed_cleanup();
	uploads_cleanup();
	servers_cleanup();
	shareddirs_cleanup ();
}


/* cleanup_after_disconnect
 *
 */

void
cleanup_after_disconnect (void)
{
	clean_up_stuff();

	if (window)					// only if main window still exists
	{
		// Note: all search clists are cleared in searchlist_cleanup_all()
		//	which is called before us
		uploadsclist_clear();
		shared_files_clist_clear();
		// Note: shared dirs list is cleared via shareddirs_cleanup()
		//	which is called before us
	}

	if (about_to_quit)
		return;
	
	if (guicoreconn_do_reconnect)
	{
		guicoreconn_reconnect();
	} else gui_connect_dialog();	// connect to dialog if auto-connect failed
	return;
}


/* gui_connect_dialog_exit_button_clicked
 *
 */

static gint
gui_connect_dialog_exit_button_clicked (GtkWidget *widget, gpointer data)
{
	option_connect_make_def_values = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_connect_makedef));
	gtk_grab_remove (GTK_WIDGET(data));
	gtk_timeout_remove (connect_timer_handle);
	mainwindow_delete_event_really_quit (widget, data);         /* use main window quit function... */
	return 0;
}


/* gui_connect_dialog_help_button_clicked
 *
 */

static gint
gui_connect_dialog_help_button_clicked (GtkWidget *widget, gpointer data)
{
	invoke_browser_with_url (DOCS_URL);
	return 0;
}


/* gui_connect_dialog_quickhelp_button_clicked
 *
 */

static gint
gui_connect_dialog_quickhelp_button_clicked (GtkWidget *widget, gpointer data)
{
	if (GTK_WIDGET_VISIBLE(frame_quick_help))
	{
		gtk_widget_hide (frame_quick_help);
	} else gtk_widget_show (frame_quick_help);
	return 0;
}



/* gui_connect_find_binary
 *
 */

static gint
gui_connect_find_binary (void)
{
	gchar *path;

	path = find_donkey_binary();
	gtk_entry_set_text(GTK_ENTRY(ew_connect_path), (path!=NULL) ? path : _("couldn't find local binary"));
	return (gint)(path!=NULL);	
}


/* gui_connect_dialog_spawn_button_clicked
 *
 */

static gint
gui_connect_dialog_spawn_button_clicked (GtkWidget *widget, gpointer data)
{
	gchar		*pathentered = NULL;
	gchar		*dir = NULL;
	const gchar	*user = NULL;
	const gchar *pass = NULL;
	gint		 i_port = 0;
	gint		 clientport = 4662;

	g_return_val_if_fail (ew_connect_path!=NULL,0);
	g_return_val_if_fail (ew_connect_user!=NULL,0);
	g_return_val_if_fail (ew_connect_pass!=NULL,0);
	g_return_val_if_fail (spinbutton_connect_port!=NULL,0);

	/* ... we should never have been called then. */
	g_return_val_if_fail ( GTK_WIDGET_IS_SENSITIVE(ew_connect_path), 0);

	pathentered = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_path), 0, -1);
	user = gtk_entry_get_text (GTK_ENTRY(ew_connect_user));
	pass = gtk_entry_get_text (GTK_ENTRY(ew_connect_pass));
	i_port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_connect_port));

	/* these cases shouldn't happen - the spawn button should
	 * be blocked until the path is fine
     */
	g_return_val_if_fail ( pathentered!=NULL, 0);
	g_return_val_if_fail ( *pathentered!=0x00, 0);
	g_return_val_if_fail ( access(pathentered,F_OK|R_OK|X_OK)==0, 0);

	/* check pref.met */
	dir = g_dirname(pathentered);
	g_return_val_if_fail (dir!=NULL,0);

//	spawn_core_listening_port_is_in_use(9999);

	/* do entered username/password + port not match with what's in the pref.met file */
	if (1 != spawn_core_pref_met_has_username(dir,user,pass,i_port,&clientport))
	{
		gchar		*pmfn = g_strdup_printf(N_("%s/pref.met"),dir);
		gint		 ret;
		guint		 new_clientport, new_aport;
		gchar		*msg_cp = NULL;
		gchar		*msg_ap = NULL;

		
		g_return_val_if_fail (pmfn!=NULL,0);

		if (access(pmfn,F_OK)==0)
		{
			status_message_blue (_("GUI: '%s' does not exist yet (will create it).\n"),pmfn);
		} else status_message_blue (_("GUI: '%s' exists but with different name/pass/port (will change it).\n"),pmfn);

		/* is some program already using the client port that the donkey is going to use? */
		if (spawn_core_listening_port_is_in_use (clientport))
		{
			new_clientport = clientport+10;
			while (spawn_core_listening_port_is_in_use (new_clientport))
			{
				new_clientport += 10;
			} 
			msg_cp = g_strdup_printf(_("\nSome other program is already using the\n"
										"port that the donkey wanted to listen on\n"
										"for connections from other clients (%u).\n"
										"I have changed this port to TCP port %u\n"
										"now. If you have port-forwarding configured\n"
										"somewhere, you will probably want to update that.\n\n"),
										clientport, new_clientport);
		} else new_clientport = clientport;

		/* is some program already using the GUI admin port that the donkey is going to use? */
		if (spawn_core_listening_port_is_in_use (i_port))
		{
			new_aport = i_port+10;
			while (spawn_core_listening_port_is_in_use (new_aport))
			{
				new_aport += 10;
			}
			msg_ap = g_strdup_printf(_("Some other program is already using the\n"
										"port that the donkey wanted to listen on\n"
										"for GUI connections (aport) (%u).\n"
										"I have changed this port to TCP port %u\n"
										"now. This is just for your information.\n\n"),
										i_port, new_aport);
			gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_connect_port), new_aport*1.0);
		} else new_aport = i_port;

		ret = spawn_core_modify_pref_met (dir,user,pass,new_aport,new_clientport);
		if (!ret)
		{
			/* failure */
			status_system_error_msg (_("Failed to change or create %s/pref.met.\n"));
			ok_dialog
			(
				_("spawning eDonkey2000 core"),
				_("I noticed that the eDonkey2000 core's configuration\n"
				  "file that contains the admin username and password\n"
				  "either does not exist, or contains a different\n"
				  "username/password/port combination than the one you\n"
				  "specified in the 'connect to dialog'.\n\n"
				  "Therefore I tried to change the pref.met file so that\n"
				  "it contains the username/password/port that you have\n"
				  "just entered here, but I couldn't (reason: see statusbar).\n\n"
				  "This means that you will have to set up the admin username\n"
				  "and password for the GUI manually in the core, using the 'pass'\n"
				  "command. Make sure you change into the directory where the core\n"
				  "binary is before you do that (with 'cd /home/joe/ed2k/').\n\n"
				  "Please consult the FAQ/documentation for details.\n\n"
				  "Sorry it didn't work out.\n\n"
				  "Note: I will try to spawn a core anyway now.\n")
			);
		} else {
			/* success */
			status_message_blue (_("GUI: %s/pref.met changed or created successfully.\n"),dir);
			if ((msg_ap)||(msg_cp))
			{
				gchar *msg_p = g_strdup_printf (N_("%s%s"),
									(msg_cp!=NULL) ? msg_cp : N_(""),
									(msg_ap!=NULL) ? msg_ap : N_(""));
				if (msg_p)
				{
					ok_dialog (_("ports changed"), msg_p);
					g_free(msg_p);
				}
			}
		}

		if (msg_ap) g_free(msg_ap);
		if (msg_cp) g_free(msg_cp);

		g_free(pmfn);

	} else status_message_blue (_("Good news - specified admin username, password and aport match the one in the pref.met file.\n"));

	g_free(dir);

	if (pathentered)
		g_free (pathentered);

	/* update the path (possibly we found the binary in the meantime) */
	pathentered = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_path), 0, -1);

	(void) spawn_core_and_print_spawn_result_message(pathentered);

	if (pathentered)
		g_free (pathentered);

	return 0;
}


/* gui_connect_dialog_connect_button_clicked
 *
 */

static gint
gui_connect_dialog_connect_button_clicked (GtkWidget *widget, gpointer connectdialogwidget)
{
	gchar	*host = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_host), 0, -1);
	gchar	*user = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_user), 0, -1);
	gchar	*pass = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_pass), 0, -1);
	gchar	*path = NULL;
//	gchar	*dummy;
	gint	 i_port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton_connect_port));

	option_connect_make_def_values = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_connect_makedef));

	g_return_val_if_fail (host!=NULL && *host!=0x00,0);
	g_return_val_if_fail (user!=NULL && *user!=0x00,0);
	g_return_val_if_fail (pass!=NULL && *pass!=0x00,0);

	/* only check pref.met if it's a local core */
	if (GTK_WIDGET_IS_SENSITIVE(ew_connect_path))
	{
		gchar *dir;

		path = gtk_editable_get_chars (GTK_EDITABLE(ew_connect_path), 0, -1);
		g_return_val_if_fail (path!=NULL,0);
		dir = g_dirname(path);

		/* do username/password + port not match with what's in the pref.met file */
		if (1 != spawn_core_pref_met_has_username(dir,user,pass,i_port,NULL))
		{
			gchar *pmfn = g_strdup_printf(N_("%s/pref.met"),dir);
			g_return_val_if_fail (pmfn!=NULL,0);

			if ( access(pmfn,F_OK) != 0 )
			{
				status_message_blue (_("GUI: %s/pref.met does not exist yet in the core's directory.\n"),dir);
			} else status_message_blue (_("GUI: %s/pref.met exists but with different name/pass/port!\n"),dir);
			g_free(pmfn);
		} else status_message_blue (_("Good news - specified admin username, password and aport match the one in the pref.met file.\n"));
		g_free(dir);
	}

	/* make the entered values the default values? */
	if (option_connect_make_def_values==1)
	{
			
		strrealloccpy (&option_defaulthost, host);
		strrealloccpy (&option_defaultuser, user);
		strrealloccpy (&option_defaultpass, pass);
		if (path)
		{
			strrealloccpy (&option_defaultdonkeypath, path);
		}
		option_defaultport = i_port;
	}

	/* remove 'core running' timer for dialog label */
    gtk_timeout_remove (connect_timer_handle);

	/* Initiate connection to core */
	if (1==1)
	{
//		gboolean show_connect_dialog = TRUE;

		ew_connect_user = ew_connect_pass = NULL;
		ew_connect_host = ew_connect_path = NULL;
		connect_dialog = NULL;
		cb_connect_makedef = NULL;
		spinbutton_connect_port = NULL;
		spawn_button = NULL;

		if (!guicoreconn_set_connection (host, i_port, user, pass, connectdialogwidget))
		{
			g_printerr (_("Connect to core failed - couldn't initiate valid connection structure with default values.\n"));
			connectdialogwidget = NULL;
			gtk_grab_remove (GTK_WIDGET(connectdialogwidget));
			gtk_widget_destroy (GTK_WIDGET(connectdialogwidget));
			gui_connect_dialog ();
			g_free(host);
			g_free(user);
			g_free(pass);
			if (path)
				g_free(path);
			return 0;
		}

		g_free(host);
		g_free(user);
		g_free(pass);
		if (path)
			g_free(path);

		if (!guicoreconn_connect ())
		{
			g_printerr (N_("%s"), _("Connect to core failed - connection structure hasn't been set up?!\n"));
			connectdialogwidget = NULL;
			gtk_grab_remove (GTK_WIDGET(connectdialogwidget));
			gtk_widget_destroy (GTK_WIDGET(connectdialogwidget));
			gui_connect_dialog ();
		}
	}
	return 0;
}


/* connect_dialog_timer
 *
 * 'data' should be a pointer to a GtkLabel widget
 *
 */

static gint
connect_dialog_timer (gpointer data)
{
	if (CORE_IS_ALIVE)
		return 0;		/* don't call us again in this case (dialog is about to be destroyed anyway)*/

	g_return_val_if_fail (data!=NULL, 0);
	g_return_val_if_fail (GTK_IS_LABEL(data), 0);

	if ((ok_button)&&(spawn_button))
	{
		gtk_widget_set_sensitive (GTK_WIDGET(ok_button), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(spawn_button), TRUE);

		if (ew_connect_host)
		{
			const gchar *txt = gtk_entry_get_text (GTK_ENTRY(ew_connect_host));
			if ((!txt) || *txt==0x00 || txt[1]==0x00)
			{
				gtk_widget_set_sensitive (GTK_WIDGET(ok_button), FALSE);
				gtk_widget_set_sensitive (GTK_WIDGET(spawn_button), FALSE);
			}
		}

		if (ew_connect_user) 
		{
			const gchar *txt = gtk_entry_get_text (GTK_ENTRY(ew_connect_user));
			if ((!txt) || *txt==0x00)
			{
				gtk_widget_set_sensitive (GTK_WIDGET(ok_button), FALSE);
				gtk_widget_set_sensitive (GTK_WIDGET(spawn_button), FALSE);
			}
		}

		if (ew_connect_pass)
		{
			const gchar *txt = gtk_entry_get_text (GTK_ENTRY(ew_connect_pass));
			if ((!txt) || *txt==0x00)
			{
				gtk_widget_set_sensitive (GTK_WIDGET(ok_button), FALSE);
				gtk_widget_set_sensitive (GTK_WIDGET(spawn_button), FALSE);
			}
		}
	}

	if ((ew_connect_path) && (!GTK_WIDGET_IS_SENSITIVE(ew_connect_path)))
	{
		gtk_label_set_text (GTK_LABEL(data), _(" Core on remote host "));
		if (spawn_button)
			gtk_widget_set_sensitive (GTK_WIDGET(spawn_button), FALSE);
		return 1;		/* do call us again (hostname might change) */
	}

	local_donkey_pid = donkey_process_running_check ();

	/* we shouldn't set the spawn button active/inactive due to the detection.
	 * people might want to run two cores on the same box or something
	 */
	if (local_donkey_pid < 0)
		gtk_label_set_text (GTK_LABEL(data), _(" Could not check "));

	if (local_donkey_pid == 0)
	{
		gtk_label_set_text (GTK_LABEL(data), _(" Not running "));

		/* now this is a bit tricky... make GO! button insensitive
		 * if it's localhost and if we know that no core is running
		 * (assuming the core to be called 'donkey*'
		 */
		if ((ew_connect_path) && GTK_WIDGET_IS_SENSITIVE(ew_connect_path))
		{
			gtk_widget_set_sensitive (GTK_WIDGET(ok_button), FALSE);
		}
	}

	if (local_donkey_pid > 0)
	{
		gchar *msg = g_strdup_printf (N_("\n %s (pid %d) \n\n %s \n"), _("Running"), local_donkey_pid, misc_get_basename_from_pid (local_donkey_pid,NULL));
		if (msg)
		{
			gtk_label_set_text (GTK_LABEL(data), msg);
			g_free(msg);
		}
	}

	return 1;	// do call us again
}


/* connect_dialog_delete_event
 *
 *
 */

static void
connect_dialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	statusbar_extra_message (_(" GUI: Once you are connected to a core, you'll get rid of this dialog."));
	/* dialog is hidden now. Let's pop it up again */
	gtk_widget_show (widget);
}



/* connect_gui_corepath_changed
 *
 *
 */

static void
connect_gui_corepath_changed (GtkWidget *widget, gpointer data)
{
	struct stat		 pathstat;
	gchar			*path;

	if (!spawn_button)
		return;

	path = gtk_editable_get_chars (GTK_EDITABLE(widget), 0, -1);

	/* if path widget is insensitive, then the host is not a local host
	 * => it doesn't make sense to spawn a local core
	 */
	if (!GTK_WIDGET_IS_SENSITIVE(widget))
	{
		statusbar_extra_message(N_("   "));
		gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),FALSE);
		g_free(path);
		return;
	}

	/* is path relative and not absolute? */
	if (*path != '/')
	{
		/* XXX - we should just make it absolute
		 * (not here though, but after all the other checks
		 */
		statusbar_extra_message(_(" Please specify an absolute path (starting with '/').\n"));
		gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),FALSE);
		g_free(path);
		return;
	}

	/* path as entered does not exist? */
	if (access(path,F_OK)!=0)
	{
		statusbar_extra_message(_(" Entered path to binary does not exist."));
		gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),FALSE);
		g_free(path);
		return;
	}

	if (stat(path, &pathstat) != 0)
	{
		statusbar_extra_message(_(" Couldn't stat() entered path to binary."));
		gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),FALSE);
		g_free(path);
		return;
	}

	if (S_ISDIR(pathstat.st_mode))
	{
		statusbar_extra_message(_(" Entered path to binary is a directory. Please enter the name of the binary as well.\n"
								  " (e.g. /home/joe/ed2k/donkey_s_1658-gaps-3)"));
		gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),FALSE);
		g_free(path);
		return;
	}

	/* everything's fine. Let's overwrite previous error messages that might be there */
	statusbar_extra_message(N_("   "));
	gtk_widget_set_sensitive(GTK_WIDGET(spawn_button),TRUE);

	g_free(path);
	return;
}




/* connect_gui_hostname_changed
 *
 * This is not very sophisticated, but then why would anyone type 'berserker.dynisp.com'
 * instead of 'localhost' if he/she means localhost? (well, if they do it's their fault,
 * I don't care...)
 *
 * checks if the hostname/IP in the hostname widget is localhost or not and sets
 * the path to binary field active/inactive according to that.
 *
 */

static void
connect_gui_hostname_changed (GtkWidget *widget, gpointer data)
{
	gchar		*hostname = gtk_editable_get_chars (GTK_EDITABLE(widget), 0, -1);
	gboolean	 host_is_local = FALSE;

	if (!hostname)
		return;

	if (g_strncasecmp(hostname,N_("localhost"),9)==0)
		host_is_local = TRUE;

	if ((!host_is_local)&&(g_strncasecmp(hostname,N_("127.0.0."),8)==0))
		host_is_local = TRUE;
	
/*
 * Let's try not to be too smart and annoy people who want to do nifty stuff
 * like ssh-forwarding GUI connections etc.
 *
	if (!host_is_local)
	{
		// note: regardless of what the documentation says - at least up until gnet 1.1.7,
		// gnet_inetaddr_list_interfaces() does _not_ list loopback interfaces (see source)
		//
		GList *if_list = gnet_inetaddr_list_interfaces();
		PROCESS_GLIST (if_list, GInetAddr,
			if (!host_is_local)
			{
				const gchar *cname = gnet_inetaddr_get_canonical_name(item);
				if (cname)
				{
					if (g_strcasecmp(cname,hostname)==0)
						host_is_local = TRUE;
				}	
			}
			gnet_inetaddr_delete(item);
		);
		g_list_free(if_list);
	}
*/

	g_free(hostname);

	if (ew_connect_path)
	{
		if (host_is_local)
		{
			gtk_widget_set_sensitive (GTK_WIDGET(ew_connect_path), TRUE);
		} else gtk_widget_set_sensitive (GTK_WIDGET(ew_connect_path), FALSE);

		/* make sure 'spawn' button is updated as well */
		connect_gui_corepath_changed (ew_connect_path, NULL);
	}
}


/* gui_connect_dialog
 *
 * Displays the 'connect to...' dialog and makes it transient
 *
 */

void
gui_connect_dialog (void)
{
	GtkWidget *vbox1, *vbox2;
	GtkWidget *connect_frame;
	GtkWidget *table1;
//	GtkWidget *label1, *label2, *label3, *label4, *label5, *dummylabel;
	GtkWidget *label_quick_help;
	GtkWidget *hbox1, *hbox2;
	GtkWidget *hbuttonbox1;
	GtkWidget *hbuttonbox3;
	GtkWidget *connect_status_frame;
	GtkWidget *connect_label;
	GtkWidget *frame3;
	GtkWidget *hbuttonbox2;
	GtkWidget *help_button;
	GtkWidget *quickhelp_button;
	GtkWidget *exit_button;

	GtkTooltips	*tooltips = NULL;

	if ((!window) || CORE_IS_ALIVE)
		return;

#if ((GTK_MAJOR_VERSION == 2) || (GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 3))
	g_print ("***** GTK2 FIXME: %s\n", __FUNCTION__);
	connect_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
#else
	connect_dialog = gtk_window_new (GTK_WINDOW_DIALOG);
#endif
	gtk_object_set_data (GTK_OBJECT (connect_dialog), N_("connect_dialog"), connect_dialog);
	gtk_window_set_title (GTK_WINDOW (connect_dialog), _("connect to donkey core"));
	gtk_window_set_position (GTK_WINDOW (connect_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (connect_dialog), FALSE, TRUE, TRUE);

	/* connect the destroy event to the dialogue, so people can't just click in the corner of
	 * the connect dialog and close it and run the gui without being connected to the core
	 */

	gtk_signal_connect (GTK_OBJECT (connect_dialog), N_("delete_event"),
					GTK_SIGNAL_FUNC (connect_dialog_delete_event), NULL);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("vbox1"), vbox1,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (connect_dialog), vbox1);

	connect_frame = gtk_frame_new (_("Connect to donkey core"));
	gtk_widget_ref (connect_frame);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("connect_frame"), connect_frame,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (connect_frame);
	gtk_box_pack_start (GTK_BOX (vbox1), connect_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER(connect_frame),5);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("vbox2"), vbox2,
								(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (connect_frame), vbox2);

	table1 = gtk_table_new (5, 2, FALSE);
	gtk_widget_ref (table1);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("table1"), table1,
								(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox2), table1, TRUE, TRUE, 0);

	OPTIONS_ADD_ENTRY(table1, _("host of running core "), ew_connect_host, 0, NULL);
	gtk_signal_connect (GTK_OBJECT (ew_connect_host), N_("changed"),
			    GTK_SIGNAL_FUNC (connect_gui_hostname_changed), NULL);
	OPTIONS_ADD_SPINBUTTON (table1, _("core port"), spinbutton_connect_port,
								option_defaultport, 10, 65535, 1, 1, NULL);
/*								_("This is the port the core listens on for GUI connections. "
								  "The standard port for this is 4663. Note: the core will only "
								  "listen for GUIs if it is started with the '-' option (in case "
								  "you start the core yourself and not via the GUI."));
*/
	OPTIONS_ADD_ENTRY(table1, _("core username"), ew_connect_user, 2, NULL);
	OPTIONS_ADD_ENTRY(table1, _("core password"), ew_connect_pass, 3, NULL);
	OPTIONS_ADD_ENTRY(table1, _("path to local core"), ew_connect_path, 4, NULL);
	/* 'changed' event is hooked up at the end of the function, because we need
	 * a valid spawn_button widget when we set this up
	 */

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("hbox1"), hbox1,
					(GtkDestroyNotify) gtk_widget_unref);

	/* fill default values into entry widgets */

	gtk_entry_set_text (GTK_ENTRY(ew_connect_host), (option_defaulthost!=NULL) ? option_defaulthost : N_(""));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinbutton_connect_port), option_defaultport*1.0);
	gtk_entry_set_text (GTK_ENTRY(ew_connect_user), (option_defaultuser!=NULL) ? option_defaultuser: N_(""));
	gtk_entry_set_text (GTK_ENTRY(ew_connect_pass), (option_defaultpass!=NULL) ? option_defaultpass : N_(""));
	gtk_entry_set_text (GTK_ENTRY(ew_connect_path), (option_defaultdonkeypath!=NULL) ? option_defaultdonkeypath : N_(""));

	/* has configured binary vanished/been deleted since last start? */
	if ((option_defaultdonkeypath) && access(option_defaultdonkeypath,F_OK|R_OK|X_OK)!=0 )
	{
		/* binary has vanished. reset to empty path */
	    strrealloccpy (&option_defaultdonkeypath, N_(""));
	}

	/* if we don't have a path to the donkey binary then we'll try to find it */
	if ((!option_defaultdonkeypath) || ((option_defaultdonkeypath) && *option_defaultdonkeypath==0x00))
	{
                gui_connect_find_binary();
	}

	/* hide clear text password */
	gtk_entry_set_visibility (GTK_ENTRY(ew_connect_pass), FALSE);

	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

	cb_connect_makedef = gtk_check_button_new_with_label (_("save as default values"));
	gtk_widget_ref (cb_connect_makedef);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("cb_connect_makedef"), cb_connect_makedef,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_connect_makedef), (option_connect_make_def_values==1));

	gtk_widget_show (cb_connect_makedef);

	gtk_box_pack_start (GTK_BOX (hbox1), cb_connect_makedef, FALSE, FALSE, 5);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_ref (hbuttonbox1);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("hbuttonbox1"), hbuttonbox1,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (hbox1), hbuttonbox1, TRUE, TRUE, 5);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);

	ok_button = gtk_button_new();
	gtk_widget_ref (ok_button);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("ok_button"), ok_button,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add(GTK_CONTAINER(ok_button), make_icon_and_label_hbox(" GO! ", icon_ignition));
	gtk_widget_show (ok_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok_button);
	GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

	connect_status_frame = gtk_frame_new (_("Donkey status"));
	gtk_widget_ref (connect_status_frame);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("connect_status_frame"), connect_status_frame,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (connect_status_frame);
	gtk_box_pack_start (GTK_BOX (vbox1), connect_status_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER(connect_status_frame),5);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("hbox2"), hbox2,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_container_add (GTK_CONTAINER (connect_status_frame), hbox2);

	connect_label = gtk_label_new (N_(" "));
	gtk_widget_ref (connect_label);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("connect_label"), connect_label,
					(GtkDestroyNotify) gtk_widget_unref);
	connect_timer_handle = gtk_timeout_add (1000, connect_dialog_timer, connect_label);
	gtk_widget_show (connect_label);
	gtk_box_pack_start(GTK_BOX(hbox2), connect_label, TRUE, FALSE, 0);

	hbuttonbox3 = gtk_hbutton_box_new ();
	gtk_widget_ref (hbuttonbox3);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("hbuttonbox3"), hbuttonbox3,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbuttonbox3);
	gtk_box_pack_start (GTK_BOX (hbox2), hbuttonbox3, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox3), GTK_BUTTONBOX_END);

	frame3 = gtk_frame_new (NULL);
	gtk_widget_ref (frame3);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("frame3"), frame3,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame3);
	gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER(frame3),5);

	spawn_button = gtk_button_new();
	gtk_widget_ref (spawn_button);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("spawn_button"), spawn_button,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add(GTK_CONTAINER(spawn_button),
					make_icon_and_label_hbox(_(" Spawn core "), icon_core));
	gtk_widget_show (spawn_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox3), spawn_button);
	GTK_WIDGET_SET_FLAGS (spawn_button, GTK_CAN_DEFAULT);


	/* PATH widget - hook up 'changed' signal now */
	gtk_signal_connect (GTK_OBJECT (ew_connect_path), N_("changed"),
			    GTK_SIGNAL_FUNC (connect_gui_corepath_changed), NULL);


	hbuttonbox2 = gtk_hbutton_box_new ();
	gtk_widget_ref (hbuttonbox2);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("hbuttonbox2"), hbuttonbox2,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbuttonbox2);
	gtk_container_add (GTK_CONTAINER (frame3), hbuttonbox2);

	help_button = gtk_button_new();
	gtk_widget_ref (help_button);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("help_button"), help_button,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add(GTK_CONTAINER(help_button),
					make_icon_and_label_hbox(_(" Help "), icon_about));
	gtk_widget_show (help_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox2), help_button);
	GTK_WIDGET_SET_FLAGS (help_button, GTK_CAN_DEFAULT);

	quickhelp_button = gtk_button_new();
	gtk_widget_ref (quickhelp_button);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("quickhelp_button"), quickhelp_button,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add(GTK_CONTAINER(quickhelp_button),
					make_icon_and_label_hbox(_(" Quick start "), icon_about));
	gtk_widget_show (quickhelp_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox2), quickhelp_button);
	GTK_WIDGET_SET_FLAGS (quickhelp_button, GTK_CAN_DEFAULT);

	exit_button = gtk_button_new();
	gtk_widget_ref (exit_button);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("exit_button"), exit_button,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add(GTK_CONTAINER(exit_button),
					make_icon_and_label_hbox(_(" Exit "), icon_exit));
	gtk_widget_show (exit_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox2), exit_button);
	GTK_WIDGET_SET_FLAGS (exit_button, GTK_CAN_DEFAULT);

	gtk_signal_connect(GTK_OBJECT (ok_button), N_("clicked"),
			GTK_SIGNAL_FUNC (gui_connect_dialog_connect_button_clicked),
			GTK_WINDOW (connect_dialog));
	gtk_signal_connect(GTK_OBJECT (spawn_button), N_("clicked"),
			GTK_SIGNAL_FUNC (gui_connect_dialog_spawn_button_clicked),
			GTK_OBJECT (spawn_button));
	gtk_signal_connect(GTK_OBJECT (help_button), N_("clicked"),
			GTK_SIGNAL_FUNC (gui_connect_dialog_help_button_clicked),
			GTK_OBJECT (help_button));
	gtk_signal_connect(GTK_OBJECT (quickhelp_button), N_("clicked"),
			GTK_SIGNAL_FUNC (gui_connect_dialog_quickhelp_button_clicked),
			GTK_OBJECT (quickhelp_button));
	gtk_signal_connect(GTK_OBJECT (exit_button), N_("clicked"),
			GTK_SIGNAL_FUNC (gui_connect_dialog_exit_button_clicked),
			GTK_OBJECT (exit_button));

	frame_quick_help = gtk_frame_new (NULL);
	gtk_widget_ref (frame_quick_help);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("frame_quick_help"), frame_quick_help,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_hide (frame_quick_help);
	gtk_box_pack_start (GTK_BOX (vbox1), frame_quick_help, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER(frame_quick_help),5);

	label_quick_help = gtk_label_new (QUICK_START_TEXT);
	gtk_widget_ref (label_quick_help);
	gtk_object_set_data_full (GTK_OBJECT (connect_dialog), N_("label_quick_help"), label_quick_help,
					(GtkDestroyNotify) gtk_widget_unref);
	gtk_label_set_justify(GTK_LABEL(label_quick_help),GTK_JUSTIFY_LEFT);
	gtk_widget_show (label_quick_help);
	gtk_container_add (GTK_CONTAINER (frame_quick_help), label_quick_help);

	gtk_grab_add(connect_dialog);
	gtk_widget_show (connect_dialog);

	gtk_window_set_transient_for (GTK_WINDOW(connect_dialog), GTK_WINDOW(window));

	/* now that the spawn button is set, update its state */
	connect_gui_corepath_changed (ew_connect_path, NULL);

	return;
}


