/***************************************************************************
              downloads_attrib.c  -  downloads attributes stuff

	* sync attributes data in this list with downloads
	* save attributes data to .xml file
	* read attributes data from .xml file

                             -------------------
    begin                : Wed Jul 10 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "downloads_attrib.h"
#include "global.h"
#include "gui_options.h"
#include "misc.h"
#include "status_page.h"

#include <stdio.h>
#include <unistd.h>
#include <gtk/gtk.h>

// separate list with records recently downloaded. Currently only
//	saving/syncing all the 'last seen XYZ' data
static GPtrArray		*downloadsAttributes = NULL;


// to remedy old sins we need to undefine these as
//	long as filelist_item records are used...
#undef lasttime_complete
#undef time_added


// --------- local functions

static downloadAttribRecord	*downloadsattriblist_get_from_hash (const guint8 *hash);
static downloadAttribRecord	*downloadsattriblist_add_record (const guint8 *hash);
static void			 downloadsattriblist_weed_out_unused (void);


// downloadsattriblist_add_record
//
// Adds a new record to the downloads attributes list with the given hash.
// Returns new record if it didn't exist, old record if it existed, or NULL on error.

static downloadAttribRecord *
downloadsattriblist_add_record (const guint8 *hash)
{
	downloadAttribRecord *rec;

	g_return_val_if_fail (hash!=NULL, NULL);
	if (!downloadsAttributes) downloadsAttributes = g_ptr_array_new();
	g_return_val_if_fail (downloadsAttributes!=NULL,NULL);
	rec = downloadsattriblist_get_from_hash (hash);
	if (!rec)
	{
		rec = g_new0 (downloadAttribRecord, 1);
		g_return_val_if_fail (rec!=NULL,NULL);
		memcpy (rec->hash, hash, 16);
		g_ptr_array_add (downloadsAttributes, (gpointer) rec);
	}
	return rec;
}



// downloadsattriblist_get_from_hash
//
// Find the attributes record corresponding to the given file hash
// Returns record or NULL if not found

static downloadAttribRecord *
downloadsattriblist_get_from_hash (const guint8 *hash)
{
	if (!downloadsAttributes) return NULL;
	g_return_val_if_fail (hash!=NULL, NULL);
	PROCESS_GPTRARRAY( downloadsAttributes, downloadAttribRecord,
		if ((item) && memcmp(item->hash, hash, 16)==0) return item;
	);
	return NULL;
}


// update_dlstatslist_from_downloadlist
//
// sync the attributes list with the downloads list (so that we always
//	save the latest values when we save the attributes list)

void
downloadsattriblist_update_from_downloadlist (void)
{
	if (!downloads) return;
	PROCESS_GPTRARRAY( downloads, downloadRecord,
		if (item)
		{
			downloadAttribRecord *rec = downloadsattriblist_get_from_hash (item->hash);
			if (!rec) rec = downloadsattriblist_add_record (item->hash);
			g_return_if_fail (rec!=NULL);
			rec->last_downloading = item->lasttime;
			rec->last_complete = item->lasttime_complete;
			rec->last_used = time(NULL);
			if (item->time_added==0) item->time_added = time(NULL);
			rec->time_added = item->time_added;
		}
	);
}


// downloadsattriblist_set_download_record_attributes
//
// Called from parse message on NEW_DOWNLOAD message.
// Sets the download record's attributes fields (ie. the last time stuff)
//	to the values that we've saved in the list if the record is in our attrib list

void
downloadsattriblist_set_download_record_attributes (downloadRecord *dl)
{
	downloadAttribRecord *rec;
	g_return_if_fail (dl!=NULL);
	if (downloadsAttributes)
	{
		rec = downloadsattriblist_get_from_hash (dl->hash);
		if (rec)
		{
			dl->lasttime_complete = rec->last_complete;
			dl->lasttime = rec->last_downloading;
			if (rec->time_added == 0) rec->time_added = time(NULL);
			dl->time_added = rec->time_added;
			rec->last_used = time(NULL);
		} // else: record not in attrib list yet => nothing to do
	} // else: no list, so record can't be in attrib list => nothing to do
}


// downloadsattriblist_weed_out_unused
//
// weed out entries in the downloads attributes list that haven't been
//	used for a long time.

static void
downloadsattriblist_weed_out_unused (void)
{
	if (!downloadsAttributes) return;
	PROCESS_GPTRARRAY_FROM_END ( downloadsAttributes, downloadAttribRecord,
		if ((item) && time(NULL)-item->last_used > 3600*24*31)
		{
			g_ptr_array_remove_fast (downloadsAttributes, (gpointer) item);
			g_free(item);
		}
	);
}


// downloadsattriblist_save
//
// Save downloads attributes list to file

void
downloadsattriblist_save (void)
{
	FILE		*f;
	const gchar	*fn = complete_pref_path (N_("gui_downloadstats.txt"), GUI_CONFIG_USE_INSTANCE);

	g_return_if_fail (fn!=NULL);

	f = fopen (fn, N_("w"));
	if (f)
	{
		fprintf (f, N_("# ed2k_gui --- gui_downloadsstats --- config format version 0.3.0\n"));
		fprintf (f, N_("#\n%s\n"), _("# This file is created by the ed2k gui. It contains information about your"));
		fprintf (f, N_("%s\n#\n"), _("# downloads. DO NOT CHANGE THE ABOVE HEADER!"));
		fprintf (f, N_("%s\n#\n"), _("# hash - last seen downloading - last seen fully available - last used - time added"));

		if (downloadsAttributes)
		{
			downloadsattriblist_weed_out_unused ();
			PROCESS_GPTRARRAY ( downloadsAttributes, downloadAttribRecord,
				if (item)
				{
					fprintf (f, N_("%32s %lu %lu %lu %lu\n"),
						hash_to_hash_str(item->hash),
						item->last_downloading,
						item->last_complete,
						item->last_used,
						item->time_added);
				}
			);
		}
		fclose(f);
	} else status_system_error_msg (_("Couldn't open gui_downloadsstats.txt for writing."));
}



// downloadsattriblist_load
//
// Loads the downloads attributes list from an XML file.

void
downloadsattriblist_load (void)
{
	const gchar	*fn;
	gchar	**line, **linearray, *oldfilename;

	// First check whether an old-style file still exists
	fn = complete_pref_path (N_("gui_downloadstats"), GUI_CONFIG_USE_INSTANCE);
	g_return_if_fail (fn!=NULL);
	if ( access(fn, F_OK) == 0)
	{
		oldfilename = g_strdup_printf (N_("%s.old"), fn);
		rename (fn, oldfilename);
		g_free (oldfilename);
		g_printerr (N_("%s"), _("Due to a change in config file format your 'last seen'\n"));
		g_printerr (N_("%s"), _("times in the download list have been lost. Sorry.\n"));
	}


	// Now read in the file
	linearray = misc_get_array_of_lines_from_textfile (complete_pref_path (N_("gui_downloadstats.txt"), GUI_CONFIG_USE_INSTANCE));
	if (linearray)	// there should be at least our header!
	{
		guint vermaj, vermin, verminmin;
		line = linearray;
		if ((*line) && sscanf(*line, N_("# ed2k_gui --- gui_downloadsstats --- config format version %u.%u.%u\n"), &vermaj, &vermin, &verminmin)==3)
		{
			gchar	hashstr[33];
			guint	last_downloading, last_fullyavailable, last_used, time_added;

			line++;
			while (*line)
			{
				if (**line!='#' && **line!=0x00)
				{
					if (5 == sscanf(*line, N_("%32s %u %u %u %u\n"), hashstr, &last_downloading, &last_fullyavailable, &last_used, &time_added))
					{
						downloadAttribRecord *rec = downloadsattriblist_add_record (hash_str_to_hash(hashstr));
						if (rec)
						{
							rec->last_downloading = (time_t) last_downloading;
							rec->last_complete = (time_t) last_fullyavailable;
							rec->last_used = (time_t) last_used;
							rec->time_added = (time_t) time_added;
						}
					}
				}
				line++;
			}
		} else g_printerr (N_("%s"), _("gui_downloadsstats.txt: invalid header?\n"));
		g_strfreev (linearray);
	} else g_printerr (N_("%s"), _("gui_downloadsstats.txt: couldn't read file or file empty.\n"));
}


