/***************************************************************************
                          linkhandler.c  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  The code in here is called from main() when we are called with
 *   an ed2k-link as the first command line argument.
 *
 *  In this case we don't want a GUI instance to pop up, but
 *   only a separate little window that passes the link on to
 *   an already running GUI and then goes away.
 *
 *  Unless, of course, there is no GUI running yet. Then we need
 *   to start one and pass the link to the core after connect.
 *
 */

#include "gui_options.h"
#include "icons.h"
#include "linkhandler.h"
#include "misc.h"
#include "misc_gtk.h"
#include "misc_sys.h"
#include "status_page.h"

#include <sys/types.h>
#include <sys/wait.h>	/* for the return code from system() */
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>

/* local variables */

static GSList   		*linkhandler_pipes_found = NULL;
static gchar    		*linkhandler_ed2klink = NULL;
static gchar      	**linkhandler_pending_links_array = NULL;
static GtkWidget		*linkhandler_entry = NULL;
static GtkWidget		*linkhandler_window = NULL;
static GtkWidget		*linkhandler_statuslabel = NULL;
static GtkWidget		*linkhandler_checkbutton_make_this_default_action = NULL;
static const gchar	*linkhandler_error_message = NULL;
static gboolean   	 linkhandler_main_run_gui_override_flag = FALSE;

/* local functions */

static void       linkhandler_remove_stale_pipes(void);
static gboolean   linkhandler_remove_stale_pipes_is_options_file_bad(const gchar *pipename, time_t pipe_mtime);
static gint       linkhandler_check_for_existing_GUI_instances (void);

static void       linkhandler_send_link_to_all_pipes (const gchar *link);

static void       linkhandler_ask_buttons_callback (GtkWidget *widget, gpointer data);
static void       linkhandler_instance_button_callback (GtkWidget *widget, gpointer data);
static GtkWidget *linkhandler_create_buttons (void);
static gint       linkhandler_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data);
static void       linkhandler_destroy (GtkWidget *widget, gpointer data);
static void       linkhandler_create_mainwindow (const gchar *ed2klink);

static gboolean   linkhandler_append_link_to_pending_file (const gchar *link);
 

/******************************************************************************
 *
 *   linkhandler_install_kde
 *
 *   installs the GUI as linkhandler for ed2k://-links in KDE
 *   (adds a file ed2k.protocol in ~/share/services/)
 *
 *   returns error string on failure, and NULL on success
 */

const gchar *
linkhandler_install_kde (void)
{
	FILE  *cfgfile;
	gchar *home;
	gchar *cfgpath;
	gchar *cfgfn;

	home = g_getenv(N_("HOME"));
	g_return_val_if_fail (home!=NULL, N_("home==NULL?!"));

	cfgpath = g_strdup_printf (N_("%s/.kde/share/services/"), home);
	g_return_val_if_fail (home!=NULL, N_("cfgpath==NULL?!"));

	if (access(cfgpath,X_OK)!=0)
		return _("$HOME/.kde/share/services/ does not exist?!");

	if (access(cfgpath,W_OK)!=0)
		return _("$HOME/.kde/share/services/ is not writable?!");

	cfgfn = g_strdup_printf (N_("%s/ed2k.protocol"), cfgpath);
	g_return_val_if_fail (cfgfn!=NULL, N_("cfgfn==NULL?!"));

	cfgfile = fopen (cfgfn, N_("w"));
	g_return_val_if_fail (cfgfile!=NULL, _("could not open $HOME/.kde/share/services/ed2k.protocol for writing?!"));

	fprintf(cfgfile,N_("[Protocol]\n"));
	fprintf(cfgfile,N_("exec=ed2k_gui %%u\n"));
	fprintf(cfgfile,N_("protocol=ed2k\n"));
	fprintf(cfgfile,N_("input=none\n"));
	fprintf(cfgfile,N_("output=none\n"));
	fprintf(cfgfile,N_("helper=true\n"));
	fprintf(cfgfile,N_("listing=\n"));
	fprintf(cfgfile,N_("reading=false\n"));
	fprintf(cfgfile,N_("writing=false\n"));
	fprintf(cfgfile,N_("makedir=false\n"));
	fprintf(cfgfile,N_("deleting=false\n"));
	fprintf(cfgfile,N_("Icon=remote\n"));
	fprintf(cfgfile,N_("Description=ed2k-gtk-gui handles eDonkey2000 ed2k://-links\n"));
	fclose(cfgfile);

	status_message_blue (_("GUI: created new file ~/.kde/share/services/ed2k.protocol\n"));

	return NULL;
}


/******************************************************************************
 *
 *   linkhandler_install_gnome1
 *
 *   installs the GUI as linkhandler for ed2k://-links in GNOME 1
 *
 *   returns error string on failure, and NULL on success
 */

const gchar *
linkhandler_install_gnome1 (void)
{
	gchar		 *gnomefilepath;
	gchar		**gnomefilebuf;
	gchar		 *gnomefilefn;
	gchar		 *home;
	const gchar	 *linkstring = N_("ed2k-show=ed2k_gui \"%s\"");
	gboolean	  done = FALSE;
	gint    	  i;
	
	home = g_getenv(N_("HOME"));
	g_return_val_if_fail (home!=NULL, N_("home==NULL?!"));
	
	gnomefilepath = g_strdup_printf(N_("%s/.gnome/"), home);
	g_return_val_if_fail (gnomefilepath!=NULL, N_("gnomefilepath==NULL?!"));
	
	if (access(gnomefilepath, X_OK) != 0)
		return (_("$HOME/.gnome does not exist ?!"));

	if (access(gnomefilepath, W_OK) != 0)
		return (_("$HOME/.gnome is not writable ?!"));

	gnomefilefn = g_strdup_printf(N_("%sGnome"), gnomefilepath);
	g_return_val_if_fail(gnomefilefn != NULL, N_("gnomefilefn=NULL?!"));

	gnomefilebuf = misc_get_array_of_lines_from_textfile(gnomefilefn);

	/* If file does not yet exist or is empty
	 *   => create it with [URL Handlers] section
	 *
	 * If file does already exist
	 *   => (1) check whether [URL Handlers] section exists
	 *           If no => add section
	 *      (2) add ed2k handler in this section and remove all existing ed2k-handlers
	 *
	 */
	if ((!gnomefilebuf) || (!*gnomefilebuf))
	{
		G_FREE(gnomefilebuf);
		gnomefilebuf = g_new0(gchar*, 2);
		g_return_val_if_fail(gnomefilebuf != NULL, N_("gnomefilebuf==NULL?!"));
		gnomefilebuf[0] = g_strdup_printf(N_("[URL Handlers]\n%s"), linkstring);
		g_return_val_if_fail(gnomefilebuf[0]!=NULL, N_("gnomefilebuf[0]==NULL?!"));
		gnomefilebuf[1] = NULL;

	} else {

		gint handler_section_line = -1;	/* line in which it exists; -1 = does not exist */
		gint linesinfile = 0;

		/* Walk through lines of file.
		 *  - count number of lines
		 *  - check whether we already have an 'URL Handlers' section and where it is
		 *  - remove existing ed2k handlers
		 */

		for (i = 0; gnomefilebuf[i]!=NULL; i++)
		{
			linesinfile++;

			/* remember where 'URL Handlers' section starts */
			if (strstr(gnomefilebuf[i], N_("[URL Handlers]")))
				handler_section_line = i;

			/* replace existing ed2k-handlers by empty line */
			if (strstr(gnomefilebuf[i], N_("ed2k-show")))
			{
				g_free(gnomefilebuf[i]);
				gnomefilebuf[i] = g_strdup(N_("SkIpThIsLiNeDoNoTwRiTeIt"));
			}
		}

		/* If section does not exist yet
		 *   => add section + handler at the end, so we don't
		 *      screw up stuff before that does not belong to 
		 *      URL Handler section
		 *
		 * If section exists, add new ed2k-handler first
		 *   thing after section header (we have already
		 *   removed all previous ed2k handlers, so this
		 *   should be fine)
		 *
		 */
		if (handler_section_line<0)
		{
			gchar *oldlastline = gnomefilebuf[linesinfile-1];
			gnomefilebuf[linesinfile-1] = g_strdup_printf(N_("%s\n[URL Handlers]\n%s"),oldlastline,linkstring);
			g_free(oldlastline);
		} else {
			g_free(gnomefilebuf[handler_section_line]);
			gnomefilebuf[handler_section_line] = g_strdup_printf(N_("[URL Handlers]\n%s"),linkstring);
		}
	}

	/* gnomefilebuf should really be set
	 *   here, but better check again
	 *
	 * write new ~/.gnome/Gnome file to disk
	 *
	 */
	if (gnomefilebuf)
	{
		FILE *gnomefile;

		gnomefile = fopen(gnomefilefn, N_("w")); 
		if (!gnomefile)
		{
			g_strfreev(gnomefilebuf);
			g_return_val_if_fail(gnomefile != NULL, _("couldn't open $HOME/.gnome/Gnome for writing ?!"));
		}

		for (i = 0; gnomefilebuf[i] != NULL; i++)
		{
			if (strcmp(gnomefilebuf[i],N_("SkIpThIsLiNeDoNoTwRiTeIt"))!=0)
			{
				fprintf(gnomefile, N_("%s\n"), gnomefilebuf[i]);
			}
		}
		
		g_strfreev(gnomefilebuf);
		fclose(gnomefile); 
	} else return N_("gnomefilebuf==NULL?!");

	g_free(gnomefilepath);
	g_free(gnomefilefn);

	status_message_blue (_("GUI: modified file ~/.gnome/Gnome\n"));

	return NULL;
}


/******************************************************************************
 *
 *   linkhandler_install_gnome2
 *
 *   installs the GUI as linkhandler for ed2k://-links in GNOME 2
 *
 *   returns error string on failure, and NULL on success
 */

const gchar *
linkhandler_install_gnome2 (void)
{
	const gchar *scriptfilename; 
	gint         ret;

	scriptfilename = complete_pref_path (N_("gnome2_ed2k_url_handler_installer.sh"), FALSE);
	g_return_val_if_fail (scriptfilename!=NULL, FIXME_STRING);

	/* check whether script already exists in config folder */
	if (access(scriptfilename, R_OK | X_OK) != 0)
	{
		/* preview script does not exist yet. Create it. */
		FILE *of;
		of = fopen (scriptfilename, N_("w"));
		if (!of)
		{
			return _("Could not create URL handler installer script in config path!?");
		}
		status_message_blue ("GUI: Creating URL handler installer script '%s'...\n", scriptfilename);

		fprintf (of, N_("#!/bin/sh\n#\n# This little script installs the ed2k_gui as default handler\n"));
		fprintf (of, N_("#  for ed2k:// URLs in the GNOME2 desktop environment\n#\n\n"));
		fprintf (of, N_("echo\necho \"--- Trying to install ed2k link handler for GNOME2 ---\"\necho\n"));
		fprintf (of, N_("echo -n -e \"Testing for gconftool-2 ...\t\"\n\n"));
		fprintf (of, N_("GCONFTOOL2=`which gconftool-2`\n\n"));

		fprintf (of, N_("if [ ! -x \"$GCONFTOOL2\" ]\nthen\n"));
		fprintf (of, N_("echo -e \"not found.\n\nAre you sure you are using GNOME2?\"\nexit 1;\n"));
		fprintf (of, N_("else\necho \"ok ($GCONFTOOL2)\"\nfi\n\n"));
		fprintf (of, N_("echo -n -e \"Setting up keys with gconftool-2 ...\t\"\n"));
		fprintf (of, N_("if $GCONFTOOL2 --recursive-unset /desktop/gnome/url-handlers/ed2k\n"));
		fprintf (of, N_("then\n if $GCONFTOOL2 --type=string --set /desktop/gnome/url-handlers/ed2k/description \"eDonkey2000-link\"\n"));
		fprintf (of, N_(" then\n  if $GCONFTOOL2 --type=string --set /desktop/gnome/url-handlers/ed2k/command 'ed2k_gui \"%%s\"'\n"));
		fprintf (of, N_("  then\n   if $GCONFTOOL2 --type=boolean --set /desktop/gnome/url-handlers/ed2k/type true\n"));
		fprintf (of, N_("   then\n    echo -e \"ok\n\nDone.\n\"\n    exit 0\n  fi\n fi\nfi\nfi\n"));
		fprintf (of, N_("echo -e \"failed\n\n\"\nexit 2;\n"));

		/* make script executable */
		if (fchmod(fileno(of), S_IRUSR|S_IWUSR|S_IXUSR)<0)
		{
			fclose(of);
			return _("Couldn't make script executable!\n");
		}
		fclose(of);
		status_message_blue (_("GUI: installer script '%s' created. Trying to run it now.\n"), scriptfilename);
	} else status_message_blue (_("GUI: installer script '%s' already existed. Trying to run it now.\n"), scriptfilename);

	ret = system(scriptfilename);

	if (ret == -1)
	{
		return N_("fork() in system() failed.");
	}

	if (WEXITSTATUS(ret) == 0)
	{
		return NULL;
	}

	if (WEXITSTATUS(ret) == 1)
	{
		return _("gconftool-2 program not found. Are you sure you are using GNOME 2?");
	}

	if (WEXITSTATUS(ret) == 2)
	{
		return _("Something went wrong when installing the configuration keys with\n"
		         "  gconftool-2. Run the script manually to see what went wrong. ");
	}

	return FIXME_STRING;
}




/******************************************************************************
 *
 *   linkhandler_remove_stale_pipes_is_options_file_bad
 *
 *   checks whether the options file corresponding to a pipe has
 *    been changed recently or exists at all.
 *
 *   Returns TRUE if the pipe is probably stale (options file not changed),
 *    and FALSE if the pipe might be in use.
 *
 */

static gboolean
linkhandler_remove_stale_pipes_is_options_file_bad(const gchar *pipename, time_t pipe_mtime)
{
	struct stat	 optionsfile_status;
	gchar      	*basename    = g_basename(pipename);
	gchar      	*dirname     = g_dirname(pipename);
	gchar      	*optionsname;
	gint       	 ret;

	g_return_val_if_fail (dirname!=NULL, FALSE);

	optionsname = g_strdup_printf(N_("%s/gui_options%s"),dirname, (strcmp(basename,N_("pipe"))==0) ? N_("") : basename+4);

	g_free(dirname);
	dirname=NULL;

	g_return_val_if_fail (optionsname!=NULL, FALSE);

	memset(&optionsfile_status,0x00,sizeof(optionsfile_status));

	ret = stat(optionsname, &optionsfile_status);
	g_free(optionsname);
	optionsname = NULL;

	/* stat() failed? if options file does not exist, check
	 *  whether pipe was created a long time ago, and
	 *  an options file should exist
	 */
	if (ret<0)
	{
		return ((errno==ENOENT) && pipe_mtime < (time(NULL)-(60*60)));
	}
	return (optionsfile_status.st_mtime < (time(NULL)-(60*60)));
}



/******************************************************************************
 *
 *   linkhandler_remove_stale_pipes
 *
 *   removes pipes that have existed longer than the system was up
 *    => those are definitively stale
 *
 */

static void
linkhandler_remove_stale_pipes(void)
{
	time_t	 uptime, system_start;
	GSList	*node;

	uptime = misc_sys_get_uptime_in_secs();

	if (uptime==0)
		return;

	system_start = time(NULL)-uptime;

	node = linkhandler_pipes_found;
	while(node)
	{
		if ((node)&&(node->data))
		{
			struct stat pipe_status;
			memset(&pipe_status,0x00,sizeof(pipe_status));
			if (stat((gchar*)node->data, &pipe_status)==0)
			{
				gboolean	 remove_it = FALSE;

				/* (1) Was pipe last modified/created before last system start?
				 *      If yes => it's stale. Remove it.
				 *
				 * (2) Was the gui_options file that belongs to the GUI instance that
				 *      owns the pipe modified in the last 60 minutes?
				 *      If no, then GUI is not running, and pipe is stale => Remove it.
				 *
				 */

				if (pipe_status.st_mtime < system_start)
				{
					remove_it = TRUE;
				} else
				{
					if (linkhandler_remove_stale_pipes_is_options_file_bad((gchar*)node->data, pipe_status.st_mtime))
					{
						remove_it = TRUE;
					}
				}

				if (remove_it)
				{
					GSList *oldnode = node;
					if (unlink((gchar*)oldnode->data)==0)
					{
						g_print ("removed stale pipe: %s\n", (gchar*)oldnode->data);
					} else g_print ("could not remove stale pipe: %s - %s\n", (gchar*)oldnode->data, g_strerror(errno));

					node = node->next;
					linkhandler_pipes_found = g_slist_remove_link(linkhandler_pipes_found, oldnode);
					if(oldnode->data);
						g_free(oldnode->data);
					g_slist_free_1(oldnode);
					continue;
				}

			}
		}
		node=node->next;
	}
}


/******************************************************************************
 *
 *   linkhandler_check_for_existing_GUI_instances
 *
 *   checks whether there is already one or more GUIs running.
 *
 *   Returns number of GUI instances found, or -1 on error.
 *
 */

static gint
linkhandler_check_for_existing_GUI_instances (void)
{
	struct dirent	*entry;
	DIR          	*directoryhandle;

	g_return_val_if_fail (prefpath!=NULL, -1);

	directoryhandle = opendir (prefpath);
	if (!directoryhandle)
	{
		g_printerr (N_("opendir(%s) failed - %s\n"), prefpath, strerror(errno));
		return -1;
	}

	/* remove and free existing, previously
	 *  found results if there are any
	 */
	if (linkhandler_pipes_found)
	{
		GSList *node = linkhandler_pipes_found;
		while (node)
		{
			G_FREE(node->data);	/* previously allocated with g_strdup(pipe_path) */
			node = node->next;
		}
	}

	while ((entry=readdir(directoryhandle)))
	{
		if (!entry->d_name)
			continue;

		if (entry->d_name[0]=='p' && entry->d_name[1]=='i'
			&& entry->d_name[2]=='p' && entry->d_name[3]=='e')
		{
			gchar *fullpath = g_strdup_printf(N_("%s/%s"),prefpath,entry->d_name);
			if (fullpath)
			{
				linkhandler_pipes_found = g_slist_append(linkhandler_pipes_found, (gpointer)fullpath);
/*				g_print (">>>>>>>>> found: '%s'\n", fullpath); */
			}
		}
	}
	closedir(directoryhandle);

	linkhandler_remove_stale_pipes();

	return (gint) g_slist_length(linkhandler_pipes_found);
}


/******************************************************************************
 *
 *   linkhandler_instance_button_callback
 *
 *
 *
 */

static void
linkhandler_instance_button_callback (GtkWidget *widget, gpointer data)
{
	const gchar *errmsg = NULL;
	gchar       *name = NULL;
	gchar       *hashstr = NULL;
	gchar       *editedname = NULL;
	guint        size = 0;

	if (data)
	{
		if (linkhandler_entry)
		{
			editedname = gtk_editable_get_chars (GTK_EDITABLE(linkhandler_entry),0,-1);
			/* link should still be valid, but better check again */
			if (editedname)
			{
				if (misc_check_if_ed2klink_is_valid(linkhandler_ed2klink,&name,&size,&hashstr))
				{
					if ((name)&&(hashstr)&&(size>0))
					{
						FILE *outpipe = fopen ((gchar*)data, N_("w"));
						if (outpipe)
						{
							fprintf (outpipe, N_("ed2k://|file|%s|%u|%s|/\n\n"), editedname, size, hashstr);
							fclose(outpipe);
#ifndef FREEBSD
						} else errmsg = _(" Could not send link to pipe - open() failed. ");
#else
						} else errmsg = N_(" Could not send link to pipe - open() failed. \n Probably the pipe doesn't exist, because of the FreeBSD pipe bug. \n http://sourceforge.net/tracker/index.php?func=detail&aid=656923&group_id=61752&atid=498271 \n please fix this if you can, I don't have a FreeBSD system! ");
#endif
					} else errmsg = N_("link ok, but fields bad ?!");
				} else errmsg = N_("link turned sour ?!");
			} else errmsg = N_("editedname==NULL ?!");
		} else errmsg = N_("linkhandler==NULL ?!");
	} else errmsg = N_("data==NULL ?!");

	G_FREE(data);	/* was a g_strdup() of the pipe path */
	G_FREE(name);
	G_FREE(hashstr);
	G_FREE(editedname);

	if (!errmsg)
	{
		gtk_widget_destroy(linkhandler_window);
		return;
	}

	g_return_if_fail (linkhandler_statuslabel!=NULL);

	gtk_label_set_text(GTK_LABEL(linkhandler_statuslabel), errmsg);
	gtk_widget_show(linkhandler_statuslabel);
	gtk_widget_grab_focus (linkhandler_entry);
}


/******************************************************************************
 *
 *   linkhandler_remember_link_button_callback
 *
 *
 */

static void
linkhandler_ask_buttons_callback (GtkWidget *widget, gpointer data)
{
	const gchar *errmsg = NULL;
	gchar       *name = NULL;
	gchar       *hashstr = NULL;
	gchar       *editedname = NULL;
	guint        size = 0;
	guint        action = GPOINTER_TO_UINT(data);
	
	if (linkhandler_entry)
	{
		editedname = gtk_editable_get_chars (GTK_EDITABLE(linkhandler_entry),0,-1);
		/* link should still be valid, but better check again */
		if (editedname)
		{
			if (misc_check_if_ed2klink_is_valid(linkhandler_ed2klink,&name,&size,&hashstr))
			{
				if ((name)&&(hashstr)&&(size>0))
				{
					gchar *link = g_strdup_printf(N_("ed2k://|file|%s|%u|%s|/"), editedname, size, hashstr);
					if (link)
					{
						if(!linkhandler_append_link_to_pending_file(link))
						{
							errmsg = _(" could not write to file ~/.ed2k_gui/pending.txt to remember link. ");
						}
						g_free(link);
					} else errmsg = N_("g_strdup_printf() failed ?!");
				} else errmsg = N_("link ok, but fields bad ?!");
			} else errmsg = N_("link turned sour ?!");
		} else errmsg = N_("editedname==NULL ?!");
	} else errmsg = N_("linkhandler==NULL ?!");

	G_FREE(name);
	G_FREE(hashstr);
	G_FREE(editedname);

	if (!errmsg)
	{
		/* check whether the chosen action should become our default action */
		if (linkhandler_checkbutton_make_this_default_action)
		{
			gboolean checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linkhandler_checkbutton_make_this_default_action));
			
			if ((checked) && action != option_linkhandler_what_to_do_if_no_GUI_running)
			{
				option_linkhandler_what_to_do_if_no_GUI_running = action;
				write_gui_options();
			}
		}
		
		if (action == LINKHANDLER_IF_NO_GUI_START_GUI)
		{
			linkhandler_main_run_gui_override_flag = TRUE;
		}
		gtk_widget_destroy(linkhandler_window);
		return;
	}

	g_return_if_fail (linkhandler_statuslabel!=NULL);
	gtk_label_set_text(GTK_LABEL(linkhandler_statuslabel), errmsg);
	gtk_widget_show(linkhandler_statuslabel);
	gtk_widget_grab_focus (linkhandler_entry);
}


/******************************************************************************
 *
 *   linkhandler_create_no_gui_running_buttons
 *
 *   creates a button box that asks the user what he/she 
 *    wants to do if no running GUI has been found.
 *
 */
 
static GtkWidget *
linkhandler_create_no_gui_running_buttons(void)
{
	GtkWidget	*box = NULL;
	GtkWidget	*vbox = NULL;
	GtkWidget *button_cancel = NULL;
	GtkWidget *button_start_gui = NULL;
	GtkWidget *button_remember_link = NULL;

	vbox = gtk_vbox_new(FALSE, 5);	
	g_return_if_fail (vbox!=NULL);
	gtk_widget_show (vbox);
	
	box = gtk_hbutton_box_new();
	g_return_if_fail (box!=NULL);
	gtk_widget_show (box);

	new_button_pack_and_signal_connect ( &button_start_gui,
	                                     box,
	                                     _(" start GUI now "),
	                                     linkhandler_ask_buttons_callback,
	                                     GUINT_TO_POINTER(LINKHANDLER_IF_NO_GUI_START_GUI),
	                                     1);

	new_button_pack_and_signal_connect ( &button_remember_link,
	                                     box,
	                                     _(" remember link for later "),
	                                     linkhandler_ask_buttons_callback,
	                                     GUINT_TO_POINTER(LINKHANDLER_IF_NO_GUI_REMEMBER_AND_QUIT),
	                                     1);

	new_button_pack_and_signal_connect ( &button_cancel,
	                                     box,
	                                     _(" cancel "),
	                                     linkhandler_destroy,
	                                     NULL,
	                                     1);
	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 15);

	add_new_check_button_to_box (vbox, &linkhandler_checkbutton_make_this_default_action, _(" make this the default action and do not ask in the future. "));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linkhandler_checkbutton_make_this_default_action), FALSE);
	
	return vbox;
}


/******************************************************************************
 *
 *   linkhandler_create_buttons
 *
 *   creates a horizontal packing box with buttons that give you
 *   a choice which GUI instance to send the link to.
 *
 */


static GtkWidget *
linkhandler_create_buttons (void)
{
	GtkWidget	*hbox = NULL;

	hbox = gtk_hbutton_box_new();
	g_return_if_fail (hbox!=NULL);
	gtk_widget_show (hbox);

	if (g_slist_length(linkhandler_pipes_found) == 1)
	{
		GtkWidget *button;
		new_button_pack_and_signal_connect ( &button,
		                                     hbox,
		                                     _("    OK    "),
		                                     linkhandler_instance_button_callback,
		                                     (gpointer)g_strdup((gchar*)linkhandler_pipes_found->data),
		                                     1);
	} else {
		guint   counter = 0;
		GSList *node = linkhandler_pipes_found;
		while (node)
		{
			if (node->data)
			{
				gchar *basename = g_basename((gchar*)node->data);
				gchar *postfix = basename + 5;  /* 5 = strlen(N_("pipe.")) */
				if (basename)
				{
					gchar      buttontxt[64];
					GtkWidget *button;

					if (*(postfix-1)!=0x00)
					{
						g_snprintf(buttontxt,sizeof(buttontxt), _(" GUI #%s "), postfix);
					} else g_snprintf(buttontxt,sizeof(buttontxt),N_(" %s "), _(" main GUI "));

					new_button_pack_and_signal_connect (
						&button,
						hbox,
						buttontxt,
						linkhandler_instance_button_callback,
						(gpointer)g_strdup((gchar*)node->data),
						1
					);

				}
			}
			node = node->next;
		}
	}

	return hbox;
}


/******************************************************************************
 *
 *   linkhandler_delete_event
 *
 *
 *
 */

static gint
linkhandler_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	return FALSE;	/* window can be destroyed */
}


/******************************************************************************
 *
 *   linkhandler_delete_event
 *
 *
 *
 */

static void
linkhandler_destroy (GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}


/******************************************************************************
 *
 *   linkhandler_create_mainwindow
 *
 *
 *
 */

#define LHWIN_SPACING	15

static void
linkhandler_create_mainwindow (const gchar *ed2klink)
{
	GtkWidget	*vbox;
	GtkWidget	*label;
	GtkWidget	*ruler;
	GtkWidget	*hbox;
	gchar    	*labeltxt;
	gchar    	*name = NULL;
	gchar    	*hashstr = NULL;
	guint    	 size = 0;

	g_return_if_fail(ed2klink!=NULL);

	linkhandler_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	g_return_if_fail (linkhandler_window!=NULL);
	gtk_widget_show (linkhandler_window);
	gtk_window_set_title (GTK_WINDOW(linkhandler_window), _("eDonkey2000 - link"));
	gtk_window_set_position (GTK_WINDOW(linkhandler_window), GTK_WIN_POS_CENTER);
	gtk_container_border_width (GTK_CONTAINER(linkhandler_window), LHWIN_SPACING);
	gtk_window_set_default_size (GTK_WINDOW(linkhandler_window), 320, 240);

	gtk_signal_connect (GTK_OBJECT (linkhandler_window), N_("destroy"),
				GTK_SIGNAL_FUNC(linkhandler_destroy), NULL);
	gtk_signal_connect (GTK_OBJECT (linkhandler_window), N_("delete_event"),
				GTK_SIGNAL_FUNC(linkhandler_delete_event), NULL);

	vbox = gtk_vbox_new(FALSE,5);
	g_return_if_fail (vbox!=NULL);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER(linkhandler_window), vbox);

	if (linkhandler_error_message == NULL)
	{
		if (!misc_check_if_ed2klink_is_valid (ed2klink, &name, &size, &hashstr))
		{
			labeltxt = g_strdup_printf (N_(" received invalid ed2k-link:\n\n %s "), ed2klink);
		} 
		else 
		{
			labeltxt = g_strdup_printf (N_(" %s \n\n %s "), name, human_size(size));
		}
	}
	else
	{
		labeltxt = g_strdup(linkhandler_error_message);
	}
	
	g_return_if_fail (labeltxt!=NULL);

	label = gtk_label_new(labeltxt);
	g_return_if_fail (label!=NULL);
	gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
	gtk_widget_show (label);
	gtk_container_border_width (GTK_CONTAINER(linkhandler_window), 5);
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, LHWIN_SPACING);

	/* valid ed2k-link => show entry widget and buttons box, otherwise only CANCEL button */
	if ((name) && (hashstr) && size>0)
	{
		linkhandler_entry = gtk_entry_new();
		g_return_if_fail (linkhandler_entry!=NULL);
		gtk_widget_show(linkhandler_entry);
		gtk_box_pack_start(GTK_BOX(vbox), linkhandler_entry, TRUE, TRUE, LHWIN_SPACING);
		gtk_entry_set_text(GTK_ENTRY(linkhandler_entry), name);

		gtk_widget_grab_focus (linkhandler_entry);

		ruler = gtk_hseparator_new();
		g_return_if_fail (ruler!=NULL);
		gtk_widget_show (ruler);
		gtk_box_pack_start(GTK_BOX(vbox), ruler, FALSE, FALSE, LHWIN_SPACING);

		if (linkhandler_pipes_found != NULL)
		{
			hbox = linkhandler_create_buttons();
		}
		else
		{
			hbox = linkhandler_create_no_gui_running_buttons();
		}
		
		g_return_if_fail (hbox!=NULL);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, LHWIN_SPACING);
	} 
	else
	{
		GtkWidget *cancelbutton = gtk_button_new_with_label(_(" OK. Sorry, won't happen again. "));
		gtk_widget_show(cancelbutton);
		(void) gtk_signal_connect (GTK_OBJECT (cancelbutton),
		                           N_("clicked"),
		                           GTK_SIGNAL_FUNC (linkhandler_destroy),
		                           NULL);
		gtk_box_pack_start(GTK_BOX(vbox), cancelbutton, FALSE, FALSE, LHWIN_SPACING);
	}

	linkhandler_statuslabel = gtk_label_new(N_(" "));
	gtk_box_pack_start(GTK_BOX(vbox), linkhandler_statuslabel, TRUE, TRUE, LHWIN_SPACING);
	gtk_widget_hide (linkhandler_statuslabel);

	g_free(labeltxt);
}


/******************************************************************************
 *
 *   linkhandler_append_link_to_pending_file
 *
 *   appends an ed2k-link to the file ~/.ed2k_gui/pending_links.txt
 *     no main GUI window is needed
 *
 *   returns TRUE if everything went fine, and FALSE on error
 *
 */

 static gboolean 
 linkhandler_append_link_to_pending_file (const gchar *link)
 {
 	const gchar *fn;
	FILE        *fout;
	
	g_return_val_if_fail (link != NULL, FALSE);
	
	fn = complete_pref_path (N_("pending_links.txt"), FALSE);
	g_return_val_if_fail (fn != NULL, FALSE);
	
	fout = fopen (fn, N_("a"));
	if (!fout)
		return FALSE;
	
	fprintf (fout, N_("%s\n"), link);
	fclose(fout);
	
	return TRUE;
 }

 
/******************************************************************************
 *
 *   linkhandler_check_for_pending_links_yes_callback
 */
 
static void
linkhandler_check_for_pending_links_yes_callback (GtkWidget *widget, gpointer data)
{
	const gchar *pendingfn = complete_pref_path (N_("pending_links.txt"), FALSE);
	
	if (pendingfn)
		unlink(pendingfn);
	
	if (linkhandler_pending_links_array)
	{
		gchar **line = linkhandler_pending_links_array;
		
		while (*line)
		{
			if (**line)
			{
				gchar   *fn = NULL;
				gchar   *hashstr = NULL;
				guint32  size = 0;
				if (misc_check_if_ed2klink_is_valid(*line,&fn,&size,&hashstr))
				{
					if ((fn)&&(hashstr)&&(size>0))
					{
						guint8 hash[16];
						memcpy(hash,hash_str_to_hash(hashstr),16);
						status_message_blue (_("GUI: adding pending link '%s' to downloads.\n"), *line);
						send_new_download (hash, size, fn); /* download immediately! */
					} else status_warning (_("GUI: ignored pending link '%s' (not valid)\n"), *line);
				} else status_warning (_("GUI: ignored pending link '%s' (not valid)\n"), *line);
			}
			line++;
		}
		
		g_strfreev(linkhandler_pending_links_array);
		linkhandler_pending_links_array = NULL;
	}

	/* Destroy the dialog widget (note that 'widget' is the button here */
	if (data)
	{
		gtk_widget_destroy (GTK_WIDGET (data));
	}

}

/******************************************************************************
 *
 *   linkhandler_check_for_pending_links_no_callback
 */
 
static void
linkhandler_check_for_pending_links_no_callback (GtkWidget *widget, gpointer data)
{
	if (linkhandler_pending_links_array)
		g_strfreev(linkhandler_pending_links_array);
		
	status_message_blue (_("If you want to make the GUI forget the remembered links, you need to delete the file ~/.ed2k_gui/pending_links.txt"));

	/* Destroy the dialog widget (note that 'widget' is the button here */
	if (data)
	{
		gtk_widget_destroy (GTK_WIDGET (data));
	}

}


 
 /******************************************************************************
 *
 *   linkhandler_check_for_pending_links
 *
 *   checks whether the file ~/.ed2k_gui/pending_links.txt
 *     exists, and if yes pops up a dialog asking whether
 *     those pending downloads should be added now.
 *
 */

void
linkhandler_check_for_pending_links (void)
{
	const gchar *fn;
	gchar       **lines, **line;
	gchar       *msg;
	guint        num = 0;
	
	fn = complete_pref_path (N_("pending_links.txt"), FALSE);
	g_return_if_fail (fn != NULL);
	
	lines = misc_get_array_of_lines_from_textfile(fn);
	
	if (!lines)
		return;
		
	if (!lines[0])
		return;

	line = lines;
	while (*line)
	{
		if (misc_check_if_ed2klink_is_valid(*line, NULL, NULL, NULL))
		{
			num++;
		} 
		line++;
	}

	if (num == 0)
	{
		status_message_blue (_("GUI: Removing file %s with pending ed2k-links. It contains no valid pending ed2k-links.\n"), fn);
		(void) unlink (fn);
		g_strfreev(lines);
		return;
	}
			
	msg = g_strdup_printf (_("There are %u pending ed2k-links.\n\nDo you want to add them to the download list now?"), num);
	g_return_if_fail (msg != NULL);	
	
	yesno_dialog (	_("pending ed2k-links"),
		msg,
		_(" Yes, add them to downloads "),
		_(" No "),
		linkhandler_check_for_pending_links_yes_callback,
		linkhandler_check_for_pending_links_no_callback
	);
 
	linkhandler_pending_links_array = lines;
	
	g_free(msg);
	
}
 

/******************************************************************************
 *
 *   linkhandler_send_link_to_all_pipes
 *
 *   sends the given link to all found pipes and
 *     frees the list of found pipes.
 */

static void
linkhandler_send_link_to_all_pipes (const gchar *link)
{
	GSList *node = linkhandler_pipes_found;

	g_return_if_fail (link != NULL);

	while (node)
	{
		if (node->data)
		{
			FILE *outpipe = fopen ((gchar*)node->data, N_("w"));
			if (outpipe)
			{
				fprintf (outpipe, N_("%s\n"), link);
				fclose(outpipe);
			}
			g_free(node->data);
			node->data = NULL; 
		}
		node = node->next;
	}

	g_slist_free(linkhandler_pipes_found);
	linkhandler_pipes_found = NULL;
}

/******************************************************************************
 *
 *   linkhandler_main
 *
 *   returns TRUE if it dealt with the link and 
 *     no main GUI window is needed
 *
 *   returns FALSE if a main GUI window is needed
 *
 */

gboolean
linkhandler_main (const gchar *ed2klink)
{
	if (ed2klink == NULL)
		return FALSE;

	linkhandler_ed2klink = g_strdup(ed2klink);
	misc_strings_unescape_url (linkhandler_ed2klink);
	
	if (linkhandler_check_for_existing_GUI_instances() == -1)
		linkhandler_error_message = N_("check for existing GUI instances FAILED (see console output).");

	if (linkhandler_error_message == NULL)
	{
		/* server link? behave differently, these are not so important */
		if (g_strncasecmp(linkhandler_ed2klink,N_("ed2k:"),5) == 0
			&& strstr(linkhandler_ed2klink,N_("|server|")) != NULL)
		{
			if (linkhandler_pipes_found == NULL)
			{
				linkhandler_error_message = N_(" Received ed2k server link, but did not find any running GUIs. ");
			}
			else
			{
				linkhandler_send_link_to_all_pipes(linkhandler_ed2klink);
				G_FREE(linkhandler_ed2klink);
				return TRUE; /* done, no main GUI window needed */
			}
		}
		else
		{
			if (linkhandler_pipes_found == NULL)
			{
				switch (option_linkhandler_what_to_do_if_no_GUI_running)
				{
					case LINKHANDLER_IF_NO_GUI_ASK:
					break;
			
					case LINKHANDLER_IF_NO_GUI_START_GUI:
					{
						G_FREE(linkhandler_ed2klink);
						return FALSE;		/* return now and run main GUI */
					}
			
					case LINKHANDLER_IF_NO_GUI_REMEMBER_AND_QUIT:
					{
						if(linkhandler_append_link_to_pending_file(linkhandler_ed2klink))
						{
							G_FREE(linkhandler_ed2klink);
							return TRUE;	/* quit now, no main GUI window */
						} 
						linkhandler_error_message = _(" could not write to file ~/.ed2k_gui/pending.txt to remember link. ");
					}
					break;
			
					default: 
					{
						linkhandler_error_message = N_("FIXME: unknown option_linkhandler_what_to_do_if_no_GUI_running value.");
					}
				}
			}
		}
	}
			
	linkhandler_create_mainwindow(linkhandler_ed2klink);

	gtk_main ();

	G_FREE(linkhandler_ed2klink);

	if (linkhandler_main_run_gui_override_flag)
	{
		return FALSE;	/* we remembered the link, but still want the GUI to start NOW */
	}
	
	return TRUE; /* quit now, no main GUI window */
}




