/***************************************************************************
                          ramfile.c  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "ramfile.h"
#include <stdio.h>
#include <string.h>	// for memcpy, strlen
#include <errno.h>

ramfile				*ramfile_now_active = NULL;
static	GSList		*ramfiles_active = NULL;


// ramfile_new
//
// creates new ramfile
//   len: initial len
//
// returns pointer if successful, otherwise NULL

//static ncount = 0;
//static fcount = 0;

ramfile *
ramfile_new (guint len)
{
	ramfile *rf = g_new (ramfile,1);
//	g_print ("%s (%u)\n", __FUNCTION__, len);
	g_return_val_if_fail (rf!=NULL, NULL);
	rf->data = g_byte_array_new ();
	g_return_val_if_fail (rf->data!=NULL, NULL);
	rf->data = g_byte_array_set_size (rf->data, len);
	g_return_val_if_fail (rf->data!=NULL, NULL);
	rf->pos = 0;
	rf->size = len;
//	ncount++;
//	g_print ("ramfile - new=%u - freed=%u\n",ncount,fcount);
	return rf;
}


// ramfile_new_from_file
//
// Reads a file from disk into a ramfile and returns it or NULL on error

ramfile *
ramfile_new_from_file (const gchar *filename)
{
	FILE	*inf;
	ramfile *rf;

	g_return_val_if_fail (filename!=NULL, NULL);

	inf = fopen (filename, N_("rb"));
	if (!inf)
		return NULL;

	rf = ramfile_new(0);
	g_return_val_if_fail (rf!=NULL,NULL);

	while ((!feof(inf))&&(!ferror(inf)))
	{
		guint8	 buf[1024];
		size_t	 len;
		len = fread(buf,1,sizeof(buf),inf);
		g_return_val_if_fail (len>=0,NULL);
		if (len>0)
			ramfile_write(rf,buf,len);
	}

	if (ferror(inf))
	{
		ramfile_free(rf);
		rf = NULL;
	}

	fclose (inf);

	return rf;
}

void
ramfile_free (ramfile *rf)
{
	g_return_if_fail (rf!=NULL);
//	if (rf==NULL) return;
	g_return_if_fail (rf->data!=NULL);
	g_byte_array_free(rf->data, TRUE);
	g_free (rf);
//	fcount++;
//	g_print ("ramfile - new=%u - freed=%u\n",ncount,fcount);
}

// ramfile_read8
//
// reads in one byte
// returns 0xFF on error

guint8
ramfile_read8 (ramfile *rf)
{
	g_return_val_if_fail (rf!=NULL, 0xff);
	g_return_val_if_fail (rf->data!=NULL, 0xff);
	if (rf->pos > rf->size) return 0xff;
	return (rf->data)->data[rf->pos++];
}

// ramfile_read16
//
// reads in two bytes
// returns 0xFFFF on error

guint16	
ramfile_read16 (ramfile *rf)
{
	guint16	ret;
	g_return_val_if_fail (rf!=NULL, 0xffff);
	g_return_val_if_fail (rf->data!=NULL, 0xffff);
	if (rf->pos+2 > rf->size) return 0xffff;
	memcpy (&ret, (rf->data)->data+rf->pos, 2);
	rf->pos += 2;
	return GUINT16_FROM_LE(ret);
}

// ramfile_read32
//
// reads in four bytes
// returns 0xFFFFFFFF on error

guint32	
ramfile_read32 (ramfile *rf)
{
	gint32 ret;
	g_return_val_if_fail (rf!=NULL, 0xffffffff);
	g_return_val_if_fail (rf->data!=NULL, 0xffffffff);
	if (rf->pos+4 > rf->size) return 0xffffffff;
	memcpy (&ret, (rf->data)->data+rf->pos, 4);
	rf->pos += 4;
	return GUINT32_FROM_LE(ret);
}


// ramfile_readfloat
//
// reads in four bytes and returns a float value
// XXX - returns -999.99 on error (not very good?!)

gfloat
ramfile_readfloat (ramfile *rf)					// endianess?!?!? XXX
{
	gfloat	ret;
	g_return_val_if_fail (rf!=NULL, -999.99);
	g_return_val_if_fail (rf->data!=NULL, -999.99);
	if (rf->pos+4 > rf->size) return -999.99;
	memcpy (&ret, (rf->data)->data+rf->pos, 4);
	rf->pos += 4;
	g_return_val_if_fail (sizeof(gfloat)==4, -999.99);
	return ret;
}


// ramfile_writefloat
//
// writes a four-byte float value - XXX - not portable?

void
ramfile_writefloat (ramfile *rf, gfloat val)
{
//	guint32	val_le = GINT32_TO_LE(val);			// XXX ???
	g_return_if_fail (rf!=NULL);
//	ramfile_write (rf, (guint8*)&val_le, 4);
	ramfile_write (rf, (guint8*)&val, 4);
}


// ramfile_write8
//

void
ramfile_write8 (ramfile *rf, const gint8 val)
{
	gint8	val_le = val;
	ramfile_write (rf, (guint8*)&val_le, 1);
}

// ramfile_write16
//

void
ramfile_write16 (ramfile *rf, const gint16 val)
{
	gint16	val_le = GINT16_TO_LE(val);
	ramfile_write (rf, (guint8*)&val_le, 2);
}

// ramfile_write32
//

void
ramfile_write32 (ramfile *rf, const gint32 val)
{
	gint32	val_le = GUINT32_TO_LE(val);
	ramfile_write (rf, (guint8*)&val_le, 4);
}

gboolean
ramfile_seek (ramfile *rf, gint inc, gint seektype)
{
	gint newpos;
	g_return_val_if_fail (rf!=NULL, FALSE);
	g_return_val_if_fail (rf->data!=NULL, FALSE);
	switch (seektype)
	{
		case RAMFILE_SEEK_FROM_HERE:
		{
			newpos = rf->pos+inc;
		}
		break;

		case RAMFILE_SEEK_FROM_START:
		{
			newpos = inc;
		}
		break;

		case RAMFILE_SEEK_FROM_END:
		{
			newpos = rf->size - inc;
		}
		break;

		default:	return FALSE;		// wrong seek type
	}	

	if (newpos > rf->size) return FALSE;		// can't seek to pos that doesn't exist
	rf->pos = newpos;
	return TRUE;
}


// ramfile_read
//
// reads in bytes from ramfile
//   rf:  ramfile
//   buf: buffer where data is stored to
//        if NULL, buffer will be allocated (0-terminated) and returned (user must free it later)
//
// returns allocated buffer if buf was set NULL, otherwise buf (or NULL if len=0)
//

guint8 *
ramfile_read (ramfile *rf, guint8 *buf, gint len)
{
	g_return_val_if_fail (rf!=NULL, NULL);
	// CHECK: shouldn't we return buf here instead 0 ?
	if (len==0) return NULL;
	if (rf->pos+len > rf->size) return NULL;
	if (!buf) buf = (guint8*) g_malloc0 (len+1);
	g_return_val_if_fail (buf!=NULL, NULL);
	memcpy (buf, (rf->data)->data+rf->pos, len);
	rf->pos += len;
	return buf;
}

// ramfile_peek
//
// reads in bytes from ramfile from position pos, without changing the current position
//   rf:  ramfile
//   buf: buffer where data is stored to
//        if NULL, buffer will be allocated (0-terminated) and returned (user must free it later)
//
// returns allocated buffer if buf was set NULL, otherwise buf (or NULL if len=0)
//

guint8 *
ramfile_peek (ramfile *rf, guint8 *buf, gint pos, gint len)
{
	g_return_val_if_fail (rf!=NULL, NULL);
	// CHECK: shouldn't we return buf here instead 0 ?
	if (len==0) return NULL;
	if (pos+len > rf->size) return NULL;
	if (!buf) buf = (guint8*) g_malloc0 (len+1);
	g_return_val_if_fail (buf!=NULL, NULL);
	memcpy (buf, (rf->data)->data+pos, len);
	return buf;
}


// ramfile_write
//
//

void
ramfile_write (ramfile *rf, guint8 *buf, gint len)
{
	g_return_if_fail (rf!=NULL);
	if (len==0) return;
	g_return_if_fail (buf!=NULL);
	// note: special case might be 2 bytes before end but 4 to write...
	if (rf->pos+len > rf->size)
	{
		g_byte_array_set_size (rf->data, rf->size+len+64); // 64, so save unnecessary allocs
		rf->size = rf->pos+len;		// only increase len if we write over/at the end
	}
	memcpy ((rf->data)->data+rf->pos, buf, len);
	rf->pos += len;
}

// ramfile_write_i4_string
//
//

void
ramfile_write_i4_string (ramfile *rf, const gchar *string)
{
	g_return_if_fail (rf!=NULL);
	if (string)
	{
		guint16	len = strlen(string);
		ramfile_write16 (rf, len);
		ramfile_write (rf, (guint8*) string, len);
	} else ramfile_write_i4_string (rf, N_(""));
}


/******************************************************************************
 *
 *  ramfile_read_i4_string
 *
 *  Reads an i4string ( 2 bytes string length, x bytes characters )
 *    from ramfile, and returns it as a newly-allocated
 *    and zero-terminated string.
 *
 *  Returns string or NULL on error
 *
 *  -=-=-=- CALLER NEEDS TO FREE RETURNED STRING -=-=-=-
 *
 */

gchar *
ramfile_read_i4_string (ramfile *rf)
{
	gchar  	*i4str;
	guint16	 len;

	g_return_val_if_fail (rf != NULL, NULL);

	len = ramfile_read16(rf);

	if (len == 0)
	{
		return g_strdup(N_(""));
	}

	if (len == (guint16) -1)
	{
		return NULL;
	}

	/* a string longer than 2000 chars? unlikely */
	g_return_val_if_fail (len < 2000, NULL);

	/* if char is NOT one byte */
	if (sizeof(gchar) != sizeof(guint8))
	{
		GString *s = g_string_sized_new (len);
		guint    i;

		for (i=0; i<len; i++)
		{
			s = g_string_append_c (s, (gchar) read_8());
		}
		i4str = s->str;
		g_string_free (s, FALSE);	/* don't free character data, only GString structure */
	}
	else
	{
		i4str = g_new0 (gchar,len+1);
		g_return_val_if_fail (i4str != NULL, NULL);

		ramfile_read(rf,(guint8*)i4str, len);
	}

	return i4str;
}

// ramfile_set_active
// ramfile_unset_active
//
// sets 'active' ramfile, so we can use macros w8_read(), w16_read() etc.
// tricky - we need a stack to implement this, because a callback routine could be called before
//		the routine setting up the callback finishes, and both could be using this thing

void
ramfile_set_active (ramfile *rf)
{
	// add active RF as first element
	ramfiles_active = g_slist_prepend (ramfiles_active, (gpointer) rf);
	ramfile_now_active = rf;
}

void
ramfile_unset_active (ramfile *rf)
{
	GSList	*node = ramfiles_active;
	g_return_if_fail (ramfiles_active!=NULL);
	g_return_if_fail ((ramfile*)ramfiles_active->data==rf);
	// remove first element
	ramfiles_active = g_slist_remove_link (ramfiles_active, node);
	g_slist_free_1(node);
	if (ramfiles_active)
	{
		ramfile_now_active = ramfiles_active->data;
	} else ramfile_now_active = NULL;
}

// ramfile_dump
//
// for debugging purposes
//
void
ramfile_dump (ramfile *rf, const gchar *infostring)
{
	gint i;
	g_return_if_fail (rf!=NULL);
	g_return_if_fail (infostring!=NULL);
	fprintf (stdout, N_("_____ ramfile dump - %s _____"), infostring);
	for (i=0; i < rf->size; i++)
	{
		if (i%16==0) fprintf (stdout, N_("\n%04x: "), i);
		// don't output this via our print handler...
		fprintf (stdout, N_("%02x "), (rf->data)->data[i]);
	}
	fprintf (stdout, N_("\n________________________________________\n"));
}


// ramfile_write_to_file
//
// writes the content of a ramfile to a file on disk
// returns TRUE if everything went OK, otherwise FALSE.

gboolean
ramfile_write_to_file (ramfile *rf, const gchar *fn)
{
	FILE		*f;
	guint8		*buf;
	guint		 size;
	gboolean	 ret = FALSE;

	g_return_val_if_fail (rf!=NULL, FALSE);
	g_return_val_if_fail (fn!=NULL, FALSE);

	size = ramfile_get_size(rf);
	buf = ramfile_get_buffer(rf);
	g_return_val_if_fail (buf!=NULL, FALSE);
	
	if ((f=fopen(fn, N_("wb"))))
	{
		if (fwrite(buf,size,1,f)<0)
			g_printerr (_("%s - fwrite() failed: %s\n"), __FUNCTION__, g_strerror(errno));
		else ret = TRUE;
		fclose(f);
	} else g_printerr (_("%s - fopen() to write failed: %s\n"), __FUNCTION__, g_strerror(errno));
	return ret;
}


