/***************************************************************************
                          spawn_core.c  -  description
                             -------------------
    begin                : Thu Oct 31 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "connect_gui.h"
#include "global.h"
#include "gui_flags.h"
#include "gui_options.h"
#include "guicoreconn.h"
#include "misc.h"
#include "ramfile.h"
#include "spawn_core.h"
#include "status_page.h"
#include "tag.h"

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

static gint		kill_core (void);
static gint		spawn_local_donkey (const gchar *path_program);


/* spawn_core_pref_met_has_username
 *
 * reads in the pref.met file from the directory where the donkey core is in,
 * and checks whether the pref.met file contains the same username/password
 * combination.
 *
 *
 * if the (gint*) clientport is not NULL, clientport will be set to the configured
 *   client port or to the default port
 *
 * returns
 * -1	on error
 * 0	no pref.met, no username/password, or username/password different
 * 1	if username/pw are the same
 *
 */

gint
spawn_core_pref_met_has_username (const gchar *dir, const gchar *username, const gchar *password, gint port, gint *clientport)
{
	guint8	 	 buf[16+6];
	GPtrArray	*taglist;
	NGS_tag		*tag;
	gchar		*fn;
	ramfile		*rf;
	gboolean	 usermatch = FALSE;
	gboolean	 passmatch = FALSE;
	gboolean	 portmatch = FALSE;

	g_return_val_if_fail (dir!=NULL, -1);
	g_return_val_if_fail (username!=NULL, -1);
	g_return_val_if_fail (password!=NULL, -1);
	g_return_val_if_fail (*username!=0x00, -1);
	g_return_val_if_fail (*password!=0x00, -1);

	/* does directory exist at all? (this case shouldn't happen!) */
	if (access(dir,F_OK|R_OK)!=0)
		return -1;

	fn = g_strdup_printf(N_("%s/pref.met"), dir);
	g_return_val_if_fail (fn!=NULL, FALSE);

	/* do we have a pref.met file at all? */
	if (access(fn,F_OK)!=0)
	{
		g_free(fn);
		return 0;
	}

	/* are we allowed to read it? */
	if (access(fn,R_OK)!=0)
	{
		g_free(fn);
		return -1;
	}

	rf = ramfile_new_from_file(fn);
	g_free(fn);

	if (!rf)
		return -1;

	if (ramfile_get_size(rf)<16+6+4+4)
		return 0;

	ramfile_set_pos(rf,0);
	ramfile_read(rf,buf,16+6);	/* skip user hash + other stuff */
	/* get first taglist - we don't need this stuff */
	taglist = taglist_read_from_ramfile(rf);
	if (taglist)
	{
		taglist_free(taglist);
		taglist = NULL;
	}
	taglist = taglist_read_from_ramfile(rf);
	if (!taglist)
		return 0;

	tag = taglist_find_tag_from_tagname (taglist, N_("adminName"));
	if ((tag)&&(tag->data))
		usermatch = (strcmp((gchar*)tag->data,username)==0);

	tag = taglist_find_tag_from_tagname (taglist, N_("adminPass"));
	if ((tag)&&(tag->data))
		passmatch = (strcmp((gchar*)tag->data,password)==0);

	tag = taglist_find_tag_from_tagname (taglist, N_("adminDoorPort"));
	if ((tag)&&(tag->data))
		portmatch = (GPOINTER_TO_UINT(tag->data) == port);


	/* set clientport to default port if not defined otherwise */

	if (clientport)
	{
		*clientport = 4662;
		tag = taglist_find_tag_from_tagname (taglist, TAGNAME_PORT_SHORT);
		if ((tag)&&(tag->data))
		{
			*clientport = GPOINTER_TO_UINT(tag->data);
		} 
	}

	taglist_free(taglist);

	if ((portmatch) && (usermatch) && (passmatch))
	{
		return 1;
	}

	return 0;
}



/* spawn_core_modify_pref_met
 *
 * returns FALSE on error, TRUE on success
 *
 */

gboolean
spawn_core_modify_pref_met (const gchar *dir, const gchar *user, const gchar *pass, guint aport, guint clientport)
{
	guint32		 id = 0;
	guint16		 port = 0;
	guint8	 	 hash[16];
	GPtrArray	*taglist0 = NULL;
	GPtrArray	*taglist = NULL;
	ramfile		*rf;
	guint		 i;
	gboolean	 ret;

	g_return_val_if_fail (dir!=NULL, FALSE);
	g_return_val_if_fail (user!=NULL, FALSE);
	g_return_val_if_fail (pass!=NULL, FALSE);
	g_return_val_if_fail (*user!=0x00, FALSE);
	g_return_val_if_fail (*pass!=0x00, FALSE);

	/* does directory exist at all? (this case shouldn't happen!) */
	if (access(dir,F_OK|X_OK)!=0)
		return FALSE;

	/* change into directory */
	if (chdir(dir)!=0)
		return FALSE;

	/* fill buffer with random values, just in case we don't have a pref.met file */
	for (i=0; i<16; i++)
		hash[i]=rand();

	/* do we have a pref.met file at all? */
	if (access(N_("pref.met"),F_OK)==0)
	{
		/* are we allowed to read AND write to it? */
		if (access(N_("pref.met"),F_OK|R_OK|W_OK)!=0)
			return FALSE;

		/* read in file */
		rf = ramfile_new_from_file(N_("pref.met"));
		if (!rf)
			return FALSE;

		if (ramfile_get_size(rf)<4+2+16+4+4)
		{
			ramfile_free(rf);
			return FALSE;
		}

		/* get ID, port and user hash */
		ramfile_set_pos(rf,0);
		id = ramfile_read32(rf);
		port = ramfile_read16(rf);
		ramfile_read(rf,hash,16);

		/* get first taglist (user meta) */
		taglist0 = taglist_read_from_ramfile(rf);

		/* get second taglist */
		taglist = taglist_read_from_ramfile(rf);
/*		if (taglist)
		{
			adminuser = taglist_find_tag_from_tagname (taglist, N_("adminName"));
			adminpass = taglist_find_tag_from_tagname (taglist, N_("adminPass"));
			adminport = taglist_find_tag_from_tagname (taglist, N_("adminDoorPort"));
			doorport = taglist_find_tag_from_tagname (taglist, TAGNAME_PORT_SHORT);
		}
*/		ramfile_free(rf);
	}

	rf = ramfile_new(4+2+16+4+4);
	g_return_val_if_fail (rf!=NULL, FALSE);

	ramfile_write32(rf,id);
	ramfile_write16(rf,port);
	ramfile_write(rf,hash,16);
	if (!taglist0)
	{
		NGS_tag *name = tag_new_string (TAGNAME_FILENAME_SHORT, N_("unix user"));
		NGS_tag *cport = tag_new_integer (TAGNAME_PORT_SHORT, 4662);
		taglist0 = g_ptr_array_new();
		if (name) g_ptr_array_add (taglist0, (gpointer) name);
		if (cport) g_ptr_array_add (taglist0, (gpointer) cport);
	}
	taglist_write_to_ramfile(taglist0,rf);

	if (!taglist)
		taglist = g_ptr_array_new();

	/* set the essential stuff to the right values */
	tag_addedit_string_tag (taglist, N_("adminName"), user, FALSE, FALSE);
	tag_addedit_string_tag (taglist, N_("adminPass"), pass, FALSE, FALSE);
	tag_addedit_integer_tag (taglist, N_("adminDoorPort"), aport, FALSE, FALSE);
	tag_addedit_integer_tag (taglist, TAGNAME_PORT_SHORT, clientport, FALSE, FALSE);


	/* now add some standard tags if they don't exist yet (otherwise core might crash)
	 * only add these default tags if they don't exist already, and don't change
     * their values if they already exist
     */
	if (1==1)
	{
		gchar *indir = g_strdup_printf (N_("%s/incoming"), dir);
		gchar *tempdir = g_strdup_printf (N_("%s/temp"), dir);
		g_return_val_if_fail (indir!=NULL,FALSE);
		g_return_val_if_fail (tempdir!=NULL,FALSE);
		tag_addedit_string_tag (taglist, N_("incoming"), indir, FALSE, TRUE);
		tag_addedit_string_tag (taglist, N_("temp"), tempdir, FALSE, TRUE);
		g_free(indir);
		g_free(tempdir);
	}

	tag_addedit_integer_tag (taglist, N_("pSize"), 0x2d000, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("maxNumUp"), 5, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("maxDown"), 5, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("pmAllow"), 0, FALSE, TRUE);
	tag_addedit_float_tag (taglist, N_("userMaxUpF"), 0.0, FALSE, TRUE);
	tag_addedit_float_tag (taglist, N_("userMaxDownF"), 0.0, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("maxCon"), 250, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("auto"), 0, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("saveCor"), 1, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("verifyCancel"), 1, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("trayCheck"), 1, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("servRemove"), 1, FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("userName"), N_(""), FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("password"), N_(""), FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("ipOver"), 0, FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("lang"), N_("default.str"), FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("lineDown"), 0, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("maxUpSpeed"), 5000, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("maxDownSpeed"), 5000, FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("pType"), 0, FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("pSName"), N_(""), FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("pUName"), N_(""), FALSE, TRUE);
	tag_addedit_string_tag (taglist, N_("pUPass"), N_(""), FALSE, TRUE);
	tag_addedit_integer_tag (taglist, N_("pUPort"), 0, FALSE, TRUE);
	
	taglist_write_to_ramfile(taglist,rf);

	/* okay, now we have our new pref.met, now save it */
	rename(N_("pref.met"), N_("pref.met.old"));
	ret = ramfile_write_to_file(rf, N_("pref.met"));
	ramfile_free(rf);

	return ret;
}



/* spawn_local_donkey
 *
 * daemon_spawn:
 * Do some standard tricks to detach from terminal
 * amongst others. Work in daemon mode
 * path_program is complete path to donkey binary
 *
 * Return codes:
 *  0 : forked fine (something may go wrong but so far fine)
 * -1 : cannot fork
 * -2 : no path_program string
 * -3 : cannot expand path
 * -4 : cannot access directory
 * -5 : cannot access program
 *
 */

static gint
spawn_local_donkey (const gchar *path_program)
{
	gchar			*parameter[] = { NULL, N_("-"), N_("!"), NULL } ;
	gchar			 resolved_path[PATH_MAX], dir_path[PATH_MAX];
	gint			 i;
	pid_t			 pid;
	struct rlimit	 rl;

	dir_path[0] = resolved_path[0] = '\0';

	if (path_program == NULL)
		return(-2);

	if (realpath(path_program, resolved_path) == NULL)
		return(-3);

	/* If you thread this program, g_dirname is not thread safe
	 * in most OSes
	 * strcpy is safe here because g_dirname never overflows
	 * PATH_MAX
	 */

	if (strcpy(dir_path, g_dirname(resolved_path)) == NULL)	// strdup->leaks
		return(-3);		/* low memory */

	if (access(dir_path, R_OK | X_OK) != 0)
		return(-4);		/* directory is not accessible */

	if (access(resolved_path, X_OK) != 0)
		return(-5);		/* binary is not accessible */

	if ((pid = fork()) == -1)
		return(-1);		/* cannot fork */

	/* parent continues */
	if (pid != 0)
	{
		sleep(1);		/* sleep a bit to avoid hammering */
		return(0);		/* forked fine */
	}

	/* 1st child */
	if ((pid = fork()) != 0)
		_exit(0);		/* parent terminates */

	/* 2nd child continues */
	(void)setsid();			/* become session leader */

	/* XXX - should we ignore HUP signal?
	 *		leave it up for donkey for now
	 *		signal (SIGHUP, SIG_IGN);	// ignore SIGHUP
	 */

	if ((pid = fork()) != 0)
		_exit(0);		/* 1st child terminate */

	/* 3rd child continues, we have a daemon */

	/* close all descriptors above 1st 3 standard ones
	 * required for donkey core spawning success
	 * this is very expensive since it can close as much as
	 * thousand descriptors
	 */
	getrlimit(RLIMIT_NOFILE, &rl);
	for (i = rl.rlim_max - 1; i >= 3; i--)
	{
		(void)close(i);
	}

	/* cd to donkey dir
	 * chdir() will work unless someone changed the dir after we tested it
	 */
	if (chdir(dir_path) != -1)	
	{
		/* If you thread this program, g_basename is not thread safe
		 * in most OSes
		 * exec donkey within a pseudo daemon container environment
		 */
		parameter[0] = g_basename(resolved_path);
		(void)execv(resolved_path, parameter);	/* strdup=>leaks */
		perror (N_("bad error in execv()"));
		while (1==1) sleep(1);	/* XXX don't know what to do here */
	}

	g_printerr (_("PANIC. donkey core binary became unavailable while working to spawn it despite all precautions. Try executing the GUI again. If this persists, re-install the donkey core binary."));
	_exit(-1);		/* BIG TROUBLE, something changed while
					 * we worked. EXIT with prejudice
					 */
}



/* kill_core
 *
 * kills core (with pid as in variable local_donkey_pid)
 *
 * returns -1 if there was a problem, otherwise 0
 */

static gint
kill_core (void)
{
	gint i;
	gchar *msg = (gchar*) alloca (256*sizeof(gchar));

	g_return_val_if_fail (msg!=NULL, -1);

	if (kill ((pid_t)local_donkey_pid, SIGKILL)==-1)
	{
		switch (errno)
		{
			case ESRCH:
			{
				/* check again: We might have had two cores and have been connected to
				 * second core (but got PID of first, as old cores don't tell us their PID)
				 * and first core is already gone, so we'd need to update the PID...
				 */
				guint new_pid = donkey_process_running_check ();

				g_printerr (_("kill() failed - PID %u does not exist anymore!?\n"), local_donkey_pid);

				if (new_pid!=local_donkey_pid && new_pid>0)
				{
					g_printerr (_("found another PID (%u) - trying again.\n"), new_pid);
					local_donkey_pid = new_pid;
					/* TODO: do we need to prevent infinite recursion here? */
					return kill_core();
				}
			}
			break;

			case EPERM:
			{
				g_printerr (_("kill() failed - we don't have permission to kill the donkey process.\n"));
			}
			break;

			default:
			{
				g_printerr (_("kill() failed - some other error (check console output).\n"));
				status_system_error_msg(_("kill() failed"));
			}
		}

		/* This should bring up 'reconnect' dialog or 'connect to' dialog */
		cleanup_after_disconnect();
		return -1;
	}

	/* TODO: should we send a SIGKILL to the pid+1 to pid+4 as well, just to make it
	 *       shut down quicker?
	 * wait a bit (call kill with signal 0 = send no signal but do error checking)
	 */
	i=0;
	while ( i<5 && kill((pid_t)local_donkey_pid, 0)==0  &&  errno!=ESRCH )
	{
		usleep(500);	/* sleep 0.5s */
		i++;
	}

	/* same check for at least one more thread to increase
	 * probability that they are all gone
	 */
	i=0;
	while ( i<3 && kill((pid_t)local_donkey_pid+1, 0)==0  &&  errno!=ESRCH )
	{
		usleep(500);	/* sleep 0.5s */
		i++;
	}

	return 0;
}




/* kill_and_restart_core
 *
 * kills (if dokill_flag>0) and (re-) starts core
 *
 * returns -1 if there was a problem, otherwise 0
 */

gint
kill_and_restart_core (gint dokill_flag)
{
	const gchar	*action = (dokill_flag==1) ? _("kill and restart the core") : _("start the core");
	gchar		*msg = (gchar*) alloca (256*sizeof(gchar));

	/*
	 *
	 * XXX - FINISH (/me later: finish what exactly?)
	 *
	 *
	 */

	g_print (N_("\n\nFINISH ME: %s\n\n"), __FUNCTION__);

	g_return_val_if_fail (local_donkey_pid>0, -1);	// we shouldn't have been called in the first place
	g_return_val_if_fail (msg!=NULL, -1);
	g_return_val_if_fail (action!=NULL, -1);
	g_return_val_if_fail (guicoreconn!=NULL, -1);

	/* TODO:
	 * - check if binary is defined and exists before killing old core
	 *   (otherwise show message box)
	 */
	status_message_blue (_("GUI: I'm about to %s... (pid=%u) (might take some seconds)\n"), action ,local_donkey_pid);

	/* now kill core */
	if (dokill_flag==1)
	{
		if (kill_core()==-1)
		{
			status_message_blue (N_("GUI: kill() failed (bug?)\n"));
			return -1;
		}
		status_message_blue (_("GUI: kill() successfull. Restarting core now... (will freeze the GUI for 3 seconds)\n"));
		/* update GUI */
		while (gtk_events_pending())
			gtk_main_iteration_do(FALSE);
		/* Wait 3 secs (this blocks the GUI). */
		sleep(3);
	}

	/* respawn core */
	if (!spawn_core_and_print_spawn_result_message(option_defaultdonkeypath))
	{
		g_printerr (_("Respawn failed for some reason.\n"));
		cleanup_after_disconnect();	/* show 'connect to'/reconnect dialog again */
		return -1;
	}

	/* is the connection structure not set up anymore for some reason? */
	if (!guicoreconn)
	{
		g_printerr (_("Re-spawned core. Won't reconnect because connection is not set (bug?)\n"));
		cleanup_after_disconnect();	/* show 'connect to'/reconnect dialog again */
		return -1;
	}

	status_message_blue (_("GUI: Re-spawned core. Will try to reconnect in a couple of seconds...\n"));

	/* respawn or reconnect succeeded => show our 'connect-to'/reconnect dialog */
	cleanup_after_disconnect();

	return 0;
}


/* spawn_core_listening_port_is_in_use
 *
 * checks whether the given tcp port is already in use
 * on the local box
 *
 * returns TRUE if yes, FALSE if not or on error
 *
 */

gboolean
spawn_core_listening_port_is_in_use (guint port)
{

/* don't know whether/how that stuff works under *BSD, MacOSX */
#ifdef LINUX

	gchar **tcpconns = misc_get_array_of_lines_from_textfile (N_("/proc/net/tcp"));
	gchar **line = tcpconns;
	
	if (!tcpconns)
		return FALSE;

	while (*line)
	{
		guint	num, port1, port2, status;
		gchar	ipstr1[64], ipstr2[64];
		if (6==sscanf(*line, N_("%u: %8[0-9a-fA-F]:%x %8[0-9a-fA-F]:%x %2x"), &num, ipstr1, &port1, ipstr2, &port2, &status))
		{
			/* is this a listening socket */
			if (status == 10)
			{
				/* is this the socket we wanted to check? */
				if (port1==port)
				{
					g_strfreev(tcpconns);
					return TRUE;	/* => socket is in use */
				}
			}
		}
		line++;
	}

	g_strfreev(tcpconns);

#endif

	return FALSE;
}



/******************************************************************************
 *
 *   spawn_core_print_spawn_result_message
 *
 *   spawns a local donkey
 *
 *   binpath is full path to donkey binary
 *
 *   returns TRUE if everything is okay, and FALSE if some error occurred.
 *
 */

gboolean
spawn_core_and_print_spawn_result_message (const gchar *binpath)
{
	gchar	*msg = NULL;
	gint	 ret;

	g_return_val_if_fail (binpath!=NULL,FALSE);

	ret = spawn_local_donkey(binpath);

	switch (ret)
	{
		case  0:	msg = _(" GUI: ok, spawned local donkey (hope it worked).");
		case -1:	if (!msg) msg = _(" GUI: cannot spawn donkey, not enough system resources.");
		case -2:	if (!msg) msg = _(" GUI: cannot spawn donkey, you have to enter the path to the donkey binary.");
		case -3:	if (!msg) msg = _("auto-spawn: path to binary does not exist.\n");
		case -4:	if (!msg) msg = _("Cannot resolve entered path to binary. You might have entered a directory instead of a path to a file");
		case -5:	if (!msg) msg = _(" GUI: cannot spawn donkey, you do not have exec permission for the donkey binary.");
		default:	if (!msg) msg = _("auto-spawn: check which are the valid return values for spawn_local_donkey function\n");
	}

	g_printerr (N_("%s"), msg);
	statusbar_extra_message (N_("%s"), msg);

	if (ret == -3)
	{
		ok_dialog (_(" Spawn local donkey "), msg);
	}

	if (ret>0 || ret<-5)
	{
		g_printerr (N_("****** in %s - exit(-1). Bye.\n"), __FUNCTION__);
		exit(-1);
	}

	return (ret==0);
}



