/*
*  status_page.c
*
*  populates status page and handles its functions
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "global.h"
#include "gui_options.h"
#include "icons.h"
#include "mainwindow.h"
#include "misc.h"
#include "notebook.h"
#include "status_page.h"
#include "statusbar.h"
#include "http_get.h"
#include <string.h>
#include <errno.h>

// global variables

static GtkWidget	*text = NULL;		// status text widget

// local
static void		 status_select_and_copy_ed2klinks_in_status_text (void);

#if (GTK_MAJOR_VERSION==1 && GTK_MINOR_VERSION<=2)
	static guint		 my_gtk_text_strrstr (GtkText *wtxt, const gchar *find, gint pos);
#endif

// functions


// gui_print_error_function
//
// This is our new error print handler (for g_printerr())
//
// If the window and the status text widget exist, it will output the error
//	to stderr and the status log (and thus also the status log file)
// If the window or the status text widget are not initialised yet, output will
//	go to stderr only

void
gui_print_error_function (const gchar *errstring)
{
	gchar *newline = N_("");
	g_return_if_fail (errstring!=NULL);
	g_return_if_fail (*errstring!=0x00);

	if (errstring[strlen(errstring)-1] != '\n')
		newline = N_("\n");

	status_warning (_("GUI: %s%s"), errstring, newline);
	fprintf (stderr, _("GUI: %s%s"), errstring, newline);
}




// status_msg_print_ed2klink
//
// Just prints an ed2k://|file| link to the status log
// Makes sure that the filename doesn't contain illegal characters.

void
status_msg_print_ed2klink (const gchar *filename, guint size, const guint8* hash)
{
	gchar	*fn, *pos;

	g_return_if_fail (filename!=NULL);
	g_return_if_fail (hash!=NULL);

	fn = g_strdup (filename);
	g_return_if_fail (fn!=NULL);

	pos = fn;
	while (*pos)
	{
		if (*pos=='|' || *pos=='\\' || *pos=='/' || *pos=='$'
			|| *pos=='\'' || *pos=='~' || *pos=='"' || *pos=='%')
		{
			*pos = ' ';
		}	
		pos++;
	}

	status_msg (N_("ed2k://|file|%s|%u|%s|/\n"), fn, size, hash_to_hash_str(hash));
	if (option_nolinkstolog)
	{
		statusbar_extra_message (_(" ed2k-link of this file written to status window and copied to the clipboard."));
	} else statusbar_extra_message (_(" ed2k-link of this file written to logfile 'gui_statuslog' and to status log."));
	g_free (fn);
	status_select_and_copy_ed2klinks_in_status_text ();
}


#if (GTK_MAJOR_VERSION==1 && GTK_MINOR_VERSION<=2)

// my_gtk_text_strrstr
//
// searches first occurence of string in text widget FROM THE END.
// Returns index position in text, or -1 if not found

static guint
my_gtk_text_strrstr (GtkText *wtxt, const gchar *find, gint pos)
{
	gint  len    = strlen(find);
	char *token  = alloca (len+1);
	gint  i,j;

	for (i=0; i<len; i++) token[i] = GTK_TEXT_INDEX(wtxt, pos+i);
	token[i]=0x00;
	while (pos>=0)
	{
		if (strcmp(token, find)==0) return pos;
		if (pos>0)
		{
			for (j=len-1; j>=0; j--) token[j] = token[j-1];		// hope tokens are short
			token[0] = GTK_TEXT_INDEX(wtxt, pos-1);
		}
		pos--;
	}
	return -1;
}

#endif

// status_select_and_copy_ed2klinks_in_status_text
//
// If the last line in the status text is an ed2k-link, this routine
//	selects all ed2k-links at the end of the text and copies them
//	to the clipboard.

static void
status_select_and_copy_ed2klinks_in_status_text (void)
{
#if ((GTK_MAJOR_VERSION == 2) || (GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 3))
	g_print ("***** GTK2 FIXME: %s\n", __FUNCTION__);
#else
	GtkText	*wtxt = GTK_TEXT(text);
	gint link, other, first;

	// (1) Check if last thing in status text is an ed2k-link:
	link  = my_gtk_text_strrstr (GTK_TEXT(text), N_("|/\n"), gtk_text_get_length(GTK_TEXT(text)));
	if ((gtk_text_get_length(GTK_TEXT(text)) - link)!=3)
	  return;


	// now see if there are links we want to select - only if link is last thing printed out!
	link  = my_gtk_text_strrstr (GTK_TEXT(text), N_("ed2k://"), gtk_text_get_length(GTK_TEXT(text)));
	first = -1;
	other = link;
	if (link>0)
	{
		while ((other=my_gtk_text_strrstr (GTK_TEXT(text), N_("|/\n"), other-1))>0)
		{
			if ((link-other)==3)
			{ 	// a link _directly_ before link
				first = my_gtk_text_strrstr (GTK_TEXT(text), N_("ed2k://"), other-1);
				if (first>0)
				{
					other = first;
					link  = first;
				}
			}
		}
	}

	// first link to last link if all uninterrupted lines of links
	if (first>0) link=first;	

	// if there are ed2k:// links at the end, select them automatically + copy to clipboard :)
	if (link>0)
	{
		// If gtk_editable_select_region() fails with
		// gdk_window_get_size() assertion window!=NULL failed!
		// on first insertion of ed2k-link into status window,
		// then the reason is that the text widget hasn't been realised
		// yet (I think). Switching to status page and back gets rid
		// of this problem, that's why we do that at the beginning.
		gtk_editable_select_region (&(wtxt->editable), link, -1);
		gtk_editable_copy_clipboard (&(wtxt->editable));
	}
#endif

}


// status_system_error_msg
//
// writes a status line in red to the statuslog, including the last error (errno must be set)
// 'GUI:' is automatically prepended

void
status_system_error_msg (const gchar *msg_text)
{
	status_warning (_("GUI: %s (%s)\n"), (msg_text!=NULL) ? msg_text: N_(""), g_strerror(errno));
}


// status_parse_error_msg
//
// prints a 'GUI: parse error in file X line Y: <line beginning> ...' message

void
status_parse_error_msg (const gchar *infile, gint lineno, const gchar *linestring)
{
	g_return_if_fail (infile!=NULL);
	g_return_if_fail (linestring!=NULL);
	status_msg (_("GUI: parse error in %s file line %d: %20s...\n"), infile, lineno, linestring);
}


// status_message_coloured
//
// Prints a message to the status window in a certain colour.
// Will take care to suppress and/or parse certain messages
// Will write the message to the status log file if not suppressed.

static void
status_message_coloured (GdkColor *colour, const gchar *msg)
{
	const gchar	*coltag = N_("");
	const gchar	*timestamp = N_("");
	gchar	 	*msgWithoutCR;
	guint		 msgWithoutCRLen;

	g_return_if_fail (colour!=NULL);
	g_return_if_fail (msg!=NULL);

	if (colour == &red) coltag = _("RED: ");
	if (colour == &blue) coltag = _("BLUE: ");
	write_to_logfile (N_("%s%s"), coltag, msg);

	if ((!window) || (!text))
		return;

	if ((option_logtimestamp) && *msg!='\n' && g_strncasecmp(msg,N_("ed2k://"),6)!=0)
	{
		timestamp = get_timestamp();
		gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, timestamp, strlen (timestamp));
	}

	/* output to status window */

	if (strlen(msg)>2 && msg[strlen(msg)-1]=='\n' && msg[strlen(msg)-2]=='\n')
	{
		gtk_text_insert (GTK_TEXT(text), NULL, colour, NULL, msg, strlen (msg)-1);
	}
	else
	{
		gtk_text_insert (GTK_TEXT(text), NULL, colour, NULL, msg, strlen (msg));
	}

	/* output to statusbar (stip off potential newlines at end) */
	msgWithoutCR = g_strdup(msg);
	g_return_if_fail(msgWithoutCR != NULL);
	(void) g_strdelimit(msgWithoutCR, N_("\n\t"), ' ');
	(void) g_strstrip(msgWithoutCR);
	statusbar_extra_message (N_(" %s"), msgWithoutCR);
	g_free(msgWithoutCR);
}


void
status_msg (const gchar *format, ...)
{
	GString	*output = NULL;
	gchar	*s_alloc = NULL;
	va_list	 ap;

	g_return_if_fail (format!=NULL);

	output = g_string_new("");
	g_return_if_fail (output!=NULL);

	va_start(ap, format);
	if (vasprintf(&s_alloc, format, ap)>=0)
	{
		output = g_string_append (output, s_alloc);
		free(s_alloc);	/* as vasprintf allocs it, we need to use free() and not g_free() */
		s_alloc = NULL;
	}
	va_end(ap);

	status_message_coloured (&black, output->str);
	g_string_free(output,TRUE);
}

void
status_warning (const gchar *format, ...)
{
	GString	*output = NULL;
	gchar	*s_alloc = NULL;
	va_list	 ap;

	g_return_if_fail (format!=NULL);

	output = g_string_new("");
	g_return_if_fail (output!=NULL);

	va_start(ap, format);
	if (vasprintf(&s_alloc, format, ap)>=0)
	{
		output = g_string_append (output, s_alloc);
		free(s_alloc);	/* as vasprintf allocs it, we need to use free() and not g_free() */
		s_alloc = NULL;
	}
	va_end(ap);

	status_message_coloured (&red, output->str);
	g_string_free(output,TRUE);
}

void
status_message_blue (const gchar *format, ...)
{
	GString	*output = NULL;
	gchar	*s_alloc = NULL;
	va_list	 ap;

	g_return_if_fail (format!=NULL);

	output = g_string_new("");
	g_return_if_fail (output!=NULL);

	va_start(ap, format);
	if (vasprintf(&s_alloc, format, ap)>=0)
	{
		output = g_string_append (output, s_alloc);
		free(s_alloc);	/* as vasprintf allocs it, we need to use free() and not g_free() */
		s_alloc = NULL;
	}
	va_end(ap);

	status_message_coloured (&blue, output->str);
	g_string_free(output,TRUE);
}




/* status_page_clear
 *
 * clear status window
 *
 */

void
status_page_clear (void)
{
	static gboolean license_shown = FALSE;
	if (text)
	{
		/* Here we can freeze the text, as it's the top of the page anyway and scrolling doesn't matter */
		gtk_text_freeze (GTK_TEXT(text));
		gtk_text_set_point (GTK_TEXT(text), (guint) 0);
		gtk_text_forward_delete(GTK_TEXT(text), gtk_text_get_length (GTK_TEXT(text)));
		status_warning (_("Welcome to this %s GUI for eDonkey2000 and Overnet.\n"), OP_SYSTEM_STRING);
		status_warning (N_("ed2k-gtk-gui version %s, Copyright (c) 2001-2003 Tim-Philipp Mller\n"), VERSION);
		if (!license_shown)
		{
			const gchar *dummy = _("***** TRANSLATORS: please refer to GNU website www.gnu.org for a version of the"
									"GNU license stuff in your language ****** (you do not need to translate this string)");
			license_shown = TRUE;
			status_warning (N_(" \n"));		/* need the space so timestamps don't look funny */
			status_warning (_("This program is free software; you can redistribute it and/or modify\n"));
			status_warning (_("it under the terms of the GNU General Public License as published by\n"));
			status_warning (_("the Free Software Foundation; either version 2 of the License, or\n"));
			status_warning (_("(at your option) any later version.\n"));
			status_warning (N_(" \n"));		/* need the space so timestamps don't look funny */
			status_warning (_("This program is distributed in the hope that it will be useful, but\n"));
			status_warning (_("without any warranty; without even the implied warranty of merchantability\n"));
			status_warning (_("or fitness for a particular purpose. See the GNU General Public License\n"));
			status_warning (_("for more details: http://www.gnu.org\n"));
			status_warning (N_(" \n"));		/* need the space so timestamps don't look funny */
			status_warning (_("By using this program you agree to be bound by the GNU General Public License.\n"));
		}
		status_warning (N_(" \n"));		/* need the space so timestamps don't look funny */
		status_msg     (_("It is written in plain C using the Gimp Toolkit (GTK+) and the\n"));
		status_msg     (_("gnet library. Project homepage: http://ed2k-gtk-gui.sourceforge.net\n"));
		status_msg     (N_(" \n"));		/* so the statusbar is empty */
		
		status_message_blue ( _("Click on the eDonkey/Overnet logo in the top left corner to access extra functions.\n"));
		status_message_blue ( _("Type 'help' in the top toolbar to see what you can do with it and how to use it.\n"));
		status_msg          (N_(" \n"));		/* so the statusbar is empty */
		
		gtk_text_thaw (GTK_TEXT(text));
	}
}




/* status_page_create
 *
 * create the status window notebook page 
 *
 */

GtkWidget *
status_page_create (void)
{
	GtkWidget *scrollwin;

	/* create non-editable text widget with word-wrap and add it to packing table */
	text = gtk_text_new (NULL, NULL);
	gtk_widget_show (text);
	gtk_text_set_editable  (GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap (GTK_TEXT(text), FALSE);
	status_page_clear();

	scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_widget_show(scrollwin);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(scrollwin), text);

	return scrollwin;
}
