/***************************************************************************
                          toolbar.c  -  description
                             -------------------
    begin                : Sat Nov 23 2002
    copyright            : (C) 2002 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "colors.h"
#include "icons.h"
#include "gui_flags.h"
#include "gui_options.h"
#include "guicoreconn.h"
#include "http_get.h"
#include "mainwindow.h"
#include "misc.h"
#include "misc_gtk.h"
#include "misc_strings.h"
#include "notebook.h"
#include "parse_message.h"
#include "search_page.h"
#include "send_command.h"
#include "servers_page.h"
#include "status_page.h"
#include "statusbar.h"
#include "toolbar.h"

#include <ctype.h>

#define TOOLBAR_MAX_SRV_NAME_LEN	30

/* local functions */

static GtkWidget	*toolbar_create_entry_field (void);
static GtkWidget	*toolbar_create_stats_box (void);
static void			 toolbar_go_button_clicked (GtkWidget *widget, gpointer data);
static void			 toolbar_server_icon_clicked (GtkWidget *widget, GdkEventButton *event, gpointer data);
static void			 toolbar_entry_clear (void);
static void			 toolbar_show_entry_help (void);
static void			 toolbar_do_website_search (const gchar *webengine, const gchar *websearch);
static gchar		*toolbar_sanitize_search_phrase (const gchar *search);
static void			 toolbar_show_entry_help_insert_coloured_text (GtkWidget *dialog, GtkWidget *helptext, GdkColor *colour, const gchar *helpstring);
static gchar		*toolbar_remove_junk_from_servername (const gchar *name);

static void			 toolbar_donkey_icon_popup_menu_about (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_popup_menu_say_thank_you (GtkWidget *widget, gpointer data);

static void			 toolbar_donkey_icon_popup_menu_shutdown_core (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_popup_menu_donot_shutdown_core (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_popup_menu_really_shutdown_core (GtkWidget *widget, gpointer data);

static void			 toolbar_donkey_icon_popup_menu_kill_and_restart_core (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_popup_menu_disconnect_core (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_popup_menu_url (GtkWidget *widget, gpointer data);
static void			 toolbar_donkey_icon_clicked (GtkWidget *widget, GdkEventButton *event, gpointer data);

/* local variables (these are here so we can set the style after window creation) */

static GSList		*toolbar_event_boxes = NULL;	/* so we can set the style */
static GtkWidget	*toolbar_label_upspeed = NULL;
static GtkWidget	*toolbar_label_downspeed = NULL;
static GtkWidget	*toolbar_label_srvname = NULL;
static GtkWidget	*toolbar_label_srvstats = NULL;
static GtkWidget	*toolbar_label_cons = NULL;


static GtkWidget	*overnetbox = NULL;			/* event box that contains overnet logo */
static GtkWidget	*donkeybox = NULL;			/* event box that contains edonkey logo */
static GtkWidget	*toolbar_handlebox = NULL;
static GtkWidget	*toolbar_ew = NULL;
static GtkWidget	*go_button = NULL;

static GtkWidget	*toolbar_hbox_stats = NULL;

static gchar		*toolbar_core_version_string = NULL;



/* toolbar_remove_junk_from_servername
 *
 * Takes a server name and removes all junk from it, so
 *  it doesn't take more space than necessary in the toolbar.
 *
 * Caller needs to free returned string.
 *
 */

static gchar *
toolbar_remove_junk_from_servername (const gchar *name)
{
	gint	 n1,n2,n3,n4;
	gchar	*newname, *pos, *match;

	g_return_val_if_fail (name!=NULL, NULL);

	/* empty string? */
	if (!*name)
		return NULL;

	/* is it an IP? we don't need to modify that. */
	if (sscanf(name,N_("%u.%u.%u.%u"),&n1,&n2,&n3,&n4)==4)
		return g_strdup(name);

	newname = g_strdup(name);
	g_return_val_if_fail (newname!=NULL, NULL);

	pos = newname;
	while (*pos)
	{
		if (   g_strncasecmp(pos,N_(".de"),3)==0
			|| g_strncasecmp(pos,N_(".vu"),3)==0
			|| g_strncasecmp(pos,N_(".fr"),3)==0
			|| g_strncasecmp(pos,N_(".com"),4)==0
			|| g_strncasecmp(pos,N_(".net"),4)==0
			|| g_strncasecmp(pos,N_(".org"),4)==0)
		{
			*pos = 0x00;
		}

		else if (!isalnum(*pos))
			*pos = ' ';

		else if (g_strncasecmp(pos,N_("http"),4)==0)
			strncpy (pos, N_("         "),4);

		else if (g_strncasecmp(pos,N_("www"),3)==0)
			strncpy (pos, N_("         "),3);

		else if (g_strncasecmp(pos,N_("server"),6)==0)
			strncpy (pos, N_("         "),6);

		else if (g_strncasecmp(pos,N_("serveur"),7)==0)
			strncpy (pos, N_("         "),7);

		else if (g_strncasecmp(pos,N_("sparen"),6)==0)
			strncpy (pos, N_("         "),6);

		else if (g_strncasecmp(pos,N_("verdienen"),9)==0)
			strncpy (pos, N_("         "),9);

		else if (g_strncasecmp(pos,N_("gewinnen"),8)==0)
			strncpy (pos, N_("         "),8);

		pos++;
	}

	if ((match=strstr(newname,N_(".to/"))))
	{
		gchar *newname_old = newname;
		newname = g_strdup(newname+4);
		g_free(newname_old);
	}
	else if ((match=strstr(newname,N_(".TO/"))))
	{
		gchar *newname_old = newname;
		newname = g_strdup(newname+4);
		g_free(newname_old);
	}

	g_strdelimit (newname, N_("_-|><.:;~[](){}+=#@/\\,!\"$%^&*"), ' ');

	/* remove leading and trailing spaces */
	g_strstrip (newname);	

	/* remove multiple spaces within the string */
	pos = newname;
	while (*pos)
	{
		if (*pos == ' ' && *(pos+1) == ' ')
		{
			gchar *nospace = pos+2;
			while (*nospace == ' ')
				nospace++;
			g_memmove(pos+1, nospace, strlen(nospace) + 1);
		}
		pos++;
	}

	return newname;
}



/* toolbar_set_core_version_string
 *
 *
 */

void
toolbar_set_core_version_string (const gchar *s)
{
	g_return_if_fail (s!=NULL);

	if (toolbar_core_version_string)
		g_free(toolbar_core_version_string);

	toolbar_core_version_string = g_strdup(s);
}


/* toolbar_set_ul_speed
 *
 *
 */

void
toolbar_set_ul_speed (gfloat ulspeed)
{
	gchar txt[32];

	g_return_if_fail (toolbar_label_upspeed != NULL);
	g_snprintf(txt, sizeof(txt), N_("%3.1f  "), ulspeed);
	gtk_label_set_text(GTK_LABEL(toolbar_label_upspeed), txt);
}


/* toolbar_set_dl_speed
 *
 *
 */

void
toolbar_set_dl_speed (gfloat dlspeed)
{
	gchar txt[32];
	g_return_if_fail (toolbar_label_downspeed != NULL);
	g_snprintf(txt, sizeof(txt), N_("%3.1f "), dlspeed);
	gtk_label_set_text(GTK_LABEL(toolbar_label_downspeed), txt);
}


/* toolbar_set_server_data
 *
 * pass NULL,0,0 to set 'not connected'
 *
 * pass NULL + stats to just update the stats
 *
 */

void
toolbar_set_server_data (const gchar *name, gint files, gint users)
{
	g_return_if_fail (toolbar_label_srvname != NULL);
	g_return_if_fail (toolbar_label_srvstats != NULL);

	if ((!name) && users<=0 && files<=0)
	{
		gtk_label_set_text(GTK_LABEL(toolbar_label_srvname), _(" [not connected] "));
		gtk_label_set_text(GTK_LABEL(toolbar_label_srvstats), N_(" "));
		servers_page_set_client_id (_(" Not connected to a server. "));
		return;
	}

	if (name)
	{
		/* is it really a servername? (don't want to cleanse our own '[bla]' strings */
		if (files>=0 && users>=0)
		{
			gchar *cleanname = toolbar_remove_junk_from_servername(name);
			gchar *txt = g_strdup_printf(N_("%s"), (cleanname!=NULL) ? cleanname : name);

			if ((cleanname) && strlen(cleanname) > TOOLBAR_MAX_SRV_NAME_LEN+1)
			{
				cleanname[TOOLBAR_MAX_SRV_NAME_LEN]='>';
				cleanname[TOOLBAR_MAX_SRV_NAME_LEN+1]=0x00;
			}

			gtk_label_set_text(GTK_LABEL(toolbar_label_srvname), txt);

			if ((cleanname)&&(txt))
			{
				g_free(cleanname);
				g_free(txt);
			}
		} else gtk_label_set_text(GTK_LABEL(toolbar_label_srvname), name);
	}

	if (users>0 && files>=0)
	{
		gchar stxt[64];
		g_snprintf (stxt, sizeof(stxt), N_(":: %u "), users);
		gtk_label_set_text(GTK_LABEL(toolbar_label_srvstats), stxt);
	} else gtk_label_set_text(GTK_LABEL(toolbar_label_srvstats), N_(" "));

	/* force widget resizing (recalculate whole window lay-out from scratch) */
	gtk_widget_queue_resize(window);
}


/* toolbar_set_not_connected_to_core
 *
 *
 *
 */

void
toolbar_set_not_connected_to_core (void)
{
	toolbar_set_server_data (_(" [not connected to an eDonkey core yet] "), -1, -1);
	gtk_label_set_text(GTK_LABEL(toolbar_label_upspeed),  N_(" --- ") );
	gtk_label_set_text(GTK_LABEL(toolbar_label_downspeed),N_(" --- "));
	gtk_label_set_text(GTK_LABEL(toolbar_label_srvstats), N_(" "));
	gtk_label_set_text(GTK_LABEL(toolbar_label_cons),     N_(" --- "));
}


/* toolbar_set_cons
 *
 * Pass '0' to just query the max. number of connections
 *  (this case is not possible for the core with a gui-core
 *   connection and listening sockets)
 *
 * returns the max. number of connections seen so far
 *
 */

guint
toolbar_set_cons (guint cons)
{
	gchar 			txt[32];
	static guint	max_core_connections_seen = 0;

	if (cons == 0)
	{
		return max_core_connections_seen;
	}

	max_core_connections_seen = MAX (max_core_connections_seen, cons);

	if (toolbar_label_cons != NULL)
	{
		g_snprintf(txt, sizeof(txt), N_(" %u :: %u "), cons, max_core_connections_seen);
		gtk_label_set_text(GTK_LABEL(toolbar_label_cons), txt);
	}

	return max_core_connections_seen;
}



/* toolbar_set_entry_field_style
 *
 *
 */

void
toolbar_set_styles (void)
{
	GtkStyle	*defstyle;

	g_return_if_fail (toolbar_handlebox!=NULL);
	set_style_recursively (toolbar_handlebox, style_dark_green_background);

	defstyle = gtk_widget_get_default_style();

	g_return_if_fail (go_button!=NULL);
	set_style_recursively (go_button, defstyle);

	g_return_if_fail (toolbar_ew!=NULL);
	set_style_recursively (toolbar_ew, defstyle);

	PROCESS_GSLIST (toolbar_event_boxes ,GtkWidget,
		if (item)
		{
			set_style_recursively (item, style_dark_green_background);
		}
	);
}



/* toolbar_entry_clear
 *
 *
 */

static void
toolbar_entry_clear (void)
{
	g_return_if_fail (toolbar_ew != NULL);
	gtk_entry_set_text(GTK_ENTRY(toolbar_ew), N_(""));
}


/* toolbar_sanitize_search_phrase
 *
 * Takes a search phrase (like 'many flying pigs')
 *  and makes sure that it does not contain bad characters
 *  and returns it in a form that can be used in URLs,
 *  e.g. 'many+flying+pigs'.
 *
 * Returned string needs to be freed by caller!
 *
 */

static gchar *
toolbar_sanitize_search_phrase (const gchar *search)
{
	gchar	*s, *pos, **words, **word, *ret;
	GString	*final = NULL;

	g_return_val_if_fail (search!=NULL && *search!=0x00, NULL);

	s = g_strdup(search);
	pos = s;
	g_return_val_if_fail (s!=NULL, NULL);

	while (*pos)
	{
		if ((iscntrl(*pos)) || (ispunct(*pos)))
		{
			*pos = ' ';
		}
		pos++;
	}

	/* convert to lower case */
	g_strdown(s);

	words = g_strsplit (s, N_(" "), 0);	
	g_free(s);
	s = NULL;
	pos = NULL;
	g_return_val_if_fail (words!=NULL, NULL);

	word = words;
	while (*word)
	{
		if ((**word) && **word!=' ')
		{
			if (!final)
			{
				final = g_string_new(*word);
			} else final = g_string_append (final, *word);
			final = g_string_append_c (final, '+');
		}
		word++;
	}

	if (final)
	{
		/* cut of last '+' */
		final = g_string_truncate (final, final->len-1);
		ret = final->str;
		g_string_free (final, FALSE);
		final = NULL;
	} else ret = g_strdup(N_(""));

	g_strfreev(words);
	return ret;
}




/* toolbar_do_website_search
 *
 *
 */

static void
toolbar_do_website_search (const gchar *webengine, const gchar *websearch)
{
	gchar *url_template = NULL;

	g_return_if_fail (webengine!=NULL && *webengine!=0x00);
	g_return_if_fail (websearch!=NULL && *websearch!=0x00);

	/* jigle? */
	if ( g_strcasecmp(webengine,N_("jigle"))==0
		|| g_strcasecmp(webengine,N_("jigl"))==0
		|| g_strcasecmp(webengine,N_("jig"))==0
		|| g_strcasecmp(webengine,N_("ji"))==0
		|| g_strcasecmp(webengine,N_("jg"))==0
		|| g_strcasecmp(webengine,N_("j"))==0)
	{
		url_template = N_("http://jigle.com/search?p=%s&ma=1&d=1&a=0&l=10&t=0&x=&sl=1&su=&v=0");
	}

	/* jigle audio? */
	else if ( g_strcasecmp(webengine,N_("ja"))==0
		|| g_strcasecmp(webengine,N_("jaudio"))==0
		|| g_strcasecmp(webengine,N_("jau"))==0
		|| g_strcasecmp(webengine,N_("jmusic"))==0
		|| g_strcasecmp(webengine,N_("jmus"))==0)
	{
		url_template = N_("http://jigle.com/search?p=%s&ma=1&d=1&a=0&l=10&t=1&x=&sl=1&su=&v=0");
	}

	/* jigle video? */
	else if ( g_strcasecmp(webengine,N_("jv"))==0
		|| g_strcasecmp(webengine,N_("jvideo"))==0
		|| g_strcasecmp(webengine,N_("jvid"))==0
		|| g_strcasecmp(webengine,N_("jmovie"))==0
		|| g_strcasecmp(webengine,N_("jmov"))==0)
	{
		url_template = N_("http://jigle.com/search?p=%s&ma=1&d=1&a=0&l=10&t=2&x=&sl=1&su=&v=0");
	}

	/* jigle program? */
	else if ( g_strcasecmp(webengine,N_("jp"))==0
		|| g_strcasecmp(webengine,N_("jprogram"))==0
		|| g_strcasecmp(webengine,N_("jprog"))==0
		|| g_strcasecmp(webengine,N_("jpro"))==0)
	{
		url_template = N_("http://jigle.com/search?p=%s&ma=1&d=1&a=0&l=10&t=4&x=&sl=1&su=&v=0");
	}

	/* imdb (general)? */
	else if (g_strcasecmp(webengine,N_("imdb"))==0)
	{
		url_template = N_("http://us.imdb.com/Find?%s");
	}

	/* imdb (name/person)? */
	else if (g_strcasecmp(webengine,N_("name"))==0
		|| g_strcasecmp(webengine,N_("person"))==0
		|| g_strcasecmp(webengine,_("name"))==0
		|| g_strcasecmp(webengine,_("person"))==0)
	{
		url_template = N_("http://us.imdb.com/Nsearch?name=%s");
	}

	/* imdb (title)? */
	else if (g_strcasecmp(webengine,N_("title"))==0
		|| g_strcasecmp(webengine,_("title"))==0)
	{
		url_template = N_("http://us.imdb.com/Tsearch?title=%s");
	}

	/* ShareReactor? */
	else if (g_strcasecmp(webengine,N_("sr"))==0
		|| g_strcasecmp(webengine,N_("sharereactor"))==0)
	{
		url_template = N_("http://www.sharereactor.com/search.php?search=%s&category=0&submit=Search");
	}

	/* FileNexus? */
	else if (g_strcasecmp(webengine,N_("fn"))==0
		|| g_strcasecmp(webengine,N_("filenexus"))==0)
	{
		statusbar_extra_message (N_("TODO: check whether fileNexus supports searching with http GET method."));
		return;
	}

	/* load serverlist? */
	else if (g_strcasecmp(webengine,N_("sl"))==0
		|| g_strcasecmp(webengine,N_("met"))==0)
	{
		if (!IS_OVERNET)
		{
			if (websearch)
			{
				http_get_retrieve_serverlist (websearch,0);
			} else http_get_retrieve_serverlist (option_serverlist_URL,0);
		} else statusbar_extra_message (_("In overnet, you don't need to add servers."));
		return;
	}

	/* google? */
	else if (g_strcasecmp(webengine,N_("gg"))==0
		|| g_strcasecmp(webengine,N_("google"))==0)
	{
		url_template = N_("http://www.google.com/search?q=%s");
	}

	/* open browser? */
	else if (g_strcasecmp(webengine,N_("url"))==0
		|| g_strcasecmp(webengine,N_("http"))==0
		|| g_strcasecmp(webengine,N_("ftp"))==0)
	{
		if (g_strcasecmp(webengine,N_("url"))!=0)
		{
			gchar *url = g_strdup_printf(N_("%s:%s"), webengine, websearch);
			invoke_browser_with_url (url);
			g_free(url);
		} else invoke_browser_with_url (websearch);
		return;
	}

	if (url_template)
	{
		gchar *sane_search_phrase = toolbar_sanitize_search_phrase (websearch);
		gchar *url;

		g_return_if_fail (sane_search_phrase!=NULL && *sane_search_phrase!=0x00);

		/* TODO: should we sanity-check the template? (it must contain exactly one '%s') */

		url = g_strdup_printf (url_template, sane_search_phrase);
		g_return_if_fail (url!=NULL);
		g_free(sane_search_phrase);
		sane_search_phrase = NULL;
		
		/* fire up the browser! */
		invoke_browser_with_url (url);

		g_free(url);
		return;
	}

	ok_dialog (_("toolbar"),
		_("\n"
		  " You typed something into the toolbar that \n"
		  " I could not interpret. Type 'help' to get \n"
		  " an overview of how you can use the toolbar. \n"
		 )
	);
}


/* toolbar_show_entry_help_insert_coloured_text
 *
 *
 */

static void
toolbar_show_entry_help_insert_coloured_text (GtkWidget *dialog, GtkWidget *helptext, GdkColor *colour, const gchar *helpstring)
{
#if ((GTK_MAJOR_VERSION == 2) || (GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 3))
	g_print ("***** GTK2 FIXME: %s\n", __FUNCTION__);
#else
	gint slen;

	if (!colour)
		colour = &(helptext->style->black);

	g_return_if_fail (colour!=NULL);
	g_return_if_fail (helptext!=NULL);
	g_return_if_fail (helpstring!=NULL);

	slen = strlen(helpstring);
	gtk_text_insert (GTK_TEXT(helptext), NULL, colour, NULL, helpstring, slen);

	if (slen>1)
	{
		GtkStyle *style = gtk_widget_get_style(helptext);
		if (style) 
		{
			static gint maxwidthset = 0;
			gint pixels = gdk_text_width (style->font, helpstring, strlen(helpstring)-1);
			if (pixels>maxwidthset)
			{
				maxwidthset = pixels;
				gtk_widget_set_usize(helptext, MAX(300,(maxwidthset+50)), 250);
			}
		}
	}
#endif
}


/* toolbar_show_entry_help
 *
 *
 */

static void
toolbar_show_entry_help (void)
{
#if ((GTK_MAJOR_VERSION == 2) || (GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 3))
	g_print ("***** GTK2 FIXME: %s\n", __FUNCTION__);
#else
	GtkWidget	*dialog, *close_button;
	GtkWidget	*text, *scrollwin;

#define printcoltext(col,string)\
			toolbar_show_entry_help_insert_coloured_text(dialog,text,col,string)

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), _("how to use the ed2k-gtk-gui toolbar"));
	(void) gtk_widget_set_uposition (GTK_WIDGET (dialog), 10, 10);
	gtk_widget_set_usize(dialog, 630, 470);

	text = gtk_text_new (NULL, NULL);
	gtk_widget_show(text);
	gtk_text_set_editable  (GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap (GTK_TEXT(text), FALSE);

	scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_widget_show(scrollwin);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(scrollwin), text);

	/* we need to do this line-by-line, so the routine can determine the window width needed */
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" You can enter the following type of information into the toolbar:\n"));
	printcoltext(&blue, _("  * search phrases to search eDonkey servers or external web sites\n"));
	printcoltext(&blue, _("  * ed2k-links (server or file)\n"));
	printcoltext(&blue, _("  * advanced commands to send to the core\n"));
	printcoltext(&blue, _("  * servers and serverlist URLs\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Normal search\n"));
	printcoltext(&black,_(" Just type one or more words. You need to be connected to a server for this.\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Search Jigle - external eDonkey search engine, opens browser\n"));
	printcoltext(&black,_(" jg:<words> - searches jigle\n"));
	printcoltext(&black,_(" ja:<words> - searches jigle for audio files\n"));
	printcoltext(&black,_(" jv:<words> - searches jigle for video files\n"));
	printcoltext(&black,_(" jp:<words> - searches jigle for program files\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Search ShareReactor - external ed2k-link collection, opens browser\n"));
	printcoltext(&black,_(" sr:<words> - searches ShareReactor for files\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Search Internet Movie Database - external database, opens browser\n"));
	printcoltext(&black,_(" imdb:<words>  - searches the internet movies database (imdb.com)\n"));
	printcoltext(&black,_(" name:<words>  - searches the internet movies database for persons\n"));
	printcoltext(&black,_(" title:<words> - searches the internet movies database for film titles\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Send advanced command to core/command line client\n"));
	printcoltext(&black,_(" !<advanced command>   - sends a 'advanced command' to the core\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Add server\n"));
	printcoltext(&black,_(" <server ip>[:<port>]  - adds a server to the server list\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Load serverlist via http\n"));
	printcoltext(&black,_(" sl:<url> or met:<url> - loads serverlist file from URL (server.met or .html)\n"));
	printcoltext(&black,N_("\n"));
	printcoltext(&blue, _(" Other stuff\n"));
	printcoltext(&black,_(" gg:<words>   - search google\n"));
	printcoltext(&black,_(" url:<url>    - opens browser with given URL\n"));
	printcoltext(&black,_(" <http://bla> - opens browser with given URL\n"));
	printcoltext(&black,_(" cls          - clears the status page window\n"));
	printcoltext(&black,N_("\n"));

	close_button = gtk_button_new_with_label (_(" Close "));
	gtk_widget_show(close_button);

	gtk_signal_connect_object (GTK_OBJECT(close_button), N_("clicked"),
		GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) dialog);

	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), close_button);
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scrollwin);

	gtk_widget_show(dialog);
#endif

}


/* toolbar_go_button_clicked
 *
 *
 */

static void
toolbar_go_button_clicked (GtkWidget *widget, gpointer data)
{
	gchar	*entrytext, *phrase, *dcolon;
	guint	 n1,n2,n3,n4,port=4661;

	g_return_if_fail (data!=NULL);

	entrytext = (gchar*) gtk_editable_get_chars (GTK_EDITABLE(data), 0, -1);
	phrase = entrytext;
	g_return_if_fail (entrytext!=NULL);

	if (!*phrase)
	{
		g_free(entrytext);
		return;
	}

	/* initial '?' is redundant. We'll skip it in order to
	 * retain syntax compatibility with Axel's bong
	 */
	if (*phrase == '?')
		phrase++;

	/* advanced command? */
	if (*phrase == '!')
	{
		if (phrase[1])
		{
			gchar *msg = g_strdup_printf (_("GUI: sending advanced command... '%s' (output in status window)\n"), phrase+1);
			g_return_if_fail (msg!=NULL);
			status_msg (N_("%s"), msg);
			just_asked_for_peers_stat = 0;
			send_command (phrase+1);
			gtk_notebook_set_page (GTK_NOTEBOOK(notebook), NOTEBOOK_PAGE_STATUS);
			g_free (msg);
		}

		toolbar_entry_clear();
		g_free(entrytext);
		return;
	}

	/* help? */
	if ( *phrase == 0x00	/* if this happens, text was '?' */
		|| g_strcasecmp(phrase,N_("help"))==0
		|| g_strcasecmp(phrase,_("help"))==0
		|| g_strncasecmp(phrase,N_("??"),2)==0)
	{
		toolbar_show_entry_help();
		toolbar_entry_clear();
		g_free(entrytext);
		return;
	}

	/* cls? */
	if ( g_strcasecmp(phrase,N_("cls"))==0)
	{
		status_page_clear();
		toolbar_entry_clear();
		g_free(entrytext);
		return;
	}


	/* add server? */
	if (sscanf(phrase, N_("%u.%u.%u.%u:%u"), &n1,&n2,&n3,&n4,&port)>=4)
	{
		if (n1<256 && n2<256 && n3<256 && n4<256 && port<65536)
		{
			guint32 ip = GUINT32_TO_LE (n1 | (n2 << 8) | (n3 << 16) | (n4 << 24));

			if (!IS_OVERNET)
			{
				send_add_server(ip,port);
				send_get_serverlist();
				statusbar_extra_message (_("Server added."));
			} else statusbar_extra_message (_("In overnet, you don't need to add servers."));
			toolbar_entry_clear();
		} else statusbar_extra_message (_("Server not added - invalid format (must be n1.n2.n3.n4[:port]) with n=0-255"));
		return;
	}

	/* ed2k-link? */
	if (g_strncasecmp(phrase,N_("ed2k:"),5)==0)
	{
		gint num = -1;
		gint option_save = option_download_ed2k_link_immediately;

		option_download_ed2k_link_immediately = TRUE;
		misc_strings_unescape_url (phrase);
		num = extract_ed2k_links (phrase);
		option_download_ed2k_link_immediately = option_save;

		if (num<=0)
		{
			statusbar_extra_message (_(" Entry field does not contain any ed2k-links!?"));
		} else statusbar_extra_message (_(" Processed %d ed2k-links."), num);

		toolbar_entry_clear();

		return;
	}

	/* special web site search request? */
	dcolon = strchr(phrase, ':');
	if (dcolon)
	{
		gchar *webengine = phrase;
		gchar *websearch = dcolon+1;
		*dcolon = 0x00;
		if (!*websearch)
		{
			g_free(entrytext);
			return;
		}
		toolbar_do_website_search (webengine, websearch);
		toolbar_entry_clear();
		g_free(entrytext);
		return;
	}

	if (toolbar_label_srvname)
	{
		gchar *sname = NULL;
		gtk_label_get(GTK_LABEL(toolbar_label_srvname), &sname);
		if (sname)
		{
			if (strstr(sname,N_("[not connected]"))==NULL && sname[0]!='-' && sname[1]!='-' && sname[2]!='-')
			{
				gtk_notebook_set_page (GTK_NOTEBOOK(notebook), NOTEBOOK_PAGE_SEARCH);
				search_from_toolbar (phrase);
				toolbar_entry_clear();
				return;
			}
		}
	}

	statusbar_extra_message (_("You must be connected to a server before you can search the eDonkey2000 network."));

	g_free(entrytext);
}



/* toolbar_server_icon_clicked
 *
 *
 */

static void
toolbar_server_icon_clicked (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if (!IS_OVERNET)
	{
		servers_toolbar_icon_popup_menu (event->button, event->time);
	} else statusbar_extra_message (_("In overnet mode this menu is disabled."));
}


/* toolbar_about_homepage_url
 *
 *
 */

static void
toolbar_about_homepage_url (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	invoke_browser_with_url(GUI_URL);
}


/* toolbar_donkey_icon_popup_menu_about
 *
 *
 *
 */

static void
toolbar_donkey_icon_popup_menu_about (GtkWidget *widget, gpointer data)
{
	GtkWidget *vbox, *eventbox;
	GtkWidget *icon;
  	GtkWidget *jed_label, *tim_label, *con_label;
  	GtkWidget *guiver_label, *ver_label, *hp_label;
	gchar     *constr, *guiverstr;

	vbox = gtk_vbox_new(FALSE,15);
	gtk_widget_show(vbox);

	icon = create_widget_from_xpm (window, icon_mainlogo);
	gtk_widget_show (icon);
	gtk_box_pack_start (GTK_BOX (vbox), icon, TRUE, FALSE, 0);

	jed_label = gtk_label_new (_("eDonkey 2000 was programmed by Jed McCaleb <jed@edonkey2000.com>"));
	gtk_widget_show (jed_label);
	gtk_label_set_justify (GTK_LABEL(jed_label), GTK_JUSTIFY_CENTER);
	gtk_box_pack_start (GTK_BOX (vbox), jed_label, TRUE, FALSE, 0);

	tim_label = gtk_label_new (_("This GUI was programmed by Tim-Philipp Muller <tim@edonkey2000.com>"));
	gtk_widget_show (tim_label);
	gtk_label_set_justify (GTK_LABEL(tim_label), GTK_JUSTIFY_CENTER);
	gtk_box_pack_start (GTK_BOX (vbox), tim_label, TRUE, FALSE, 0);

	constr = g_strdup_printf (_("Contributors:\n%s \n... and others."), CONTRIBUTORS);
	con_label = gtk_label_new (constr);
	gtk_widget_show (con_label);
	gtk_label_set_justify (GTK_LABEL(con_label), GTK_JUSTIFY_CENTER);
	gtk_box_pack_start (GTK_BOX (vbox), con_label, TRUE, FALSE, 0);

	guiverstr = g_strdup_printf(N_("%s: %s (build %s %s)"), _("GUI version"), VERSION, __DATE__, __TIME__);
	ver_label = gtk_label_new (guiverstr);
	gtk_widget_show (ver_label);
	gtk_box_pack_start (GTK_BOX (vbox), ver_label, TRUE, FALSE, 0);

	ver_label = gtk_label_new (toolbar_core_version_string);
	gtk_widget_show (ver_label);
	gtk_box_pack_start (GTK_BOX (vbox), ver_label, TRUE, FALSE, 0);

	eventbox = gtk_event_box_new();
	gtk_widget_show (eventbox);
	hp_label = gtk_label_new (N_("http://ed2k-gtk-gui.sourceforge.net"));
	gtk_label_set_pattern (GTK_LABEL(hp_label), N_("___________________________________"));
	gtk_widget_show(hp_label);
	gtk_container_add (GTK_CONTAINER(eventbox), hp_label);
	gtk_signal_connect ( GTK_OBJECT (eventbox),
	                     N_("button_press_event"),
	                     GTK_SIGNAL_FUNC(toolbar_about_homepage_url),
	                     NULL);
	set_style_recursively (hp_label, style_url_label);
	gtk_box_pack_start (GTK_BOX (vbox), eventbox, TRUE, FALSE, 0);

	ok_dialog_from_widget(_(" About GUI and core "), vbox);

	if ((constr)&&(guiverstr))
	{
		g_free(constr);
		g_free(guiverstr);
	}
}


/* toolbar_donkey_icon_popup_menu_say_thank_you
 *
 * The 'say thank you' button has been clicked
 *
 * XXXX - The amazon number changed shortly before the release.
 *        Do this replacement thing, so that we don't need
 *        translators to update now. CHANGE TEXT AFTER RELEASE!
 */

static void
toolbar_donkey_icon_popup_menu_say_thank_you (GtkWidget *widget, gpointer data)
{
	const gchar *txt = _("Send me an e-mail. Positive feedback always makes me happy.\n\n"
			"If this GUI makes you extremely happy, have a look at my amazon wishlist at\n\n"
			"http://www.amazon.co.uk/exec/obidos/wishlist/2XT29C8FTUIFN or\n"
			"http://www.amazon.de/exec/obidos/wishlist/3G88NU8L43WL6 :)\n\n"
			"If you have any other feedback or suggestions, please let me know as well.\n\n"
			"tim@edonkey2000.com");

	gchar *thankyoutext = g_strdup(txt);
	gchar *oldnumber = strstr(thankyoutext, N_("2XT29C8FTUIFN"));

	if (oldnumber)
	{
		strncpy(oldnumber,N_("1NL5R93EL1WA6"),13);
	}

	ok_dialog (_("How to say thank you for this GUI"), thankyoutext);

	G_FREE(thankyoutext);
}

/******************************************************************************
 *
 * toolbar_donkey_icon_popup_menu_shutdown_core
 *
 * toolbar_donkey_icon_popup_menu_donot_shutdown_core
 * toolbar_donkey_icon_popup_menu_really_shutdown_core
 *
 *
 * The 'shutdown core' button has been clicked.
 * Check if we really want to shutdown the core
 *  and do it if the user is sure.
 *
 */

static void
toolbar_donkey_icon_popup_menu_donot_shutdown_core (GtkWidget *widget, gpointer data)
{
	/* Destroy the dialog (note that variable 'widget' is the button widget here) */
	if (widget)
	{
		gtk_widget_destroy (GTK_WIDGET(data));
	}
}

static void
toolbar_donkey_icon_popup_menu_really_shutdown_core (GtkWidget *widget, gpointer data)
{
	/* Destroy the dialog (note that variable 'widget' is the button widget here) */
	if (widget)
	{
		gtk_widget_destroy (GTK_WIDGET(data));
	}

	status_message_blue (_("GUI: telling core to stop and shutdown...\n"));
	dns_lookup_remove_dns_servers();
	send_logout();
}


static void
toolbar_donkey_icon_popup_menu_shutdown_core (GtkWidget *widget, gpointer data)
{
	yesno_dialog (
		_(" --- SHUTDOWN CORE --- "),
		_("\nDo you REALLY want to shutdown the core?\n\nAll downloads will stop.\n"),
		_("   YES, do it   "),
		_("   NO, don't do it   "),
		toolbar_donkey_icon_popup_menu_really_shutdown_core,
		toolbar_donkey_icon_popup_menu_donot_shutdown_core
	);
}


/******************************************************************************
 *
 * toolbar_donkey_icon_popup_menu_kill_and_restart_core
 *
 * The 'kill and restart core' button has been clicked.
 * Make sure that the user really wants this.
 *
 */

static void
toolbar_donkey_icon_popup_menu_donot_kill_and_restart_core (GtkWidget *widget, gpointer data)
{
	/* Destroy the dialog (note that variable 'widget' is the button widget here) */
	if (data)
	{
		gtk_widget_destroy (GTK_WIDGET(data));
	}
}

static void
toolbar_donkey_icon_popup_menu_really_kill_and_restart_core (GtkWidget *widget, gpointer data)
{
	/* Destroy the dialog (note that variable 'widget' is the button widget here) */
	if (data)
	{
		gtk_widget_destroy (GTK_WIDGET(data));
	}

	/* dokill_flag = 1 => kill it as well */
	kill_and_restart_core (1);
}

static void
toolbar_donkey_icon_popup_menu_kill_and_restart_core (GtkWidget *widget, gpointer data)
{
	yesno_dialog (
		_(" --- KILL AND RESTART CORE --- "),
		_("\nDo you REALLY want to kill and restart the core?\n"),
		_("   YES, do it   "),
		_("   NO, don't do it   "),
		toolbar_donkey_icon_popup_menu_really_kill_and_restart_core,
		toolbar_donkey_icon_popup_menu_donot_kill_and_restart_core
	);
}


/* toolbar_donkey_icon_popup_menu_disconnect_core
 *
 * The 'close GUI-core connection' button has been clicked
 */

static void
toolbar_donkey_icon_popup_menu_disconnect_core (GtkWidget *widget, gpointer data)
{
	status_message_blue (_("GUI: closing connection (socket) to core...\n"));
	guicoreconn_close_connection ();
}

/* toolbar_donkey_icon_popup_menu_url
 *
 * a button that is supposed to call a browser with an URL has been clicked
 * data is a (gchar*) to the URL string.
 */

static void
toolbar_donkey_icon_popup_menu_url (GtkWidget *widget, gpointer data)
{
	g_return_if_fail (data!=NULL);
	invoke_browser_with_url ((gchar*)data);
}


/* toolbar_donkey_icon_clicked
 *
 *
 */

static void
toolbar_donkey_icon_clicked (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	static GtkWidget	*menu = NULL, *killrestart;

	const gchar *EXIT_STR = 			_(" Exit eDonkey ");
	const gchar *DISCONNECT_STR =		_(" Disconnect from core ");
	const gchar *SHUTDOWN_STR = 		_(" Shutdown core ");
	const gchar *KILLRESTART_STR = 		_(" Kill + restart core ");
	const gchar *THANKYOU_STR =			_(" Say \'thank you\' ");
	const gchar	*ABOUT_STR =			_(" About GUI and core ");
	const gchar *GUIHP_STR =			_(" ed2k-gtk-gui homepage ");
	const gchar *GUIHELP_STR =			_(" ed2k-gtk-gui documentation ");
	const gchar *ED2KHP_STR =			_(" edonkey2000 homepage ");
	menu  = gtk_menu_new();

	(void) MENU_ITEM_ADD (ABOUT_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_about), NULL, icon_status);
	(void) MENU_ITEM_ADD__________;
	(void) MENU_ITEM_ADD (THANKYOU_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_say_thank_you), NULL, icon_mail);
	(void) MENU_ITEM_ADD__________;
	(void) MENU_ITEM_ADD (GUIHP_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_url), GUI_URL, icon_tux);
	(void) MENU_ITEM_ADD (GUIHELP_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_url), DOCS_URL, icon_about);
	(void) MENU_ITEM_ADD__________;
	(void) MENU_ITEM_ADD (ED2KHP_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_url), ED2K_URL, icon_none);
	(void) MENU_ITEM_ADD__________;
	(void) MENU_ITEM_ADD (EXIT_STR, GTK_SIGNAL_FUNC(mainwindow_delete_event), NULL, icon_exit);
	(void) MENU_ITEM_ADD__________;
	(void) MENU_ITEM_ADD (DISCONNECT_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_disconnect_core), NULL, icon_disconnect);
	(void) MENU_ITEM_ADD (SHUTDOWN_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_shutdown_core), NULL, icon_skull);
	killrestart = MENU_ITEM_ADD (KILLRESTART_STR, GTK_SIGNAL_FUNC(toolbar_donkey_icon_popup_menu_kill_and_restart_core), NULL, icon_refresh);

	if ((CORE_IS_ALIVE) && (CORE_IS_LOCAL) && local_donkey_pid>0 && first_time_we_get_options==0)
	{
		gtk_widget_set_sensitive(killrestart, TRUE);
	} else gtk_widget_set_sensitive(killrestart, FALSE);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
}


/* toolbar_create_entry_field
 *
 *
 */

static GtkWidget *
toolbar_create_entry_field (void)
{
	GtkWidget	*hbox;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();

	hbox = gtk_hbox_new(FALSE, FALSE);
	gtk_widget_show(hbox);

	toolbar_ew = gtk_entry_new();
	gtk_widget_show(toolbar_ew);
	/* enter key = 'Go!' button pressed */
	gtk_signal_connect ( GTK_OBJECT(toolbar_ew),
	                     N_("activate"),
	                     GTK_SIGNAL_FUNC(toolbar_go_button_clicked),
	                     toolbar_ew);
	/* make num block enter key behave like normal enter key */
	gtk_signal_connect ( GTK_OBJECT (toolbar_ew),
	                     N_("insert_text"),
	                     GTK_SIGNAL_FUNC(misc_gtk_entry_insert_text_handler),
	                     toolbar_ew);
	gtk_tooltips_set_tip(tooltips,
	                     toolbar_ew,
	                     _("You can enter search phrases, ed2k-links, advanced commands, servers, and server list URLs here.\n"
	                       "Type 'help' in here to get more information.\n"
	                       "You can paste stuff from the clipboard with Control-V or by positioning "
	                       "the mouse pointer over the entry field and clicking the middle mouse button"),
	                     NULL);

	gtk_box_pack_start (GTK_BOX (hbox), toolbar_ew, TRUE, TRUE, 1);

	/* 'Go!' button */

	go_button = gtk_button_new();
	gtk_widget_ref (go_button);
	gtk_container_add(GTK_CONTAINER(go_button),
	make_icon_and_label_hbox(_(" Go "), icon_ignition_small));
	gtk_widget_show (go_button);
	gtk_signal_connect ( GTK_OBJECT(go_button),
	                     N_("clicked"),
	                     GTK_SIGNAL_FUNC(toolbar_go_button_clicked),
	                     toolbar_ew);
	gtk_box_pack_start (GTK_BOX (hbox), go_button, FALSE, FALSE, 0);
	GTK_WIDGET_SET_FLAGS (go_button, GTK_CAN_DEFAULT);

	return hbox;
}



/* toolbar_create_stats_box
 *
 *
 */

static GtkWidget *
toolbar_create_stats_box (void)
{
	GtkWidget	*downbox, *upbox, *srvnbox, *srvsbox, *consbox;
	GtkWidget	*hbox, *imgdown, *imgup, *imgsrv, *imgsrvbox, *imgcons;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();

	hbox = gtk_hbox_new(FALSE, FALSE);
	toolbar_hbox_stats = hbox;
	gtk_widget_show(hbox);

	/* down speed */

	imgdown = create_widget_from_xpm (window, icon_download);
	gtk_widget_show(imgdown);
	gtk_box_pack_start (GTK_BOX(hbox), imgdown, FALSE, FALSE,0);

	downbox = gtk_event_box_new();
	gtk_widget_show(downbox);
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)downbox);
	toolbar_label_downspeed = gtk_label_new(NULL);
	gtk_widget_show(toolbar_label_downspeed);
	gtk_label_set_justify (GTK_LABEL(toolbar_label_downspeed), GTK_JUSTIFY_LEFT);
	gtk_container_add (GTK_CONTAINER(downbox), toolbar_label_downspeed);
	gtk_tooltips_set_tip(tooltips, downbox, _("download speed in kB/s"), NULL);
	gtk_box_pack_start (GTK_BOX(hbox), downbox, FALSE, FALSE, 0);

	/* up speed */

	imgup = create_widget_from_xpm (window, icon_upload);
	gtk_widget_show(imgup);
	gtk_box_pack_start (GTK_BOX(hbox), imgup, FALSE, FALSE,0);

	upbox = gtk_event_box_new();
	gtk_widget_show(upbox);
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)upbox);
	toolbar_label_upspeed = gtk_label_new(NULL);
	gtk_label_set_justify (GTK_LABEL(toolbar_label_upspeed), GTK_JUSTIFY_LEFT);
	gtk_widget_show(toolbar_label_upspeed);
	gtk_container_add (GTK_CONTAINER(upbox), toolbar_label_upspeed);
	gtk_tooltips_set_tip(tooltips, upbox, _("upload speed in kB/s"), NULL);
	gtk_box_pack_start (GTK_BOX(hbox), upbox, FALSE, FALSE, 0);

	/* server */

	imgsrvbox = gtk_event_box_new();
	gtk_widget_show(imgsrvbox);
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)imgsrvbox);
	imgsrv = create_widget_from_xpm (window, icon_servers);
	gtk_widget_show(imgsrv);
	gtk_container_add (GTK_CONTAINER(imgsrvbox), imgsrv);
	gtk_tooltips_set_tip(tooltips, imgsrvbox, _("click this icon for easy access to some server functions"), NULL);
	gtk_signal_connect ( GTK_OBJECT (imgsrvbox),
	                     N_("button_press_event"),
	                     GTK_SIGNAL_FUNC(toolbar_server_icon_clicked),
	                     NULL);
	gtk_box_pack_start (GTK_BOX(hbox), imgsrvbox, FALSE, FALSE,0);

	/* server name */
	srvnbox = gtk_event_box_new();
	gtk_widget_show(srvnbox);
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)srvnbox);
	toolbar_label_srvname = gtk_label_new(NULL);
	gtk_label_set_justify (GTK_LABEL(toolbar_label_srvname), GTK_JUSTIFY_LEFT);
	gtk_widget_show(toolbar_label_srvname);
	gtk_container_add (GTK_CONTAINER(srvnbox), toolbar_label_srvname);
	gtk_tooltips_set_tip(tooltips, srvnbox, _("name or IP of the server you are currently connected to.\n"), NULL);
	gtk_box_pack_start (GTK_BOX(hbox), srvnbox, FALSE, FALSE, 4);

	/* server users */
	srvsbox = gtk_event_box_new();
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)srvsbox);
	gtk_widget_show(srvsbox);
	toolbar_label_srvstats = gtk_label_new(NULL);
	gtk_label_set_justify (GTK_LABEL(toolbar_label_srvstats), GTK_JUSTIFY_LEFT);
	gtk_widget_show(toolbar_label_srvstats);
	gtk_container_add (GTK_CONTAINER(srvsbox), toolbar_label_srvstats);
	gtk_tooltips_set_tip(tooltips, srvsbox, _("Number of users connected to this server.\n"
	                                          "Note: eDonkey will search ALL known servers for sources."), NULL);
	gtk_box_pack_start (GTK_BOX(hbox), srvsbox, FALSE, FALSE, 0);


	/* connections used */
	imgcons = create_widget_from_xpm (window, icon_connections);
	gtk_widget_show(imgcons);
	toolbar_event_boxes = g_slist_prepend (toolbar_event_boxes, (gpointer)imgcons);
	gtk_box_pack_start (GTK_BOX(hbox), imgcons, FALSE, FALSE,0);

	consbox = gtk_event_box_new();
	gtk_widget_show(consbox);
	toolbar_label_cons = gtk_label_new(NULL);
	gtk_label_set_justify (GTK_LABEL(toolbar_label_cons), GTK_JUSTIFY_LEFT);
	gtk_widget_show(toolbar_label_cons);
	gtk_container_add (GTK_CONTAINER(consbox), toolbar_label_cons);
	gtk_tooltips_set_tip(tooltips, consbox, _("Number of connections currently used by the core (and max.number seen)."
	                                          "This value is not utterly important, it is just there to monitor whether "
	                                          "your 'max.connections' setting in options is realistic."), NULL);
	gtk_box_pack_start (GTK_BOX(hbox), consbox, FALSE, FALSE, 0);

	toolbar_set_not_connected_to_core ();

	return hbox;
}


/******************************************************************************
 *
 *   toolbar_set_overnet_mode
 *
 */

void
toolbar_set_overnet_mode (gboolean overnet)
{
	if (overnet)
	{
		gtk_widget_show(overnetbox);
		gtk_widget_hide(donkeybox);
	} else {
		gtk_widget_show(donkeybox);
		gtk_widget_hide(overnetbox);
	}
}


/* toolbar_create
 *
 *
 */

GtkWidget *
toolbar_create (void)
{
	GtkWidget	*hbox;
	GtkWidget	*ed2klogo;
	GtkWidget	*overnetlogo;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox);

	/* handle box */
	toolbar_handlebox = gtk_handle_box_new();
	gtk_widget_show(toolbar_handlebox);
	gtk_handle_box_set_shadow_type(GTK_HANDLE_BOX(toolbar_handlebox), GTK_SHADOW_NONE);
	gtk_handle_box_set_handle_position(GTK_HANDLE_BOX(toolbar_handlebox), GTK_POS_RIGHT);
	gtk_container_set_border_width(GTK_CONTAINER (toolbar_handlebox), 1);
	gtk_container_add (GTK_CONTAINER (toolbar_handlebox), hbox);

	/* logo donkey */
	donkeybox = gtk_event_box_new();
	gtk_widget_show(donkeybox);
	gtk_tooltips_set_tip(tooltips, donkeybox, _("About, help, and advanced options.\n"), NULL);
	gtk_signal_connect ( GTK_OBJECT (donkeybox),
	                     N_("button_press_event"),
	                     GTK_SIGNAL_FUNC(toolbar_donkey_icon_clicked),
	                     NULL);
	ed2klogo = create_widget_from_xpm (window, icon_mainlogo);
	gtk_widget_show(ed2klogo);
	gtk_container_add (GTK_CONTAINER(donkeybox), ed2klogo);
	g_return_val_if_fail (ed2klogo!=NULL, hbox);

	/* logo overnet */
	overnetbox = gtk_event_box_new();
	gtk_tooltips_set_tip(tooltips, overnetbox, _("About, help, and advanced options.\n"), NULL);
	gtk_signal_connect ( GTK_OBJECT (overnetbox),
	                     N_("button_press_event"),
	                     GTK_SIGNAL_FUNC(toolbar_donkey_icon_clicked),
	                     NULL);
	overnetlogo = create_widget_from_xpm (window, icon_mainlogo_overnet);
	gtk_widget_show(overnetlogo);
	gtk_container_add (GTK_CONTAINER(overnetbox), overnetlogo);
	g_return_val_if_fail (overnetlogo!=NULL, hbox);
	gtk_widget_hide(overnetbox);

	gtk_box_pack_start (GTK_BOX (hbox), donkeybox, FALSE, FALSE, 2);
	gtk_box_pack_start (GTK_BOX (hbox), overnetbox, FALSE, FALSE, 2);
	gtk_box_pack_start (GTK_BOX (hbox), toolbar_create_stats_box (), FALSE, FALSE, 2);
	gtk_box_pack_end   (GTK_BOX (hbox), toolbar_create_entry_field(), TRUE, TRUE, 2);

	return toolbar_handlebox;
}


