/* uploads_page.c
*
*  populates upload page in notepad and deals with its functions
*
*  (c) 2001 Tim-Philipp Muller <t.i.m@orange.net>
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>

#include "downloads_data.h"
#include "downloads_page.h"
#include "global.h"
#include "gui_flags.h"
#include "gui_options.h"
#include "icons.h"
#include "misc.h"
#include "misc_gtk.h"
#include "shared_files.h"
#include "statusbar.h"
#include "uploads_page.h"
#include "parse_message.h"


// global variables

GtkWidget *clist_uploads, *ulpanedwin2, *clist_ulstats;
GtkWidget *ulstats_scrollwin, *ul_scrollwin;

float	total_uploaded = 0.0;
time_t	stats_last_cleared = 0;
time_t	seconds_upload_was_nonzero = 1;		// so we don't divide by zero


static GtkWidget	*uploads_page_stats_label = NULL;

/* functions */

static gint		ulstats_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2);
static gint		uploads_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2);
static void		uploads_clist_column_clicked (GtkCList *clist, gint column, gpointer data);
static void		ulstats_clist_column_clicked (GtkCList *clist, gint column, gpointer data);


// sync_ulstats_filenames
//
// sync filenames from shared files list to upload stats list
//	(in case a file got renamed)

void
sync_ulstats_filenames (void)
{
	if (!sharedfiles) return;
	if (!uploadstats) return;

	PROCESS_GPTRARRAY( sharedfiles, sharedfileRecord,
		if (item)
		{
			uploadstatsRecord *ulstatsr = uploadstats_find_from_hash (item->hash);
			if (ulstatsr)
				ulstatsr->namequark = item->namequark;
		}
	);
}

// compare two cells
//
//  we're using stuff from the gtk source gtkclist.c default_compare function here
//

static gint
uploads_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	uploadRecord	*ul1, *ul2;
	gchar			*text1=NULL, *text2=NULL;

	g_return_val_if_fail (ptr1!=NULL,0);				/* just making sure... */
	g_return_val_if_fail (ptr2!=NULL,0);

	ul1 = (uploadRecord*) ((GtkCListRow*)ptr1)->data;
	ul2 = (uploadRecord*) ((GtkCListRow*)ptr2)->data;

	g_return_val_if_fail (ul1!=NULL,0);					/* just making sure... */
	g_return_val_if_fail (ul2!=NULL,0);

	/* sort depending on type of content (i.e. which column is the one to sort) */

	switch (uploadlist_sortcolumn)
	{
		/* column 0 -> filename */
		case 0:
		break;	/* dealt with below */

		/* column 1 -> nickname */
		case 1: MISC_RETURN_STR_COMPARE(ul1->username,ul2->username);

		/* column 2 -> speed    */
		case 2: MISC_RETURN_NUM_COMPARE(ul1->speed,ul2->speed);
		default:
		{
			g_printerr (N_("%s: unknown sort column (%u)!\n"), __FUNCTION__, uploadlist_sortcolumn);
			return 0;
		}
		break;
	}

	/* here we retrieve the filenamefrom the clist. This _should_
	 *	 be faster than looking up the string via the GQuark (?)
	 */
	get_text_from_gtk_clist_rows ((GtkCListRow *) ptr1, (GtkCListRow *) ptr2, clist->sort_column, &text1, &text2);

	if (!text2)
		return (text1 != NULL);
	if (!text1)
		return -1;

	return g_strcasecmp (text1, text2);
}


static gint
ulstats_clist_compare_func_sub (uploadstatsRecord *ul1, uploadstatsRecord *ul2, gint sort_column)
{
	g_return_val_if_fail (ul1!=NULL,0);					// just making sure...
	g_return_val_if_fail (ul2!=NULL,0);

	switch (sort_column)
	{
		case 0:	MISC_RETURN_NUM_COMPARE(ul1->uploaded,ul2->uploaded);	// 0 - amount uploaded
		case 1:															// 1 - upload/size ratio
		{
			gfloat ratio1 = ul1->uploaded/((gfloat)ul1->size+0.0000001);
			gfloat ratio2 = ul2->uploaded/((gfloat)ul2->size+0.0000001);
			if (ratio1>10000.0) ratio1 = -1.0;
			if (ratio2>10000.0) ratio2 = -1.0;
			MISC_RETURN_NUM_COMPARE(ratio1,ratio2);
		}
		case 3:															// 3 - last uploaded (higher=>more recent)
		{
			MISC_RETURN_NUM_COMPARE(ul1->last_uploaded,ul2->last_uploaded);
		}
		case 2:															// 2  - filename (we shouldn't be called here!)
		default:
			g_printerr (N_("FIXME: unknown upload stats clist sort column in sub!?\n"));
		return 0;
	}
}

static gint
ulstats_clist_compare_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	uploadstatsRecord	*ul1, *ul2;
	gchar 				*text1=NULL, *text2=NULL;

	g_return_val_if_fail (ptr1!=NULL,0);				// just making sure...
	g_return_val_if_fail (ptr2!=NULL,0);

	ul1 = (uploadstatsRecord*) ((GtkCListRow*)ptr1)->data;
	ul2 = (uploadstatsRecord*) ((GtkCListRow*)ptr2)->data;

	g_return_val_if_fail (ul1!=NULL,0);					// just making sure...
	g_return_val_if_fail (ul2!=NULL,0);

	// if sort column is not filename, compare using the uploadstatsRecord
	//	if values are equal, we'll sort by filename as well (otherwise equal rows tend to 'jump' within the clist)
	if (clist->sort_column != 2)
	{
		gint ret = ulstats_clist_compare_func_sub (ul1, ul2, clist->sort_column);
		if (ret!=0)		// values not equal => good, return them
			return ret;
	}

	// here we retrieve the text from the clist. This _should_
	//	 be faster than looking up the string via the GQuark
	get_text_from_gtk_clist_rows ((GtkCListRow *) ptr1, (GtkCListRow *) ptr2, clist->sort_column, &text1, &text2);
	if (!text2) return (text1 != NULL);
	if (!text1) return -1;
	return g_strcasecmp (text1, text2);
}


static void
uploads_clist_column_clicked (GtkCList *clist, gint column, gpointer data)
{
	last_time_of_user_click = time(NULL);
	/* column that is already active for sorting? -> change sort type */

	if (column == uploadlist_sortcolumn) {
		if (uploadlist_sorttype == GTK_SORT_ASCENDING)
			uploadlist_sorttype = GTK_SORT_DESCENDING;
		else
			uploadlist_sorttype = GTK_SORT_ASCENDING;
	} else {
		/* set new sort column; default type = ascending */
		uploadlist_sortcolumn = column;
		uploadlist_sorttype   = GTK_SORT_ASCENDING;
	}

	gtk_clist_set_auto_sort (clist, FALSE);
	gtk_clist_set_sort_column (clist, uploadlist_sortcolumn);
	gtk_clist_set_sort_type (clist, uploadlist_sorttype);
	gtk_clist_set_compare_func (clist,uploads_clist_compare_func);
	gtk_clist_sort (clist);
}


static void
ulstats_clist_column_clicked (GtkCList *clist, gint column, gpointer data)
{
	last_time_of_user_click = time(NULL);
	/* column that is already active for sorting? -> change sort type */

	if (column == ulstatslist_sortcolumn) {
		if (ulstatslist_sorttype == GTK_SORT_ASCENDING)
			ulstatslist_sorttype = GTK_SORT_DESCENDING;
		else
			ulstatslist_sorttype = GTK_SORT_ASCENDING;
	} else {
		/* set new sort column; default type = ascending */
		ulstatslist_sortcolumn = column;
		ulstatslist_sorttype   = GTK_SORT_ASCENDING;
	}

	gtk_clist_set_auto_sort (clist, FALSE);
	gtk_clist_set_sort_column (clist, ulstatslist_sortcolumn);
	gtk_clist_set_sort_type (clist, ulstatslist_sorttype);
	gtk_clist_set_compare_func (clist,ulstats_clist_compare_func);
	gtk_clist_sort (clist);
}


/*
static void
uploads_view_queue_button_clicked (GtkWidget *widget, gpointer data)
{
	g_printerr (N_("FIX ME: VIEW UPLOAD QUEUE\n"));
}
*/

static void
uploads_column_resized (GtkCList *clist, int col, int new_width, gpointer data)
{
	last_time_of_user_click = time(NULL);
	column_widths[UL_COLS][col] = new_width;
}

static void
ulstats_column_resized (GtkCList *clist, int col, int new_width, gpointer data)
{
	last_time_of_user_click = time(NULL);
	column_widths[ULSTATS_COLS][col] = new_width;
}


/*
static void
queue_column_resized (GtkCList *clist, int col, int new_width, gpointer data)
{
	last_time_of_user_click = time(NULL);
	column_widths[Q_COLS][col] = new_width;
}
*/
static void
ulstats_remove_item_button_clicked (GtkWidget *widget, gpointer gdata)
{
	if (g_list_length(GTK_CLIST(clist_ulstats)->selection)>1)
	{
		g_printerr (N_("FIX ME: sort items selected before removing them! (%s)\n"), __FUNCTION__);
		return;
	}

	PROCESS_CLIST_SELECTION_LIST( GTK_CLIST(clist_ulstats), uploadstats, uploadstatsRecord,
		if (item) uploadstats_remove_record(item);
    );
}

static void
ulstats_remove_items_by_size_button_clicked (GtkWidget *widget, gpointer gdata)
{
	gfloat size = (gfloat) ((guint)gdata);

	PROCESS_GPTRARRAY_FROM_END ( uploadstats, uploadstatsRecord,
		if (1==1) 	// TODO: new option clear upload stats items by FILEsize, not by tramsferred amount
		{
			if ((item->uploaded*1024.0) <= size) // uploaded is in kB, size is in bytes
				uploadstats_remove_record (item);
		}
	);
}

static void
ulstats_clear_list_button_clicked (GtkWidget *widget, gpointer gdata)
{
	uploadstats_cleanup ();
}

static void
uploads_reset_stats_cancel_button_clicked_donot_cancel_callback (GtkWidget *widget, gpointer data)
{
	/* Destroy the dialog widget (note that 'widget' is the button here */
	if (data)
	{
		gtk_widget_destroy (GTK_WIDGET (data));
	}
}

static void
uploads_reset_stats_cancel_button_clicked_really_cancel_callback (GtkWidget *widget, gpointer data)
{
        /* "ok" button clicked. now destroy window (if the widget called us) */
        if (widget != NULL) gtk_widget_destroy (GTK_WIDGET (data));

	// okay, do it...
	stats_last_cleared = time(NULL);
	total_downloaded = 0.0;
	total_uploaded   = 0.0;
	seconds_download_was_nonzero = (time_t) 1;
	seconds_upload_was_nonzero   = (time_t) 1;
	statusbar_extra_message (_("okay, set total ul/dl stats to zero."));
}

static void
ulstats_reset_stats_button_clicked (GtkWidget *widget, gpointer gdata)
{
	yesno_dialog (_("reset total upload/download stats"),
		      _("Do you REALLY want to set the total amount uploaded and downloaded to zero?"),
		      _("   Yes, do it   "), _("   No, don't do it   "),
	 	uploads_reset_stats_cancel_button_clicked_really_cancel_callback,
	 	uploads_reset_stats_cancel_button_clicked_donot_cancel_callback);

}

static void
ulstats_remove_not_shared_button_clicked (GtkWidget *widget, gpointer gdata)
{
	PROCESS_GPTRARRAY_FROM_END ( uploadstats, uploadstatsRecord,
		if (item) 
		{
			if (shared_files_find_from_hash(item->hash)) continue;
			if (downloads_find_from_hash(item->hash)) continue;
			uploadstats_remove_record (item);
		}
	);
}

/* right-click pop-up menu code by Mike Hicks, found on the gtk-list@redhat.com
*  mailing list on 25/11/98
*/

static gint
ulstats_button_pressed (GtkWidget *widget, GdkEvent *event)
{
	GtkWidget   *menu, *item1, *item2, *item3, *item4, *item5, *item6, *item7, *item8;
	GtkWidget   *item9;// *item10, *item11;
	int	    row,col;

	last_time_of_user_click = time(NULL);

	if (event->type == GDK_BUTTON_PRESS) {
		/* convert GdkEvent to GdkEventButton */
		GdkEventButton *bevent = (GdkEventButton *) event;
		if (bevent->button == 3) {
			gtk_clist_get_selection_info (GTK_CLIST(clist_ulstats),
						(int) bevent->x ,(int) bevent->y, &row, &col);
			gtk_clist_select_row (GTK_CLIST(clist_ulstats),row,col);

//			DEBUG ("BUTTON 3 (right button) is pressed (r=%d,c=%d)\n", row,col);

			if ((!uploadstats) || uploadstats->len==0)
				return TRUE;

			menu  = gtk_menu_new();
			item1 = add_menuitem (menu, _(" remove this item "), ulstats_remove_item_button_clicked, NULL, icon_none);
			item2 = add_menuitem (menu, NULL, NULL, NULL, NULL);
			item3 = add_menuitem (menu, _(" remove all items <  10M "), ulstats_remove_items_by_size_button_clicked, (gpointer) ((unsigned int) (10*1024*1024)), icon_none);
			item4 = add_menuitem (menu, _(" remove all items <  50M "), ulstats_remove_items_by_size_button_clicked, (gpointer) ((unsigned int) (50*1024*1024)), icon_none);
			item5 = add_menuitem (menu, _(" remove all items < 100M "), ulstats_remove_items_by_size_button_clicked, (gpointer) ((unsigned int) (100*1024*1024)), icon_none);
			item6 = add_menuitem (menu, _(" remove all items < 500M "), ulstats_remove_items_by_size_button_clicked, (gpointer) ((unsigned int) (500*1024*1024)), icon_none);
				add_menuitem (menu, NULL, NULL, NULL, NULL);
			item7 = add_menuitem (menu, _(" remove all items I don't share anymore "), ulstats_remove_not_shared_button_clicked, NULL, icon_none);
				add_menuitem (menu, NULL, NULL, NULL, NULL);
			item8 = add_menuitem (menu, _(" clear entire list "), ulstats_clear_list_button_clicked, NULL, icon_cancel);
				add_menuitem (menu, NULL, NULL, NULL, NULL);
			item9 = add_menuitem (menu, _(" reset total upload/download stats "), ulstats_reset_stats_button_clicked, NULL, icon_cancel);

			gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
					bevent->button, bevent->time);
		}
		return TRUE;	 /* event has been dealt with */
	}

	return FALSE;
}


/* Populate uploads Page  -----------------------------------------------------------
*
*/

static GtkWidget *
create_scrolled_upload_clist (void)
{
	gchar **tabtitles;
	gint    i;

	tabtitles = g_strsplit ( _(" file being uploaded| who| kB/s"), N_("|"), 0 );
	g_return_val_if_fail (tabtitles!=NULL, NULL);

	/* create uploads clist window and set its properties */

	clist_uploads = gtk_clist_new_with_titles (NO_OF_UPLOAD_COLS, tabtitles);
	gtk_signal_connect (GTK_OBJECT (clist_uploads), N_("click-column"),
			    GTK_SIGNAL_FUNC(uploads_clist_column_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_uploads), N_("resize-column"),
				GTK_SIGNAL_FUNC(uploads_column_resized), NULL);

	gtk_clist_set_auto_sort (GTK_CLIST(clist_uploads),FALSE);
	gtk_clist_set_compare_func (GTK_CLIST(clist_uploads), uploads_clist_compare_func);
	gtk_clist_sort (GTK_CLIST(clist_uploads));

	/* always exactly one item selected (unless less is empty) */
	gtk_clist_set_selection_mode(GTK_CLIST (clist_uploads), GTK_SELECTION_EXTENDED);

	/* set default column widths */
	set_clist_columns (GTK_CLIST(clist_uploads),UL_COLS, NO_OF_UPLOAD_COLS);

	gtk_clist_set_column_justification (GTK_CLIST(clist_uploads), 2, GTK_JUSTIFY_RIGHT);

	/* set columns resizable by user (which is not default in gtk2) */
	for (i=0; i<NO_OF_UPLOAD_COLS; i++)
	{
		gtk_clist_set_column_resizeable(GTK_CLIST(clist_uploads), i, TRUE);
	}

	/* create a scrollbar window to contain the clist later on*/
	ul_scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(ul_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(ul_scrollwin), clist_uploads);
	gtk_widget_show (clist_uploads);
	gtk_widget_show (ul_scrollwin);

	g_strfreev(tabtitles);

	return (ul_scrollwin);
}


static GtkWidget *
create_scrolled_ulstats_clist (void)
{
	gchar **tabtitles;
	gint    i;

	tabtitles = g_strsplit ( _(" upload | ul/size | filename | last time "), N_("|"), 0 );
	g_return_val_if_fail (tabtitles!=NULL, NULL);

	// create ulstats clist window and set its properties

	clist_ulstats = gtk_clist_new_with_titles (NO_OF_ULSTATS_COLS, tabtitles);
	gtk_signal_connect (GTK_OBJECT (clist_ulstats), N_("click-column"),
			    GTK_SIGNAL_FUNC(ulstats_clist_column_clicked), NULL);
	gtk_signal_connect (GTK_OBJECT (clist_ulstats), N_("button_press_event"),
				GTK_SIGNAL_FUNC(ulstats_button_pressed), GTK_OBJECT(clist_ulstats));
	gtk_signal_connect (GTK_OBJECT (clist_ulstats), N_("resize-column"),
				GTK_SIGNAL_FUNC(ulstats_column_resized), NULL);

	gtk_clist_set_auto_sort (GTK_CLIST(clist_ulstats),FALSE);
	gtk_clist_set_compare_func (GTK_CLIST(clist_ulstats), ulstats_clist_compare_func);
	gtk_clist_sort (GTK_CLIST(clist_ulstats));

	gtk_clist_set_column_justification (GTK_CLIST(clist_ulstats), 0, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification (GTK_CLIST(clist_ulstats), 1, GTK_JUSTIFY_RIGHT);

	/* set columns resizable by user (which is not default in gtk2) */
	for (i=0; i<NO_OF_ULSTATS_COLS; i++)
	{
		gtk_clist_set_column_resizeable(GTK_CLIST(clist_ulstats), i, TRUE);
	}

	// always exactly one item selected (unless less is empty)
	gtk_clist_set_selection_mode(GTK_CLIST (clist_ulstats), GTK_SELECTION_SINGLE);

	// set default column widths
	set_clist_columns (GTK_CLIST(clist_ulstats),ULSTATS_COLS, NO_OF_ULSTATS_COLS);

	// create a scrollbar window to contain the clist
	ulstats_scrollwin = gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(ulstats_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER(ulstats_scrollwin), clist_ulstats);
	gtk_widget_show (clist_ulstats);
	gtk_widget_show (ulstats_scrollwin);

	g_strfreev(tabtitles);

	return (ulstats_scrollwin);
}


/* uploads_page_set_people_on_queue
 *
 *
 *
 */

void
uploads_page_set_people_on_queue (guint num)
{
	gchar *ltxt;

	g_return_if_fail (uploads_page_stats_label!=NULL);

	ltxt = g_strdup_printf (_(" Uploads - %u on queue "), num);
	g_return_if_fail (ltxt!=NULL);

	gtk_label_set_text (GTK_LABEL(uploads_page_stats_label), ltxt);
	g_free(ltxt);
}


/* uploads_page_create_stats_label
 *
 *
 *
 *
 */

static GtkWidget *
uploads_page_create_stats_label (void)
{
	GtkWidget	*hbox, *emptylabel;

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show(hbox);

	emptylabel = gtk_label_new(N_(" "));
	gtk_widget_show(emptylabel);

	uploads_page_stats_label = gtk_label_new (_(" Uploads "));
	gtk_label_set_justify (GTK_LABEL(uploads_page_stats_label), GTK_JUSTIFY_LEFT);
	gtk_widget_show(uploads_page_stats_label);

	gtk_box_pack_start (GTK_BOX(hbox), uploads_page_stats_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), emptylabel, TRUE, TRUE, 0);

	if (option_nogreen == 0)
		set_style_recursively (hbox, style_notebook_label);

	return hbox;
}


/* uploads_page_create
 *
 *
 *
 */

GtkWidget *
uploads_page_create (void)
{
	GtkWidget *vbox, *ulstatslabel;

	vbox = gtk_vbox_new(FALSE,0);
	gtk_widget_show(vbox);

	ulstatslabel = uploads_page_create_stats_label();
	gtk_box_pack_start (GTK_BOX(vbox), ulstatslabel, FALSE, FALSE, 0);

	/* create a paned window for the whole upload page */
	ulpanedwin2 = gtk_vpaned_new ();
	gtk_widget_show (ulpanedwin2);
	gtk_paned_add1 (GTK_PANED(ulpanedwin2), create_scrolled_upload_clist());
	gtk_paned_add2 (GTK_PANED(ulpanedwin2), create_scrolled_ulstats_clist());

	/* in case someone's upgrading the GUI and using the old gui_options file */
	if (paned_win_position[UL_PANE2_POS]==0)
		paned_win_position[UL_PANE2_POS]=200;

	gtk_paned_set_position (GTK_PANED(ulpanedwin2),paned_win_position[UL_PANE2_POS]);

	gtk_box_pack_start (GTK_BOX(vbox), ulpanedwin2, TRUE, TRUE,0);

	return vbox;
}



void
uploadsclist_add_record (uploadRecord *ul)
{
	gchar *tab[NO_OF_UPLOAD_COLS];

	g_return_if_fail (ul!=NULL);
	g_return_if_fail (UPL_CLIST!=NULL);

	// make sure we don't have it already in there (mode=paranoid)
	// XXX - this has a negative impact on performance
	if (gtk_clist_find_row_from_data (UPL_CLIST, (gpointer)ul)<0)
	{
		gchar	*fn = g_quark_to_string (ul->namequark);
		gint	 row;

		tab[0] = (fn) ? fn : N_("");	// filename
		tab[1] = (ul->username) ? ul->username : N_("(???)");	// user nick name
		tab[2] = N_("");	// speed (will be set later when upload stats packets come in)
		// ...and add row to clist
		row = gtk_clist_append (UPL_CLIST, tab);
		gtk_clist_set_row_data (UPL_CLIST, row, (gpointer) ul);
		uploadsclist_resort();
	}
}

// uploadsclist_remove_record
//
// removes an uploads record from the uploads clist

void
uploadsclist_remove_record (uploadRecord *ul)
{
	g_return_if_fail (ul!=NULL);

	if (clist_uploads)
	{
		gint row = gtk_clist_find_row_from_data (UPL_CLIST, (gpointer)ul);
		if (row>=0) gtk_clist_remove (UPL_CLIST, row);
	}
}

void
uploadsclist_clear(void)
{
	if (clist_uploads)
		gtk_clist_clear (UPL_CLIST);
}


void
uploadsclist_update(void)
{
	g_return_if_fail (UPL_CLIST!=NULL);
	gtk_clist_freeze (UPL_CLIST);
	if (uploads)
	{
		PROCESS_GPTRARRAY( uploads, uploadRecord,
			if (item)
			{
				gint row = gtk_clist_find_row_from_data (UPL_CLIST, (gpointer)item);
				if (row>=0)
				{
					gchar speedstr[64];
					g_snprintf (speedstr, sizeof(speedstr), _("%3.1f kB/s"), item->speed);	// speed
					gtk_clist_set_text (UPL_CLIST, row, 2, speedstr);
				} else g_printerr (N_("uploads list <-> clist inconsistency in %s\n"), __FUNCTION__);
			}
		);
	}
	uploadsclist_resort();
	gtk_clist_thaw (UPL_CLIST);
}


void
uploadsclist_resort(void)
{
	gtk_clist_set_auto_sort (UPL_CLIST, FALSE);
	gtk_clist_set_sort_column (UPL_CLIST, uploadlist_sortcolumn);
	gtk_clist_set_sort_type (UPL_CLIST, uploadlist_sorttype);
	gtk_clist_set_compare_func (UPL_CLIST,uploads_clist_compare_func);
	gtk_clist_sort (UPL_CLIST);
}



// uploadstatsclist_update
//
// - update the values in the upload stats clist
// - add records that are not in the clist to the clist

void
uploadstatsclist_update (void)
{
	if (!clist_ulstats) return;
	if (!uploadstats) return;
	gtk_clist_freeze (GTK_CLIST(clist_ulstats));
	PROCESS_GPTRARRAY ( uploadstats, uploadstatsRecord,
		if (item)
		{
			const gchar	*fn = g_quark_to_string(item->namequark);
			gchar		 valstr[128];
			gint		 row = gtk_clist_find_row_from_data (GTK_CLIST(clist_ulstats), (gpointer)item);

			if (row<0)	// not in clist yet?
			{
				guint k;
				gchar *tab[NO_OF_ULSTATS_COLS];
				for (k=0; k<NO_OF_ULSTATS_COLS; k++) tab[k]=N_("");
				// ... and add row to list
				row = gtk_clist_append (GTK_CLIST (clist_ulstats), tab);
				gtk_clist_set_row_data (GTK_CLIST(clist_ulstats), row, (gpointer)item);
			}

			g_snprintf (valstr, sizeof(valstr), _("%.1fM "), item->uploaded/1024.0);
			gtk_clist_set_text (GTK_CLIST(clist_ulstats), row, 0, valstr);

			strncpy (valstr, N_("??? "), sizeof(valstr));
			if (item->size>0)
			{
				gfloat ratio = (item->uploaded*1024.0/((gfloat)item->size));
				if (ratio<10000.0)
					g_snprintf (valstr, sizeof(valstr), N_("%.1f "), ratio);
			}
			gtk_clist_set_text (GTK_CLIST(clist_ulstats), row, 1, valstr);
			gtk_clist_set_text (GTK_CLIST(clist_ulstats), row, 2, (fn) ? fn : N_("???"));

			if (1==1)			// 3 - last time uploaded
			{
				struct tm		*timethen = localtime (&item->last_uploaded);
				time_t			 timediff = time(NULL) - item->last_uploaded;
				static gchar	**months = NULL;

				if (!months)
				{
					months = g_strsplit ( _("Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec"), N_("|"), 0);
				}

				/* 'now' ? */
				if (timediff == 0)
				{
					g_snprintf (valstr, sizeof(valstr), _("%s %02d  %02d.%02dh  (%s)"), months[timethen->tm_mon], timethen->tm_mday,
						timethen->tm_hour, timethen->tm_min, seconds_into_human_time(timediff));
				} else {
					g_snprintf (valstr, sizeof(valstr), _("%s %02d  %02d.%02dh  (%s ago)"), months[timethen->tm_mon], timethen->tm_mday,
						timethen->tm_hour, timethen->tm_min, seconds_into_human_time(timediff));
				}
			}
			gtk_clist_set_text (GTK_CLIST(clist_ulstats), row, 3, valstr);
		}
	);
	uploadstatsclist_resort();
	gtk_clist_thaw (GTK_CLIST(clist_ulstats));
}


void
uploadstatsclist_resort (void)
{
	if (!clist_ulstats) return;
	gtk_clist_set_auto_sort (GTK_CLIST(clist_ulstats), FALSE);
	gtk_clist_set_sort_column (GTK_CLIST(clist_ulstats), ulstatslist_sortcolumn);
	gtk_clist_set_sort_type (GTK_CLIST(clist_ulstats), ulstatslist_sorttype);
	gtk_clist_set_compare_func (GTK_CLIST(clist_ulstats),ulstats_clist_compare_func);
	gtk_clist_sort (GTK_CLIST(clist_ulstats));
}


// uploadstatsclist_remove_record
//
// removes an upload stats record from the upload stats clist

void
uploadstatsclist_remove_record (uploadstatsRecord *ulstatsr)
{
	g_return_if_fail (ulstatsr!=NULL);
	if (clist_ulstats)
	{
		gint row = gtk_clist_find_row_from_data (GTK_CLIST(clist_ulstats), (gpointer)ulstatsr);
		if (row>=0)
		{
			gtk_clist_remove (GTK_CLIST(clist_ulstats), row);
		}
		else g_printerr (_("%s: record to remove from clist is not in clist?! (bug?)\n"), __FUNCTION__);
	}
}

